@@ -1,3 +1,4 @@
+#!/usr/bin/env python
 """
 Compute the cross spectral density of two signals
 """
@@ -27,7 +28,7 @@ xlabel('time')
 ylabel('s1 and s2')
 
 subplot(212)
-cxy, f = cohere(s1, s2, 256, 1/dt)
+cxy, f = csd(s1, s2, 256, 1/dt)
 show()
 
 
