@@ -1,4 +1,10 @@
+#!/usr/bin/env python
+"""
+See pcolor_demo2 for an alternative way of generating pcolor plots
+using imshow that is likely faster for large grids
+"""
 from __future__ import division
+from matplotlib.patches import Patch
 from matplotlib.matlab import *
 
 def func3(x,y):
@@ -14,8 +20,7 @@ X,Y = meshgrid(x, y)
 
 Z = func3(X, Y)
 pcolor(X, Y, Z, shading='flat')
-#pcolor(Z, shading='flat')
-axis('off')
+colorbar()
 #savefig('pcolor_demo')
 show()
 
