@@ -1,4 +1,5 @@
 #!/usr/bin/env python
+
 from matplotlib.matlab import *
 
 figure(1)
@@ -11,6 +12,6 @@ ylabel('voltage (mV)')
 title('About as simple as it gets, folks')
 grid(True)
 #axis([0,1,-1,1])
-savefig('simple_plot')
+#savefig('simple_plot')
 
 show()
