@@ -113,9 +113,9 @@ Most of the other commands are from the Numeric, MLab and FFT, with
 the exception of those in mlab.py provided by matplotlib.
 """
 
-__version__  = '0.54.1'
-__revision__ = '$Revision: 1.49 $'
-__date__     = '$Date: 2004/05/26 22:19:27 $'
+__version__  = '0.61.0'
+__revision__ = '$Revision: 1.72 $'
+__date__     = '$Date: 2004/08/09 14:40:09 $'
 
 import sys, os
 import distutils.sysconfig
@@ -151,6 +151,7 @@ else:
 
 
 def get_data_path():
+
     path = os.path.join(distutils.sysconfig.PREFIX, 'share', 'matplotlib')
     if os.path.isdir(path): return path
 
@@ -165,6 +166,15 @@ def get_data_path():
     if os.environ.has_key('MATPLOTLIBDATA'):
         path = os.environ['MATPLOTLIBDATA']
         if os.path.isdir(path): return path
+
+	
+    # CODE ADDED TO SUPPORT PY2EXE - you will need to copy
+    # C:\Python23\share\matplotlib into your dist dir.  See
+    # http://starship.python.net/crew/theller/moin.cgi/MatPlotLib
+    # for more info
+    if sys.platform=='win32' and sys.frozen:
+        path = os.path.join(os.path.split(sys.path[0])[0], 'matplotlibdata')
+        return path
     raise RuntimeError('Could not find the matplotlib data files')
 
 
@@ -187,6 +197,12 @@ def validate_float(s):
     try: return float(s)
     except ValueError:
         raise ValueError('Could not convert "%s" to float' % s)
+
+def validate_int(s):
+    'convert s to float or raise'
+    try: return int(s)
+    except ValueError:
+        raise ValueError('Could not convert "%s" to int' % s)
         
 def validate_numerix(s):
     'return "Numeric" or "Numarray" or raise'
@@ -196,6 +212,17 @@ def validate_numerix(s):
     else:
         raise ValueError('Numerix must be Numeric or numarray')
 
+def validate_toolbar(s):
+    """
+    return toolbar string 'None', 'classic', 'toolbar2'
+    """
+    s = s.lower()
+    if s=='none': return 'None'
+    elif s=='classic': return s
+    elif s=='toolbar2': return s    
+    else:
+        raise ValueError('toolbar must be None | classic | toolbar2')
+
 class validate_nseq_float:
     def __init__(self, n):
         self.n = n
@@ -210,6 +237,7 @@ class validate_nseq_float:
 
 def validate_color(s):
     'return a valid color arg'
+    if s.lower() == 'none': return 'None'
     if len(s)==1 and s.isalpha(): return s
     if s.find(',')>=0: # looks like an rgb
         # get rid of grouping symbols
@@ -260,6 +288,7 @@ def validate_fontsize(s):
 defaultParams = {
     'backend'           : ['GTK', str],
     'numerix'           : ['Numeric', validate_numerix],
+    'toolbar'           : ['classic', validate_toolbar],
     'datapath'          : [get_data_path(), validate_path_exists],
     'interactive'       : [False, validate_bool],
 
@@ -267,10 +296,21 @@ defaultParams = {
     'lines.linewidth'   : [0.5, validate_float],     # line width in points
     'lines.linestyle'   : ['-', str],                # solid line
     'lines.color'       : ['b', validate_color],     # blue
+    'lines.marker'       : ['None', str],     # black
+    'lines.markeredgecolor'       : ['k', validate_color],     # black
+    'lines.markerfacecolor'       : ['b', validate_color],     # blue
+    'lines.markeredgewidth'       : [0.5, validate_float],     
     'lines.markersize'  : [6, validate_float],       # markersize, in points
     'lines.antialiased' : [True, validate_bool],     # antialised (no jaggies)
     'lines.data_clipping' : [False, validate_bool],  # clip data
 
+    # patch props    
+    'patch.linewidth'   : [0.5, validate_float], # line width in points
+    'patch.edgecolor'   : ['k', validate_color], # black
+    'patch.facecolor'   : ['b', validate_color], # blue
+    'patch.antialiased' : [True, validate_bool], # antialised (no jaggies)
+
+
     # font props
     'font.family'       : ['serif', str],            # used by text object
     'font.style'        : ['normal', str],           #
@@ -292,6 +332,13 @@ defaultParams = {
     'text.fontweight'   : ['normal', str],
     'text.fontsize'     : ['medium', validate_fontsize],
 
+
+    'image.aspect' : ['free', str],  # free| preserve
+    'image.interpolation'  : ['bilinear', str], 
+    'image.cmap'   : ['gray', str],        # one of gray, jet
+    'image.lut'    : [256, validate_int],  # lookup table
+    'image.origin'    : ['upper', str],  # lookup table    
+
     # axes props
     'axes.hold'         : [True, validate_bool],    # background color; white    
     'axes.facecolor'    : ['w', validate_color],    # background color; white
@@ -314,6 +361,7 @@ defaultParams = {
     'grid.linestyle'   :   [':', str],       # dotted
     'grid.linewidth'   :   ['0.5', validate_float],     # in points            
 
+
     # figure props
     # figure size in inches: width by height
     'figure.figsize'    : [ (8,6), validate_nseq_float(2)], 
@@ -331,7 +379,21 @@ defaultParams = {
 
 
 def matplotlib_fname():
-    'Return the path to the rc file'
+    """
+    Return the path to the rc file
+
+    Search order:
+
+     * current working dir
+     * environ var MATPLOTLIBRC
+     * HOME/.matplotlibrc
+     * MATPLOTLIBDATA/.matplotlibrc
+     
+    """
+
+    fname = os.path.join( os.getcwd(), '.matplotlibrc')
+    if os.path.exists(fname): return fname
+
     if os.environ.has_key('MATPLOTLIBRC'):
         path =  os.environ['MATPLOTLIBRC']
         if os.path.exists(path):
@@ -411,6 +473,94 @@ def rc_params():
 # this is the instance used by the matplotlib classes
 rcParams = rc_params() 
 
+rcParamsDefault = dict(rcParams.items()) # a copy
+
+def rc(group, **kwargs):
+    """
+    Set the current rc params.  Group is the grouping for the rc, eg
+    for lines.linewidth the group is 'lines', for axes.facecolor, the
+    group is 'axes', and so on.  kwargs is a list of attribute
+    name/value pairs, eg
+
+      rc('lines', linewidth=2, color='r')
+
+    sets the current rc params and is equivalent to
+    
+      rcParams['lines.linewidth'] = 2
+      rcParams['lines.color'] = 'r'
+
+    The following aliases are available to save typing for interactive
+    users
+        'lw'  : 'linewidth'
+        'ls'  : 'linestyle'        
+        'c'   : 'color'
+        'fc'  : 'facecolor'
+        'ec'  : 'edgecolor'
+        'mfc' : 'markerfacecolor'
+        'mec' : 'markeredgecolor'
+        'mew' : 'markeredgewidth'
+        'aa'  : 'antialiased'            
+        'l'   : 'lines'
+        'a'   : 'axes'
+        'f'   : 'figure'
+        'p'   : 'patches'
+        'g'   : 'grid'
+
+    Thus you could abbreviate the above rc command as
+
+          rc('l', lw=2, c='r')
+
+
+    Note you can use python's kwargs dictionary facility to store
+    dictionaries of default parameters.  Eg, you can customize the
+    font rc as follows
+
+      font = {'family' : 'monospace',
+              'weight' : 'bold',
+              'size'   : 'larger',
+             }
+
+      rc('font', **font)  # pass in the font dict as kwargs
+
+    This enables you to easily switch between several configurations.
+    Use rcdefaults to restore the default rc params after changes.
+    """
+
+    aliases = {
+        'lw'  : 'linewidth',
+        'ls'  : 'linestyle',
+        'c'   : 'color',                        
+        'fc'  : 'facecolor',
+        'ec'  : 'edgecolor',
+        'mfc' : 'markerfacecolor',
+        'mec' : 'markeredgecolor',
+        'mew' : 'markeredgewidth',
+        'aa'  : 'antialiased',                        
+        'l'   : 'lines',
+        'a'   : 'axes',
+        'f'   : 'figure',
+        'p'   : 'patches',
+        'g'   : 'grid',        
+        }
+    
+    for k,v in kwargs.items():
+        name = aliases.get(k) or k
+        key = '%s.%s' % (group, name)
+        if not rcParams.has_key(key):
+            raise KeyError('Unrecognized key "%s" for group "%s" and name "%s"' %
+                           (key, group, name))
+        
+        rcParams[key] = v
+
+
+def rcdefaults():
+    """
+    Restore the default rc params - the ones that were created at
+    matplotlib load time
+    """
+    rcParams.update(rcParamsDefault)
+
+
 
 # Now allow command line to override
 
@@ -423,9 +573,9 @@ if hasattr(sys,'argv'):         # mod_python doesn't have argv attr
             rcParams['backend'] = s[2:].strip()
             break
 
-_knownBackends = {'PS':1, 'GTK':1, 'GTKGD':1, 'Template':1, 'GD':1,
+_knownBackends = {'PS':1, 'GTK':1, 'Template':1, 'GD':1,
                   'WX':1, 'Paint':1, 'Agg':1, 'GTKAgg':1, 'SVG':1,
-                  'TkAgg':1, 'WXAgg':1,}
+                  'TkAgg':1, 'WXAgg':1, 'FltkAgg':1,}
 
 def use(arg):
     """
