@@ -2,7 +2,7 @@
 Classes for the ticks and x and y axis
 """
 from __future__ import division
-import sys, math, re
+import sys, math, re, copy
 from numerix import arange, array, asarray, ones, zeros, \
      nonzero, take, Float, log10, logical_and
 
@@ -43,7 +43,7 @@ class Tick(Artist):
     """
     def __init__(self, axes, loc, label,
                  size   = None,  # points
-                 gridOn = rcParams['axes.grid'],
+                 gridOn = None, # defaults to axes.grid
                  tick1On = True,
                  tick2On = True,
                  label1On = True,
@@ -56,6 +56,9 @@ class Tick(Artist):
         size is the tick size in relative, axes coords
         """
         Artist.__init__(self)
+
+        if gridOn is None: gridOn = rcParams['axes.grid']
+
         self.set_figure(axes.figure)
         self.axes = axes
 
@@ -72,14 +75,15 @@ class Tick(Artist):
         self._pad = Value(pad)
         self._padPixels = self.figure.dpi*self._pad*Value(1/72.0)
 
+
         self.tick1line = self._get_tick1line(loc)
         self.tick2line = self._get_tick2line(loc)
         self.gridline = self._get_gridline(loc)        
+
         self.label1 = self._get_text1(loc)
         self.label = self.label1  # legacy name
-
         self.label2 = self._get_text2(loc)
-        
+
         self.gridOn = gridOn
         self.tick1On = tick1On
         self.tick2On = tick2On
@@ -120,6 +124,8 @@ class Tick(Artist):
         return self._loc
     
     def draw(self, renderer):
+
+        renderer.open_group(self.__name__)
         midPoint = self.get_view_interval().contains_open( self.get_loc() )
 
         if midPoint and self.tick1On: self.tick1line.draw(renderer)
@@ -129,8 +135,9 @@ class Tick(Artist):
 
         if self.label1On: self.label1.draw(renderer)
         if self.label2On: self.label2.draw(renderer)
-        #print 'axis draw', self.label1._x, self.label1._text, self.label1._transform.get_funcx()
-    
+
+        renderer.close_group(self.__name__)
+
     def set_xy(self, loc):
         'Set the location of tick in data coords with scalar loc'
         raise NotImplementedError('Derived must override')
@@ -163,18 +170,21 @@ class XTick(Tick):
     Contains all the Artists needed to make an x tick - the tick line,
     the label text and the grid line
     """
-    
+    __name__ = 'xtick'
     def _get_text1(self, loc):
         'Get the default Text instance'
         # the y loc is 3 points below the min of y axis
         # get the affine as an a,b,c,d,tx,ty list
         # x in data coords, y in axes coords
-        t =  Text(x=loc, y=0,
+        t =  Text(
+            x=loc, y=0,
             fontproperties=FontProperties(size=rcParams['tick.labelsize']),
             color=rcParams['tick.color'],
             verticalalignment='top',
             horizontalalignment='center',
             )
+
+
         trans = blend_xy_sep_transform( self.axes.transData,
                                         self.axes.transAxes)
         #offset the text downward with a post transformation
@@ -182,12 +192,13 @@ class XTick(Tick):
             Value(0), Value(-1)*self._padPixels)
         trans.set_offset( (0,0), transOffset)
         t.set_transform( trans)
-        
+
         self._set_artist_props(t)
         return t
 
 
     def _get_text2(self, loc):
+
         'Get the default Text 2 instance'
         # x in data coords, y in axes coords
         t =  Text(x=loc, y=1,
@@ -196,6 +207,7 @@ class XTick(Tick):
             verticalalignment='bottom',
             horizontalalignment='center',
             )
+
         trans = blend_xy_sep_transform( self.axes.transData,
                                         self.axes.transAxes)
         # offset the text upward with a post transformation
@@ -216,12 +228,8 @@ class XTick(Tick):
                     markersize=self._size,
                     )
 
-        #l.set_data_clipping(False)
         l.set_transform( blend_xy_sep_transform( self.axes.transData,
                                                  self.axes.transAxes) )
-
-
-        #l.set_clip_box(self.axes.bbox)
         self._set_artist_props(l)
         return l
 
@@ -235,10 +243,8 @@ class XTick(Tick):
                        markersize=self._size,
                        )
 
-        l.set_data_clipping(False)
         l.set_transform( blend_xy_sep_transform( self.axes.transData,
                                                  self.axes.transAxes) )
-
         self._set_artist_props(l)
         return l
 
@@ -248,11 +254,12 @@ class XTick(Tick):
         l = Line2D( xdata=(loc, loc), ydata=(0, 1),
                     color=rcParams['grid.color'],
                     linestyle=rcParams['grid.linestyle'],
+                    linewidth=rcParams['grid.linewidth'],                    
                     antialiased=False, 
                     )
         l.set_transform( blend_xy_sep_transform( self.axes.transData,
                                                  self.axes.transAxes) )
-        #l.set_data_clipping(False)
+        l.set_clip_box(self.axes.bbox)
         self._set_artist_props(l)
 
         return l
@@ -283,7 +290,8 @@ class YTick(Tick):
     Contains all the Artists needed to make a Y tick - the tick line,
     the label text and the grid line
     """
-
+    __name__ = 'ytick'
+    
     # how far from the y axis line the right of the ticklabel are
     def _get_text1(self, loc):
         'Get the default Text instance'
@@ -298,10 +306,12 @@ class YTick(Tick):
         trans = blend_xy_sep_transform( self.axes.transAxes,
                                         self.axes.transData)
         # offset the text leftward with a post transformation
+
         transOffset = translation_transform(
             Value(-1)*self._padPixels, Value(0))
         trans.set_offset( (0,0), transOffset)
         t.set_transform( trans )
+        #t.set_transform( self.axes.transData )
         self._set_artist_props(t)
         return t
     
@@ -317,6 +327,7 @@ class YTick(Tick):
         trans = blend_xy_sep_transform( self.axes.transAxes,
                                         self.axes.transData)
         # offset the text rightward with a post transformation
+
         transOffset = translation_transform(
             self._padPixels, Value(0))
         trans.set_offset( (0,0), transOffset)
@@ -335,8 +346,6 @@ class YTick(Tick):
                        )
         l.set_transform( blend_xy_sep_transform( self.axes.transAxes,
                                                  self.axes.transData) )
-        
-        l.set_data_clipping(False)
         self._set_artist_props(l)
         return l
     
@@ -351,8 +360,6 @@ class YTick(Tick):
 
         l.set_transform( blend_xy_sep_transform( self.axes.transAxes,
                                                  self.axes.transData) )
-
-        l.set_data_clipping(False)
         self._set_artist_props(l)
         return l
     
@@ -362,13 +369,14 @@ class YTick(Tick):
         l = Line2D( xdata=(0,1), ydata=(loc,loc), 
                     color=rcParams['grid.color'],
                     linestyle=rcParams['grid.linestyle'],
+                    linewidth=rcParams['grid.linewidth'],
                     antialiased=False, 
                     )
 
         l.set_transform( blend_xy_sep_transform( self.axes.transAxes,
                                                  self.axes.transData) )
-
-        l.set_data_clipping(False)
+        l.set_clip_box(self.axes.bbox)
+        
         self._set_artist_props(l)
         return l
 
@@ -447,15 +455,19 @@ class Axis(Artist):
     def draw(self, renderer, *args, **kwargs):
         'Draw the axis lines, grid lines, tick lines and labels'
 
+        renderer.open_group(__name__)
         ticklabelBoxes = []
-
+        
         majorTicks = self.get_major_ticks()
         majorLocs = self._majorLocator()
         majorLabels = [self._majorFormatter(val, i) for i, val in enumerate(majorLocs)]
 
         seen = {}
 
+        interval = self.get_view_interval()
         for tick, loc, label in zip(majorTicks, majorLocs, majorLabels):
+            if not interval.contains(loc): continue
+            #print loc, label, interval.get_bounds()
             seen[loc] = 1
             tick.update_position(loc)
             tick.set_label1(label)
@@ -470,6 +482,7 @@ class Axis(Artist):
         minorLabels = [self._minorFormatter(val, i) for i, val in enumerate(minorLocs)]
 
         for tick, loc, label in zip(minorTicks, minorLocs, minorLabels):
+            if not interval.contains(loc): continue
             if seen.has_key(loc): continue
             tick.update_position(loc)
             tick.set_label(label)
@@ -496,6 +509,8 @@ class Axis(Artist):
                 bbox_artist(label, renderer)
             bbox_artist(self._label, renderer)
 
+        renderer.close_group(__name__)
+
     def _get_label(self):
         raise NotImplementedError('Derived must override')
 
@@ -530,6 +545,7 @@ class Axis(Artist):
 
     def _copy_tick_props(self, src, dest):
         'Copy the props from src tick to dest tick'
+
         dest.label1.copy_properties(src.label1)
         dest.label2.copy_properties(src.label2)
 
@@ -545,15 +561,22 @@ class Axis(Artist):
     def get_major_locator(self):
         'Get the locator of the major ticker'
         return self._majorLocator 
+
+    def get_minor_locator(self):
+        'Get the locator of the minor ticker'
+        return self._minorLocator 
         
     def get_major_ticks(self):
         'get the tick instances; grow as necessary'
+
         numticks = len(self._majorLocator())
+
         if len(self._majorTicks)<numticks:
             # update the new tick label properties from the old
             protoTick = self._majorTicks[0]
             for i in range(numticks-len(self._majorTicks)):
                 tick = self._get_tick(major=True)
+                #tick = protoTick
                 if self._gridOnMajor: tick.gridOn = True
                 self._copy_tick_props(protoTick, tick)
                 self._majorTicks.append(tick)
@@ -582,15 +605,16 @@ class Axis(Artist):
         'major' | 'minor' to set the grid for major or minor ticks
         """
         if which.lower().find('minor')>=0:
-            self._gridOnMinor = b
+            if b is None: self._gridOnMinor = self._gridOnMinor
+            else: self._gridOnMinor = b
             for tick in self._minorTicks:  # don't use get_ticks here!
                 tick.gridOn = self._gridOnMinor
         else:
-            self._gridOnMajor = b
+            if b is None: self._gridOnMajor = self._gridOnMajor
+            else: self._gridOnMajor = b
             for tick in self._majorTicks:  # don't use get_ticks here!
                 tick.gridOn = self._gridOnMajor
                 
-
     
     def set_major_formatter(self, formatter):
         'Set the formatter of the major ticker'
@@ -663,7 +687,6 @@ class XAxis(Axis):
     __name__ = 'xaxis'
 
     def _get_tick(self, major):
-
         return XTick(self.axes, 0, '', major=major)
 
     def _get_label(self):
