@@ -22,11 +22,10 @@ elif backend=='Agg':
     from backend_agg import new_figure_manager
 elif backend=='SVG':
     from backend_svg import new_figure_manager
-elif backend=='GTKGD':
-    from backend_gtkgd import \
-         error_msg, draw_if_interactive, show, new_figure_manager
 elif backend=='GTKAgg':
     from backend_gtkagg import error_msg, draw_if_interactive, show, new_figure_manager
+elif backend=='FltkAgg':
+    from backend_fltkagg import error_msg, draw_if_interactive, show, new_figure_manager
 elif backend=='TkAgg':
     from backend_tkagg import error_msg, draw_if_interactive, show, new_figure_manager
 elif backend=='PS':
