@@ -93,8 +93,7 @@ class RendererAgg(RendererBase):
     The renderer handles all the drawing primitives using a graphics
     context instance that controls the colors/styles
     """
-
-
+    
     debug=1
     def __init__(self, width, height, dpi):
         if DEBUG: print 'RendererAgg.__init__'
@@ -102,11 +101,11 @@ class RendererAgg(RendererBase):
         self.width = width
         self.height = height
         self._renderer = _RendererAgg(int(width), int(height), dpi.get(),
-                                      debug=DEBUG)
-
+                                    debug=DEBUG)
         self.draw_polygon = self._renderer.draw_polygon
         self.draw_rectangle = self._renderer.draw_rectangle
         self.draw_lines = self._renderer.draw_lines
+        self.draw_image = self._renderer.draw_image
         self.draw_line_collection = self._renderer.draw_line_collection
         self.draw_poly_collection = self._renderer.draw_poly_collection
         self.draw_regpoly_collection = self._renderer.draw_regpoly_collection
@@ -127,13 +126,14 @@ class RendererAgg(RendererBase):
             gcEdge, rgbFace, x, y, width/2, height/2)  # ellipse takes radius
         
 
-    def draw_image(self, x, y, im):
+    def _draw_image(self, x, y, im):
         """
         Draw the Image instance into the current axes; x, y is the
         upper left hand corner of the image
         """
         if DEBUG: print 'RendererAgg.draw_image'                
-        self._renderer.draw_image(int(x), int(self.height-y), im)
+        #self._renderer.draw_image(int(x), int(self.height-y), im)
+        self._renderer.draw_image(int(x), int(y), im)        
 
     def draw_line(self, gc, x1, y1, x2, y2):
         """
@@ -162,19 +162,20 @@ class RendererAgg(RendererBase):
         if DEBUG: print 'RendererAgg.draw_mathtext'
         size = prop.get_size_in_points()
         width, height, fonts = math_parse_s_ft2font(
-            s, self.dpi.get(), size)
+            s, self.dpi.get(), size, angle)
         
-        rgb = gc.get_rgb()
-        rgba = (rgb[0], rgb[1], rgb[2], gc.get_alpha())        
-
-
+        if angle == 90:
+            width, height = height, width
         for font in fonts:
-            self._renderer.draw_text( font, int(x), int(y)-height, rgba)
-
+            if angle == 90:             
+                font.horiz_image_to_vert_image() # <-- Rotate
+                self._renderer.draw_text( font, int(x)-width, int(y)-height, gc)
+            else:
+                self._renderer.draw_text( font, int(x), int(y)-height, gc)                
         if 0:
             self._renderer.draw_rectangle(gc, None,
-                                          int(x+ox),
-                                          int(y+oy),
+                                          int(x),
+                                          self.height-int(y),
                                           width, height)
 
         
@@ -189,16 +190,10 @@ class RendererAgg(RendererBase):
 
         font = self._get_agg_font(prop)
         if font is None: return None
-        rgb = gc.get_rgb()
-        rgba = [rgb[0], rgb[1], rgb[2], gc.get_alpha()]
-
         font.set_text(s, angle)
         font.draw_glyphs_to_bitmap()
-        
-        
-        
-        self._renderer.draw_text(
-            font, int(x), int(y), rgba)
+                
+        self._renderer.draw_text(font, int(x), int(y), gc)
 
         
     def get_text_width_height(self, s, prop, ismath):
@@ -234,13 +229,12 @@ class RendererAgg(RendererBase):
         
         if font is None:
             fname = fontManager.findfont(prop)
-            try: font = FT2Font(fname)
+            try: font = FT2Font(str(fname))
             except RuntimeError, msg:
                 print >> sys.stderr, 'Could not load filename for text', \
                       fname
                 return None
             else:
-
                 _fontd[key] = font
 
         font.clear()
@@ -261,6 +255,14 @@ class RendererAgg(RendererBase):
     def tostring_rgb(self):
         if DEBUG: print 'RendererAgg.tostring_rgb'                
         return self._renderer.tostring_rgb()
+        
+    def buffer_rgba(self):
+        if DEBUG: print 'RendererAgg.buffer_rgba'                
+        return self._renderer.buffer_rgba()
+        
+    def clear(self):
+        self._renderer.clear()
+        
 
     
 def new_figure_manager(num, *args, **kwargs):
@@ -284,6 +286,7 @@ class FigureCanvasAgg(FigureCanvasBase):
 
       figure - A Figure instance
     """
+    
 
     def draw(self):
         """
@@ -292,12 +295,23 @@ class FigureCanvasAgg(FigureCanvasBase):
         if DEBUG: print 'FigureCanvasAgg.draw'                        
 
         l,b,w,h = self.figure.bbox.get_bounds()
-        self.renderer = RendererAgg(w, h, self.figure.dpi)
+        key = w, h, self.figure.dpi.get()
+        try: self._lastKey
+        except AttributeError: need_new_renderer = True
+        else:  need_new_renderer = (self._lastKey != key)
+
+        if need_new_renderer:
+            self.renderer = RendererAgg(w, h, self.figure.dpi)
+            self._lastKey = key
         self.figure.draw(self.renderer)
 
     def tostring_rgb(self):
         if DEBUG: print 'FigureCanvasAgg.tostring_rgb'                        
         return self.renderer.tostring_rgb()
+        
+    def buffer_rgba(self):
+        if DEBUG: print 'FigureCanvasAgg.buffer_rgba'                        
+        return self.renderer.buffer_rgba()
     
     def print_figure(self, filename, dpi=150,
                      facecolor='w', edgecolor='w',
@@ -314,7 +328,7 @@ class FigureCanvasAgg(FigureCanvasBase):
 
         """
         if DEBUG: print 'FigureCanvasAgg.print_figure'
-        
+
         # store the orig figure dpi, color and size information so we
         # can restore them later.  For image creation alone, this is
         # not important since after the print the figure is done.  But
@@ -349,6 +363,12 @@ class FigureCanvasAgg(FigureCanvasBase):
         elif ext.find('png')>=0:
             # agg doesn't handle unicode yet
             self.renderer._renderer.write_png(str(filename))
+            #pass
+        elif ext.find('svg')>=0:
+	    from backend_svg import FigureCanvasSVG
+	    svg = self.switch_backends(FigureCanvasSVG)
+            svg.figure.dpi.set(72)
+	    svg.print_figure(filename, 72, facecolor, edgecolor, orientation)
         elif ext.find('ps')>=0 or ext.find('ep')>=0:
             from backend_ps import FigureCanvasPS # lazy import
             ps = self.switch_backends(FigureCanvasPS)
