@@ -26,7 +26,8 @@ def new_figure_manager(num, *args, **kwargs):
     canvas = FigureCanvasGTKAgg(thisFig)
     return FigureManagerGTK(canvas, num)
 
-class FigureCanvasGTKAgg(FigureCanvasGTK):
+class FigureCanvasGTKAgg(FigureCanvasGTK, FigureCanvasAgg):
+
 
     def draw(self):
         """
@@ -34,27 +35,35 @@ class FigureCanvasGTKAgg(FigureCanvasGTK):
         gtk.gdk.drawable.
 
         """
-        if DEBUG: print 'FigureCanvasGTKAgg.draw'        
-        drawable = self.window
-        if drawable is None: return
-
-        agg = self.switch_backends(FigureCanvasAgg)
-        agg.draw()
-        agg_to_gtk_drawable(drawable, agg.renderer._renderer)
-        self.renderer = agg.renderer
+        if DEBUG: print 'FigureCanvasGTKAgg.draw'  
+            
+        FigureCanvasAgg.draw(self)
+        if self.window is None: 
+            return
+        else: 
+            self.blit()
+        
 
+    def blit(self):
+        if self.window is None: return        
+        agg_to_gtk_drawable(self.window, self.renderer._renderer)
 
     def print_figure(self, filename, dpi=150,
                      facecolor='w', edgecolor='w',
                      orientation='portrait'):
         if DEBUG: print 'FigureCanvasGTKAgg.print_figure'
+        # delete the renderer to prevent improper blitting after print
+
         agg = self.switch_backends(FigureCanvasAgg)
         agg.print_figure(filename, dpi, facecolor, edgecolor, orientation)
 
+
+
     def configure_event(self, widget, event=None):
         if DEBUG: print 'FigureCanvasGTKAgg.configure_event'
         if widget.window is None: return 
-
+        try: del self.renderer
+        except AttributeError: pass
         w,h = widget.window.get_size()
         if w==1 or h==1: return # empty fig
 
@@ -72,7 +81,8 @@ class FigureCanvasGTKAgg(FigureCanvasGTK):
         if widget.window is None: return 
 
         def callback(w):
-            self.draw()
+            if hasattr(self, 'renderer'):  self.blit()
+            else: self.draw()
             self._idleID=0
             return gtk.FALSE
 
