@@ -1,6 +1,6 @@
 # A postscript backend
 
-# postscript for mathmeatical drawing:
+# postscript for mathematical drawing:
 # http://www.math.ubc.ca/people/faculty/cass/graphics/text/www/
 
 from __future__ import division
@@ -17,10 +17,11 @@ from matplotlib.cbook import iterable, is_string_like, flatten, enumerate,\
 from matplotlib.figure import Figure
 from matplotlib.font_manager import fontManager
 from matplotlib.ft2font import FT2Font
+from matplotlib.mathtext import math_parse_s_ps, bakoma_fonts
 from matplotlib._matlab_helpers import Gcf
 from matplotlib.text import Text
 
-from matplotlib import rcParams
+from matplotlib import rcParams, get_data_path
 
 from matplotlib.numerix import fromstring, UInt8, Float32
 import binascii
@@ -42,15 +43,19 @@ def _int_or_float(val, fmt='%1.3f'):
     if val==ival: return str(ival)
     else: return fmt%val
 
-fontd = {}
+_fontd = {}
+_type42 = []
+
+DEBUG = 0
+
 class RendererPS(RendererBase):
 
     def __init__(self, width, height, pswriter):
+
         self.width = width
         self.height = height
         self._pswriter = pswriter
 
-
     def get_canvas_width_height(self):
         'return the canvas width and height in display coords'
         return self.width, self.height
@@ -60,24 +65,39 @@ class RendererPS(RendererBase):
         get the width and height in display coords of the string s
         with FontPropertry prop
         """
+        if ismath:
+            width, height, pswriter = math_parse_s_ps(
+                s, 72, prop.get_size_in_points())
+            return width, height
         font = self._get_font(prop)
-        l,b,w,h = font.get_str_bbox(s)
+        font.set_text(s, 0.0)
 
-        fontsize = prop.get_size_in_points()
-        w *= 0.001*fontsize
-        h *= 0.001*fontsize
+        w, h = font.get_width_height()
+        w /= 64.0  # convert from subpixels
+        h /= 64.0
         return w, h
     
     def flipy(self):
         return False
 
-
     def _get_font(self, prop):
         key = hash(prop)
-        font = fontd.get(key)
+        font = _fontd.get(key)
         if font is None:
-            font = AFM(file(fontManager.findfont(prop, fontext='afm')))
-            fontd[key] = font
+            fname = fontManager.findfont(prop)
+            try:
+                font = FT2Font(str(fname))
+            except RuntimeError, msg:
+                print >> sys.stderr, 'Could not load filename for text',fname
+                return None
+            else:
+                _fontd[key] = font
+                if fname not in _type42:
+                    _type42.append(fname)
+        font.clear()
+        size = prop.get_size_in_points()
+        font.set_size(size, 72.0)
+
         return font
     
     def draw_postscript(self, ps):
@@ -88,7 +108,7 @@ class RendererPS(RendererBase):
         ps = 'newpath %s ellipse' % _nums_to_str(
             (x,y,0.5*width,0.5*height,angle1,angle2))
         self._draw_ps(ps, gc, rgbFace)
-        
+    
     def draw_line(self, gc, x1, y1, x2, y2):
         """
         Draw a single line from x1,y1 to x2,y2
@@ -120,18 +140,18 @@ class RendererPS(RendererBase):
         # todo: is there a better way to draw points in postscript?
         self.draw_line(gc, x, y, x+1, y+1)
 
-    def _rgba(self, im):
-        return im.as_str()
+    def _rgba(self, im, flipud):
+        return im.as_str(fliud)
     
-    def _rgb(self, im):
-        rgbat = im.as_str()
-        rgba = fromstring(rgbat[2], typecode=UInt8)
+    def _rgb(self, im, flipud):
+        rgbat = im.as_str(flipud)
+        rgba = fromstring(rgbat[2], UInt8)
         rgba.shape = (rgbat[0], rgbat[1], 4)
         rgb = rgba[:,:,:3]
         return rgbat[0], rgbat[1], rgb.tostring()
 
-    def _gray(self, im, rc=0.3, gc=0.59, bc=0.11):
-        rgbat = im.as_str()
+    def _gray(self, im, flipud, rc=0.3, gc=0.59, bc=0.11):
+        rgbat = im.as_str(flipud)
         rgba = fromstring(rgbat[2], UInt8)
         rgba.shape = (rgbat[0], rgbat[1], 4)
         r = rgba[:,:,0].astype(Float32)
@@ -149,33 +169,66 @@ class RendererPS(RendererBase):
             lines.append(s[i:limit])
         return lines
 
-    def draw_image(self, x, y, im):
+    def draw_image(self, x, y, im, origin, bbox):
+
+        flipud = origin=='lower'        
         if im.is_grayscale:
-            w, h, bits = self._gray(im)
+            h, w, bits = self._gray(im, flipud)
             imagecmd = "image"
         else:
-            w, h, bits = self._rgb(im)
+            h, w, bits = self._rgb(im, flipud)
             imagecmd = "false 3 colorimage"
         hexlines = '\n'.join(self._hex_lines(bits))
-        y -= h
+
         xscale, yscale = w, h
+
+        figh = self.height*72
+        #print 'values', origin, flipud, figh, h, y
+        if bbox is not None:
+            cliprect = bbox.get_bounds()
+            box = ['%1.5f'%val for val in cliprect]
+            clip =   '%s clipbox' % ' '.join(box)
+        else: clip = 'figure_clip'
+        if not flipud: y = figh-(y+h)
         if 0:
             print "RendererPS.draw_image:"
             print "x, y: ", x, y
             print "xscale, yscale:", xscale, yscale
-        ps = """
-        gsave
-        figure_clip
-        %(x)s %(y)s translate
-        %(xscale)s %(yscale)s scale
-        /DataString %(w)s string def
-        %(w)s %(h)s 8 [ %(w)s 0 0 -%(h)s 0 %(h)s ]
-        {
-           currentfile DataString readhexstring pop
-        } bind %(imagecmd)s
-        %(hexlines)s
-        grestore
-        """ % locals()
+        ps = """gsave
+%(clip)s
+%(x)s %(y)s translate
+%(xscale)s %(yscale)s scale
+/DataString %(w)s string def
+%(w)s %(h)s 8 [ %(w)s 0 0 -%(h)s 0 %(h)s ]
+{
+currentfile DataString readhexstring pop
+} bind %(imagecmd)s
+%(hexlines)s
+grestore
+""" % locals()
+        self.draw_postscript(ps)
+
+    def draw_mathtext(self, gc, x, y, s, prop, angle):
+        """
+        Draw the math text using matplotlib.mathtext
+        """
+        if DEBUG: print 'RendererPS.draw_mathtext'
+        fontsize = prop.get_size_in_points()
+        width, height, pswriter = math_parse_s_ps(s, 72, fontsize)
+        
+        if 0:
+            print 'RendererPS.draw_mathtext:'
+            print 'x, y: ', x, y
+            print 's: ', s
+            self.draw_rectangle(gc, (1, 1, 1), x, y, width, height)
+
+        thetext = pswriter.getvalue()
+        ps = """gsave
+%(x)f %(y)f translate
+%(angle)f rotate
+%(thetext)s
+grestore
+""" % locals()
         self.draw_postscript(ps)
 
     def draw_text(self, gc, x, y, s, prop, angle, ismath):
@@ -183,31 +236,29 @@ class RendererPS(RendererBase):
         draw a Text instance
         """
 
-
-        font = self._get_font(prop)
+        if ismath:
+            return self.draw_mathtext(gc, x, y, s, prop, angle)
         
-        l,b,w,h = font.get_str_bbox(s)
+        font = self._get_font(prop)
+        font.set_text(s, 0.0)
+        l, b = 0., 0.
+        w, h = font.get_width_height()
 
         fontsize = prop.get_size_in_points()
-        l *= 0.001*fontsize
-        b *= 0.001*fontsize
-        w *= 0.001*fontsize
-        h *= 0.001*fontsize
 
         if angle==90: l,b = -b, l # todo generalize for arb rotations
         pos = _nums_to_str((x-l, y-b))
         thetext = '(%s)' % s
-        fontname = font.get_fontname()
+        
+        fontname = font.get_sfnt()[(1,0,0,6)]
         fontsize = prop.get_size_in_points()
         rotate = '%1.1f rotate' % angle
         setcolor = '%1.3f %1.3f %1.3f setrgbcolor' % gc.get_rgb()
-        #h = 0
-        ps = """\
-gsave
+        ps = """gsave
 /%(fontname)s findfont
 %(fontsize)s scalefont
 setfont
-%(pos)s moveto
+%(pos)s rmoveto
 %(rotate)s
 %(thetext)s
 %(setcolor)s
@@ -218,8 +269,6 @@ grestore
         
     def get_ps(self):
         return self._pswriter.getvalue()
-
-
     
     def finish(self):
         self._pswriter.write('showpage\n')
@@ -233,13 +282,12 @@ grestore
         else: fill = ''
         gcprops = self._get_gc_props_ps(gc)
         clip = self._get_gc_clip_ps(gc)
-        s = """
-gsave
- %(clip)s
- %(ps)s
- %(gcprops)s
- gsr
- %(fill)s
+        s = """gsave
+%(clip)s
+%(ps)s
+%(gcprops)s
+gsr
+%(fill)s
 grestore
 """ % locals()
         self._pswriter.write(s)
@@ -261,7 +309,6 @@ grestore
         cint = {'butt':0, 'round':1, 'projecting':2}[gc.get_capstyle()]
         cap = '%d setlinecap' % cint
         offset, seq = gc.get_dashes()
-        #print offset, seq
         if seq is not None:
             seq = ' '.join(['%d'%val for val in  seq])
             dashes = '[%s] %d setdash' % (seq, offset)            
@@ -291,15 +338,93 @@ def show():
     for manager in Gcf.get_all_fig_managers():
         manager.figure.realize()
 
-
 def new_figure_manager(num, *args, **kwargs):
     thisFig = Figure(*args, **kwargs)
     canvas = FigureCanvasPS(thisFig)
     manager = FigureManagerPS(canvas, num)
     return manager
 
+def encodeTTFasPS(fontfile):
+    """
+    Encode a TrueType font file for embedding in a PS file.
+    """
+    fontfile = str(fontfile) # todo: handle unicode filenames
+    font = file(fontfile)
+    hexdata, data = '', font.read(65520)
+    while len(data):
+        hexdata += '<'+'\n'.join([binascii.b2a_hex(data[j:j+36]).upper() \
+                   for j in range(0, len(data), 36)])+'>\n'
+        data  = font.read(65520)
+    
+    hexdata = hexdata[:-2] + '00>'
+    font    = FT2Font(fontfile)
+    
+    headtab  = font.get_sfnt_table('head')
+    version  = '%d.%d' % headtab['version']
+    revision = '%d.%d' % headtab['fontRevision']
+
+    dictsize = 8
+    fontname = font.postscript_name
+    encoding = 'StandardEncoding'
+    fontbbox = '[%d %d %d %d]' % font.bbox
+    
+    posttab  = font.get_sfnt_table('post')
+    minmemory= posttab['minMemType42']
+    maxmemory= posttab['maxMemType42']
+
+    infosize = 7
+    sfnt     = font.get_sfnt()
+    notice   = sfnt[(1,0,0,0)]
+    family   = sfnt[(1,0,0,1)]
+    fullname = sfnt[(1,0,0,4)]
+    iversion = sfnt[(1,0,0,5)]
+    fixpitch = str(bool(posttab['isFixedPitch'])).lower()
+    ulinepos = posttab['underlinePosition']
+    ulinethk = posttab['underlineThickness']
+    italicang= '(%d.%d)' % posttab['italicAngle']
+
+    numglyphs = font.num_glyphs
+    glyphs = ''
+    for j in range(numglyphs):
+        glyphs += '/%s %d def' % (font.get_glyph_name(j), j)
+        if j != 0 and j%4 == 0:
+            glyphs += '\n'
+        else:
+            glyphs += ' '
+    
+    data = '%%!PS-TrueType-%(version)s-%(revision)s\n' % locals()
+    if maxmemory:
+        data += '%%%%VMusage: %(minmemory)d %(maxmemory)d' % locals()
+    data += """%(dictsize)d dict begin
+/FontName /%(fontname)s def
+/FontMatrix [1 0 0 1 0 0] def
+/FontType 42 def
+/Encoding %(encoding)s def
+/FontBBox %(fontbbox)s def
+/PaintType 0 def
+/FontInfo %(infosize)d dict dup begin
+/Notice (%(notice)s) def
+/FamilyName (%(family)s) def
+/FullName (%(fullname)s) def
+/version (%(iversion)s) def
+/isFixedPitch %(fixpitch)s def
+/UnderlinePosition %(ulinepos)s def
+/UnderlineThickness %(ulinethk)s def
+end readonly def
+/sfnts [
+%(hexdata)s
+] def
+/CharStrings %(numglyphs)d dict dup begin
+%(glyphs)s
+end readonly def
+FontName currentdict end definefont pop""" % locals()
+    return data
+
+
 class FigureCanvasPS(FigureCanvasBase):
 
+    basepath = get_data_path()
+
     def draw(self):
         pass
     
@@ -308,7 +433,8 @@ class FigureCanvasPS(FigureCanvasBase):
                      orientation='portrait'):
         'dpi is ignored for PS output, it depends on the output device'
         # ignore dpi for ps
-
+        self.figure.dpi.set(72)
+        
         # center the figure on the paper
         if orientation=='landscape':
             isLandscape = True
@@ -343,13 +469,11 @@ class FigureCanvasPS(FigureCanvasBase):
             ury = lly + h 
             bboxstr = '%%BoundingBox:    %d %d %d %d' % (llx, lly, urx, ury)
 
-            pstype = 'PS-Adobe-3.0 EPSF-3.0'
+            pstype = 'PS-Adobe-2.0 EPSF-2.0'
         else:
             pstype = 'PS'
             bboxstr = ''
 
-        self._pswriter.write(_psProlog % (pstype, bboxstr))
-        self._pswriter.write(_psDefs)
         if isLandscape:
             xo, yo = 72*defaultHeight - yo, xo
             rotation = 90
@@ -374,7 +498,7 @@ class FigureCanvasPS(FigureCanvasBase):
             print "translation xo, yo:", xo, yo
             print "rotation:", rotation
 
-        renderer = RendererPS(defaultWidth, defaultHeight, self._pswriter)
+        renderer = RendererPS(width, height, self._pswriter)
         self.figure.draw(renderer)
         renderer.finish()
 
@@ -382,7 +506,18 @@ class FigureCanvasPS(FigureCanvasBase):
         except IOError:
             error_msg_ps('Could not open %s for writing' % filename)
             return
+
+        print >>fh, _psProlog % (pstype, bboxstr)
+        print >>fh, _psDefs
+
+        type42 = _type42 + [os.path.join(self.basepath, name) + '.ttf' \
+                 for name in bakoma_fonts]
+        for font in type42:
+            font = str(font)  # todo: handle unicode filenames
+            print >>fh, _psFonts % (os.path.basename(font),
+                                    encodeTTFasPS(font))
         print >>fh, renderer.get_ps()
+
         self.figure.set_facecolor(origfacecolor)
         self.figure.set_edgecolor(origedgecolor)
 
@@ -404,6 +539,12 @@ _psProlog = """\
 %%%%EndProlog
 """
 
+_psFonts = """\
+%%BeginFont: %s
+%s
+%%EndFont
+"""
+
 _psDefs = """
 /gsr {gsave stroke grestore} def
 
@@ -438,9 +579,6 @@ clip
 }
 bind def
 
-
-
-
 /make-polygon
 {
 3 dict
