@@ -15,18 +15,28 @@ import matplotlib
 from matplotlib import rcParams
 
 from matplotlib.cbook import is_string_like,  enumerate, True, False
-from matplotlib.backend_bases import \
-     RendererBase, GraphicsContextBase, FigureManagerBase, FigureCanvasBase
+from matplotlib.backend_bases import RendererBase, GraphicsContextBase, \
+     FigureManagerBase, FigureCanvasBase, NavigationToolbar2, cursors, MplEvent
+
 from matplotlib.figure import Figure
 from matplotlib._matlab_helpers import Gcf
 from matplotlib.numerix import asarray
 
 import matplotlib.windowing as windowing
 
+import warnings
+
 # the true dots per inch on the screen; should be display dependent
 # see http://groups.google.com/groups?q=screen+dpi+x11&hl=en&lr=&ie=UTF-8&oe=UTF-8&safe=off&selm=7077.26e81ad5%40swift.cs.tcd.ie&rnum=5 for some info about screen dpi
 PIXELS_PER_INCH = 75
 
+cursord = {
+    cursors.MOVE: "fleur",    
+    cursors.HAND: "hand2",
+    cursors.POINTER: "arrow",
+    cursors.SELECT_REGION: "tcross",
+    }
+
 def round(x):
     return int(math.floor(x+0.5))
 
@@ -56,8 +66,8 @@ def show():
         manager.show()
     import matplotlib
     matplotlib.interactive(True)
-    os.environ['PYTHONINSPECT'] = '1'
-    # Tk.mainloop()
+    #os.environ['PYTHONINSPECT'] = '1'
+    Tk.mainloop()
 
 
 def new_figure_manager(num, *args, **kwargs):
@@ -73,6 +83,14 @@ def new_figure_manager(num, *args, **kwargs):
 
 
 class FigureCanvasTkAgg(FigureCanvasAgg):
+    keyvald = {65507 : 'control',
+               65505 : 'shift',
+               65513 : 'alt',
+               65508 : 'control',
+               65506 : 'shift',
+               65514 : 'alt',
+               }
+
     def __init__(self, figure, master=None, resize_callback=None):
         FigureCanvasAgg.__init__(self, figure)
         t1,t2,w,h = self.figure.bbox.get_bounds()
@@ -84,8 +102,12 @@ class FigureCanvasTkAgg(FigureCanvasAgg):
         self._tkcanvas.create_image(w/2, h/2, image=self._tkphoto)
         self._resize_callback = resize_callback
         self._tkcanvas.bind("<Configure>", self.resize)
+        self._tkcanvas.bind("<Key>", self.key_press)
+        self._tkcanvas.bind("<KeyRelease>", self.key_release)
         self._master = master
-
+        self._tkcanvas.focus_set()
+        self._key = None  # the key that is pressed
+        
     def resize(self, event):
         width, height = event.width, event.height
         if self._resize_callback is not None:
@@ -128,19 +150,90 @@ class FigureCanvasTkAgg(FigureCanvasAgg):
 
     def connect(self, eventname, handler):
         tkname = {
-            "motion_notify_event":"<Motion>",
-            "button_press_event":"<Button-1>",
-            "button_release_event":"<ButtonRelease-1>",
-            "key_press_event":"<Key>",
-            "key_release_event":"<KeyRelease>",            
+            "motion_notify_event":["<Motion>"],
+            "button_press_event": ["<Button-1>",
+                                   "<Button-2>",
+                                   "<Button-3>",
+                                   ],
+            "button_release_event": ["<ButtonRelease-1>",
+                                     "<ButtonRelease-2>",
+                                     "<ButtonRelease-3>",
+                                     ],
+            "key_press_event":["<Key>"],
+            "key_release_event":["<KeyRelease>"],            
             }
-        def tk_event_to_matplotlib_event(e):
-            'tkevent maps a Tkinter event object onto a matplotlib event'
+        def the_binding(e):
             e.button = e.num
-            return e
-        self._tkcanvas.bind(tkname[eventname],
-                            lambda e: handler(e.widget,
-                                              tk_event_to_matplotlib_event(e)))
+            return handler(e.widget, e)
+        for name in tkname[eventname]:
+            self._tkcanvas.bind(name, the_binding)
+        return (eventname, the_binding)
+
+    def key_press(self, event):
+        val = event.keysym_num
+        if self.keyvald.has_key(val):
+            key = self.keyvald[val]
+        elif val<256:
+            key = chr(val)
+        else:
+            key = None
+
+        self._key = key
+
+    def key_release(self, event):
+        self._key = None
+
+
+    def mpl_connect(self, s, func):
+        if s not in self.events:
+            error_msg('Can only connect events of type "%s"\nDo not know how to handle "%s"' %(', '.join(self.events), s))    
+
+        needPosition = s in  ('button_press_event',
+                              'button_release_event',
+                              'motion_notify_event',
+                              )
+
+        needButton = s in  ('button_press_event',
+                            'button_release_event',
+                              )
+
+        def wrapper(widget, event):
+            thisEvent = MplEvent(s, self) 
+            if needPosition:  
+                thisEvent.x = event.x
+                # flipy so y=0 is bottom of canvas
+                thisEvent.y = self.figure.bbox.height() - event.y
+
+            #print 'wrapper', event.keysym, event.keycode, event.keysym_num        
+
+            try: num = event.num
+            except AttributeError: thisEvent.button = None
+            else:
+                if num in (1,2,3):
+                    if sys.platform=='darwin':
+                        # 2 and 3 were reversed on the OSX platform I
+                        # tested under tkagg
+                        if   num==2: num=3
+                        elif num==3: num=2
+                    thisEvent.button = num
+                else: thisEvent.button = None
+            thisEvent.inaxes = None
+            thisEvent.key = self._key
+            for a in self.figure.get_axes():
+                if a.in_axes(thisEvent.x, thisEvent.y):
+                    thisEvent.inaxes = a
+                    xdata, ydata = a.transData.inverse_xy_tup((thisEvent.x, thisEvent.y))
+                    thisEvent.xdata  = xdata
+                    thisEvent.ydata  = ydata
+                    break
+                
+            func(thisEvent)
+            
+        return self.connect(s, wrapper)
+
+    def mpl_disconnect(self, cid):
+        eventname, the_binding = cid
+        self._tkcanvas.unbind(eventname) #, the_binding)
 
 class FigureManagerTkAgg(FigureManagerBase):
     """
@@ -161,8 +254,14 @@ class FigureManagerTkAgg(FigureManagerBase):
         t1,t2,w,h = canvas.figure.bbox.get_bounds()
         w, h = int(w), int(h)
         self.window.minsize(int(w*3/4),int(h*3/4))
-        self.toolbar = NavigationToolbar( canvas, self )
-        self.toolbar.update()
+        if matplotlib.rcParams['toolbar']=='classic':
+            self.toolbar = NavigationToolbar( canvas, self )
+        elif matplotlib.rcParams['toolbar']=='toolbar2':
+            self.toolbar = NavigationToolbar2TkAgg( canvas, self )
+        else:
+            self.toolbar = None
+        if self.toolbar is not None:
+            self.toolbar.update()
         self.canvas._tkcanvas.pack(side=Tk.TOP, fill=Tk.BOTH, expand=1)
 
         def destroy(*args):
@@ -181,12 +280,14 @@ class FigureManagerTkAgg(FigureManagerBase):
         
     def add_subplot(self, *args, **kwargs):
         a = FigureManagerBase.add_subplot(self, *args, **kwargs)
-        self.toolbar.update()
+        if self.toolbar is not None:
+            self.toolbar.update()
         return a
     
     def add_axes(self, rect, **kwargs):
         a = FigureManagerBase.add_axes(self, rect, **kwargs)
-        self.toolbar.update()
+        if self.toolbar is not None:
+            self.toolbar.update()
         return a
     
     def set_current_axes(self, a):
@@ -410,5 +511,125 @@ class NavigationToolbar(Tk.Frame):
         else:
             self.omenu.adjust(naxes)
 
+class NavigationToolbar2TkAgg(NavigationToolbar2, Tk.Frame):
+    """
+    Public attriubutes
+
+      canvas   - the FigureCanvas  (gtk.DrawingArea)
+      win   - the gtk.Window
+    """
+    def __init__(self, canvas, figman):
+        self.canvas = canvas
+        self.figman = figman
+        self._idle = True
+        NavigationToolbar2.__init__(self, canvas)
+
+    def draw_rubberband(self, event, x0, y0, x1, y1):
+        height = self.canvas.figure.bbox.height()
+        y0 =  height-y0
+        y1 =  height-y1
+        try: self.lastrect
+        except AttributeError: pass
+        else: self.canvas._tkcanvas.delete(self.lastrect)
+        self.lastrect = self.canvas._tkcanvas.create_rectangle(x0, y0, x1, y1)
+
+        #self.canvas.draw()
+
+    def release(self, event):
+        try: self.lastrect
+        except AttributeError: pass
+        else:
+            self.canvas._tkcanvas.delete(self.lastrect)
+            del self.lastrect
+
+    def set_cursor(self, cursor):
+        self.figman.window.configure(cursor=cursord[cursor])
+    
+    def _Button(self, text, file, command):
+        file = os.path.join(rcParams['datapath'], file)
+        im = Tk.PhotoImage(master=self, file=file)
+        b = Tk.Button(
+            master=self, text=text, padx=2, pady=2, image=im, command=command)
+        b._ntimage = im
+        b.pack(side=Tk.LEFT)
+        return b
+
+    def _init_toolbar(self):
+        xmin, xmax = self.canvas.figure.bbox.intervalx().get_bounds()
+        height, width = 50, xmax-xmin
+        Tk.Frame.__init__(self, master=self.figman.window,
+                          width=width, height=height,
+                          borderwidth=2)
+        
+        self.update()  # Make axes menu
+
+        self.bHome = self._Button( text="Home", file="home.ppm",
+                                   command=self.home)
+
+        self.bBack = self._Button( text="Back", file="back.ppm",
+                                   command = self.back)
+        
+        self.bForward = self._Button(text="Forward", file="forward.ppm",
+                                     command = self.forward)
+
+        self.bPan = self._Button( text="Pan", file="move.ppm",
+                                  command = self.pan)
+
+        self.bZoom = self._Button( text="Zoom",
+                                   file="zoom_to_rect.ppm",
+                                   command = self.zoom)
+
+        self.bsave = self._Button( text="Save", file="filesave.ppm",
+                                   command = self.save_figure)
+        
+        self.pack(side=Tk.BOTTOM)
+        
+    def save_figure(self):
+        fs = FileDialog.SaveFileDialog(master=self.figman.window,
+                                       title='Save the figure')
+        try:
+            self.lastDir
+        except AttributeError:
+            self.lastDir = os.curdir
+            
+        fname = fs.go(dir_or_file=self.lastDir) # , pattern="*.png")
+        if fname is None: # Cancel
+            return
+        
+        self.lastDir = os.path.dirname(fname)
+        try:
+            self.canvas.print_figure(fname)
+        except IOError, msg:                
+            err = '\n'.join(map(str, msg))
+            msg = 'Failed to save %s: Error msg was\n\n%s' % (
+                fname, err)
+            error_msg_tkpaint(msg)
+
+    def set_active(self, ind):
+        self._ind = ind
+        self._active = [ self._axes[i] for i in self._ind ]
+
+    def update(self):
+        _focus = windowing.FocusManager()
+        self._axes = self.canvas.figure.axes
+        naxes = len(self._axes)
+        #if not hasattr(self, "omenu"):
+        #    self.set_active(range(naxes))
+        #    self.omenu = AxisMenu(master=self, naxes=naxes)
+        #else:
+        #    self.omenu.adjust(naxes)
+        NavigationToolbar2.update(self)
+
+    def dynamic_update(self):
+        'update drawing area only if idle'
+        d = self._idle
+        self._idle = False
+        def idle_draw(*args):
+            self.canvas.draw()
+            self._idle = True
+
+        if d: self.canvas._tkcanvas.after_idle(idle_draw)
+            
+
 FigureManager = FigureManagerTkAgg
 error_msg = error_msg_tkpaint
