@@ -17,177 +17,6 @@ from __future__ import division
  should be included with this source code.
 
 """
-# CVS log information
-#
-# $Log: backend_wx.py,v $
-# Revision 1.62  2004/05/21 14:41:32  jdh2358
-# added frameon for figure
-#
-# Revision 1.61  2004/05/14 18:23:35  jdh2358
-# new transformation / artist constructors
-#
-# Revision 1.60  2004/05/09 21:13:25  jodonoghue
-# Minor bugfixes: navigation toolbar now shows on MacOSX and quit operates correctly after last frame closed on all platforms.
-#
-# Revision 1.59  2004/05/07 14:48:47  jdh2358
-# fixed ismath sig across backends
-#
-# Revision 1.58  2004/05/04 18:37:43  jdh2358
-# major overhaul of text
-#
-# Revision 1.57  2004/05/03 16:18:42  jdh2358
-# removed close buttons on all GUIs per Steves advice
-#
-# Revision 1.56  2004/04/17 22:24:28  jdh2358
-# multiple scales on same axes
-#
-# Revision 1.55  2004/04/16 22:49:33  jdh2358
-# fixed wx exception swallowing bug
-#
-# Revision 1.54  2004/04/16 21:37:07  jdh2358
-# renamed coords demo
-#
-# Revision 1.53  2004/04/16 19:30:01  barrett
-# Minor fixes to GTK and WX backends to work with fonts_demo example.
-#
-# Revision 1.51  2004/04/05 12:58:54  jdh2358
-# removed debug info from wxagg
-#
-# Revision 1.50  2004/04/04 20:10:31  jdh2358
-# added prototype wxagg backend
-#
-# Revision 1.49  2004/03/16 20:26:39  jdh2358
-# sync to 0.52 release
-#
-# Revision 1.48  2004/03/08 22:17:21  jdh2358
-#
-# * Fixed embedding_in_wx and dynamic_demo_wx examples
-#
-# * Ported build to darwin
-#
-# * Tk:
-#
-#   removed default figman=None from nav toolbar since it needs the
-#   figman
-#
-#   fixed close bug
-#
-#   small changes to aid darwin build
-#
-# Revision 1.47  2004/03/08 14:56:52  jdh2358
-# fixed subplot bug
-#
-# Revision 1.46  2004/03/03 19:27:43  jdh2358
-# fixed gtk vert text bug
-#
-# Revision 1.45  2004/02/27 21:46:32  jdh2358
-# close to porting mathtext to gtk
-#
-# Revision 1.44  2004/02/27 14:47:48  jdh2358
-# added matplotlibrc and tk win32 compile fixes
-#
-# Revision 1.43  2004/02/23 16:20:56  jdh2358
-# resynced cvs to freetype2
-#
-# Revision 1.40  2004/02/16 18:05:01  jdh2358
-# fexed wx to work on windows and linux
-#
-# Revision 1.38  2004/02/16 03:50:25  jdh2358
-# fixed wx draw_poly bug
-#
-# Revision 1.37  2004/02/16 03:30:05  jdh2358
-# fixed wx backend
-#
-# Revision 1.35  2004/02/15 09:31:59  jodonoghue
-# Update backend_wx to account for API changes. Added select() and unselect() to GraphicsContextWx to enable fine control over which GC instances are selected into the bitmap. Issues remain with clipping and dotted line types.
-#
-# Revision 1.33  2004/02/10 21:36:01  jdh2358
-# added agg backend, changed facecolor to gcface
-#
-# Revision 1.32  2004/02/05 15:42:20  jdh2358
-# added paint backend
-#
-# Revision 1.31  2004/02/03 16:24:59  jdh2358
-# gtkgd ext mod and bugfixes
-#
-# Revision 1.30  2004/01/30 21:33:21  jdh2358
-# last changes for 0.50e
-#
-# Revision 1.28  2004/01/30 19:52:24  jdh2358
-# ported wx to new backend
-#
-# Revision 1.26  2004/01/28 21:57:24  jdh2358
-# refactored figure backend
-#
-# Revision 1.25  2004/01/27 18:23:38  jdh2358
-# 0.42 release
-#
-# Revision 1.23  2004/01/26 20:10:55  jdh2358
-# added EPS save
-#
-# Revision 1.20  2003/12/30 20:29:07  jdh2358
-# changed interactive interface
-#
-# Revision 1.19  2003/12/30 17:26:14  jodonoghue
-# Fixed bugs 866967 (ShowOn.set(1) not working in interactive mode) and
-# 866969 (Dynamic update not possible with backend_wx)
-#
-# Revision 1.18  2003/12/15 15:43:52  jdh2358
-# fixed install path problem
-#
-# Revision 1.17  2003/12/05 18:23:49  jdh2358
-# resyncing wx
-#
-# Revision 1.16  2003/12/02 23:54:07  jodonoghue
-# Fixed bug on wxGTK platform: assert self.Ok() in
-# GraphicsContextWx.__init__() fails. This seems
-# to be because the paltform does not allow call
-# to Ok() within derived constructor.
-#
-# Revision 1.15  2003/12/02 15:28:20  jodonoghue
-# Implemented double buffered drawing. Implemented save
-# to several file formats. Corrected initial figure
-# placement bug on Linux. Corrected figure size bug
-# on Windows. Corrected text attribute redraw bug on
-# all platforms. Fixed mousewheel operation on Windows
-# (subject to known bug behaviour). Fixed toolbar
-# buttons causing multiple events on Windows.
-#
-# Revision 1.13  2003/11/21 17:37:02  jodonoghue
-# Update to changed Gcf and FigureManager API
-#
-# Revision 1.12  2003/11/17 18:15:14  jodonoghue
-# Implemented backend changes for gcFace attribute in polygons. Fixed
-# figure size difference from GTK, fixed incorrectly sized figure under
-# Linux, modified toolbar to be exportable, modified mousewheel control
-# to work over buttons as for GTK. Started implementation of saving bitmaps
-# (not yet working).
-#
-# Revision 1.11  2003/11/14 18:49:53  jodonoghue
-# Fixed bug in _onSize: self.bbox.set_bounds(), fixed interactive buttons
-# to work without need for menu to be pressed first, menu button contains
-# identities of selected axes. Preliminary support for a a wxPython control 
-# encapsulating a FigureWx and toolbar - this is a work in progress, with
-# some wrinkles, but works for examples.
-#
-# Revision 1.10  2003/11/12 18:30:54  jodonoghue
-# Updates for refactored axes. Fixed xlabel location bug. Fixed pcolor
-# rounding error (JDH patch), fixed problem in subplot_demo.py.
-#
-# Revision 1.9  2003/11/11 22:23:51  jdh2358
-# refactored axes
-#
-# Revision 1.8  2003/11/11 16:22:02  jodonoghue
-# Added status bar. Added mousewheel support. Fixed script session does
-# not exit cleanly, fixed legend not displayed, fixed no clipping on text
-# labels, fixed vertical text renders improperly
-#
-# Revision 1.7  2003/11/10 17:40:10  jodonoghue
-# Added preliminary interactive zoom and pan support. Fixed incorrect circle
-# centres, graphics clipping (text clipping stillnot implemented), coloured
-# text, axis background.
-#
-#
 """
 KNOWN BUGS -
  - Mousewheel (on Windows) only works after menu button has been pressed
@@ -263,7 +92,7 @@ Examples which work on this release:
  (3) - Clipping seems to be broken.
 """
 
-cvs_id = '$Id: backend_wx.py,v 1.62 2004/05/21 14:41:32 jdh2358 Exp $'
+cvs_id = '$Id: backend_wx.py,v 1.81 2004/08/05 18:27:33 jdh2358 Exp $'
 
 import sys, os, os.path, math, StringIO
 
@@ -320,7 +149,8 @@ class fake_stderr:
 
 import matplotlib
 from matplotlib.backend_bases import RendererBase, GraphicsContextBase,\
-     FigureCanvasBase, FigureManagerBase, error_msg
+     FigureCanvasBase, FigureManagerBase, error_msg, NavigationToolbar2, \
+     MplEvent, cursors
 from matplotlib._matlab_helpers import Gcf
 from matplotlib.artist import Artist
 from matplotlib.cbook import exception_to_str
@@ -591,16 +421,11 @@ class RendererWx(RendererBase):
 
         font = self.fontd.get(key)
         if font is not None:
-            #print 'returning cached'
             return font
 
         # Allow use of platform independent and dependent font names
-        try:
-            wxFontname = self.fontnames[fontname]
-            wxFacename = '' # Empty => wxPython chooses based on wx_fontname
-        except KeyError:
-            wxFontname = wxSWISS
-            wxFacename = fontname
+        wxFontname = self.fontnames.get(fontname, wxROMAN)
+        wxFacename = '' # Empty => wxPython chooses based on wx_fontname
 
         # Font colour is determined by the active wxPen
         # TODO: It may be wise to cache font information
@@ -715,7 +540,7 @@ class GraphicsContextWx(GraphicsContextBase, wxMemoryDC):
                  int(w), int(h))
         self.unselect()
 
-    def set_foreground(self, fg):
+    def set_foreground(self, fg, isRGB=None):
         """
         Set the foreground color.  fg can be a matlab format string, a
         html hex color string, an rgb unit tuple, or a float between 0
@@ -728,7 +553,7 @@ class GraphicsContextWx(GraphicsContextBase, wxMemoryDC):
         # Same goes for text foreground...
         DEBUG_MSG("set_foreground()", 1, self)
         self.select()
-        GraphicsContextBase.set_foreground(self, fg)
+        GraphicsContextBase.set_foreground(self, fg, isRGB)
 
         pen = self.GetPen()
         pen.SetColour(self.get_wxcolour())
@@ -841,11 +666,16 @@ class FigureCanvasWx(FigureCanvasBase, wxPanel):
     implements a wxSizer to control the displayed control size - but we give a
     hint as to our preferred minimum size.
     """
+    keyvald = {308 : 'control',
+               306 : 'shift',
+               307 : 'alt',
+               }
+
     def __init__(self, parent, id, figure):
         """
         Initialise a FigureWx instance.
         
-        - Initialise the FigureBase and wxPanel parents.
+        - Initialise the FigureCanvasBase and wxPanel parents.
         - Set event handlers for:
           EVT_SIZE  (Resize event)
           EVT_PAINT (Paint event)
@@ -858,6 +688,9 @@ class FigureCanvasWx(FigureCanvasBase, wxPanel):
         w = int(math.ceil(w))
         h = int(math.ceil(h))
 
+        self._key = None
+
+        
         wxPanel.__init__(self, parent, id, size=wxSize(w, h))
         # Create the drawing bitmap
         self.bitmap = wxEmptyBitmap(w, h)
@@ -869,19 +702,71 @@ class FigureCanvasWx(FigureCanvasBase, wxPanel):
         # Event handlers
         EVT_SIZE(self, self._onSize)
         EVT_PAINT(self, self._onPaint)
+        EVT_KEY_DOWN(self, self._onKeyDown)
+        EVT_KEY_UP(self, self._onKeyUp)
         EVT_RIGHT_DOWN(self, self._onRightButtonDown)
         EVT_RIGHT_UP(self, self._onRightButtonUp)
         EVT_MOUSEWHEEL(self, self._onMouseWheel)
         EVT_LEFT_DOWN(self, self._onLeftButtonDown)
         EVT_LEFT_UP(self, self._onLeftButtonUp)
+
+        self.macros = {} # dict from wx id to seq of macros
+
         
-    def add_axis(self, a):
-        """
-        Add an instance of Axis to the figure. 
-        """
-        FigureBase.add_axis(self, a)
-        DEBUG_MSG("add_axis()", 1, self)
-        #a.set_fig_bgcolor(self._gray)
+    def mpl_disconnect(self, cid):
+        for macro in self.macros.get(cid, []):
+            macro(self, None)
+
+
+        
+    def mpl_connect(self, s, func):
+
+        if s not in self.events:
+            error_msg('Can only connect events of type "%s"\nDo not know how to handle "%s"' %(', '.join(self.events), s))    
+            
+        cid = wxNewId()
+
+        def wrapper(event):
+            thisEvent = MplEvent(s, self) 
+
+            thisEvent.x = event.GetX()
+            # flipy so y=0 is bottom of canvas
+            thisEvent.y = self.figure.bbox.height() - event.GetY()
+
+            if event.LeftDown(): button = 1
+            elif event.MiddleDown(): button = 2
+            elif event.RightDown(): button = 3
+            else: button = None
+            thisEvent.button = button
+            thisEvent.key = self._key
+
+            thisEvent.inaxes = None
+            for a in self.figure.get_axes():
+                if a.in_axes(thisEvent.x, thisEvent.y):
+                    thisEvent.inaxes = a
+                    xdata, ydata = a.transData.inverse_xy_tup((thisEvent.x, thisEvent.y))
+                    thisEvent.xdata  = xdata
+                    thisEvent.ydata  = ydata
+                    break
+                
+            
+            func(thisEvent)
+            event.Skip()
+            return False  # return True blocks other connects
+
+
+        if s=='button_press_event':
+            EVT_LEFT_DOWN(self,  wrapper)
+            EVT_RIGHT_DOWN(self, wrapper)
+            self.macros[cid] = (EVT_LEFT_DOWN, EVT_RIGHT_DOWN)
+        if s=='button_release_event':
+            EVT_LEFT_UP(self,  wrapper)            
+            EVT_RIGHT_UP(self, wrapper)
+            self.macros[cid] = (EVT_LEFT_UP, EVT_RIGHT_UP)
+        elif s=='motion_notify_event':
+            EVT_MOTION(self, wrapper)
+            self.macros[cid] = (EVT_MOTION,)
+        return cid
                
     def draw(self):
         """
@@ -891,8 +776,19 @@ class FigureCanvasWx(FigureCanvasBase, wxPanel):
         DEBUG_MSG("draw()", 1, self)
         self.renderer = RendererWx(self.bitmap, self.figure.dpi)
         self.figure.draw(self.renderer)
-        
-        
+        self.gui_repaint()
+
+    def _get_imagesave_wildcards(self):
+        'return the wildcard string for the filesave dialog'
+        return "JPEG (*.jpg)|*.jpg|" \
+               "PS (*.ps)|*.ps|"     \
+               "EPS (*.eps)|*.eps|"  \
+               "SVG (*.svg)|*.svg|"  \
+               "BMP (*.bmp)|*.bmp|"  \
+               "PCX (*.pcx)|*.pcx|"  \
+               "PNG (*.png)|*.png|"  \
+               "XPM (*.xpm)|*.xpm" 
+
     def gui_repaint(self):
         """
         Performs update of the displayed image on the GUI canvas
@@ -902,7 +798,7 @@ class FigureCanvasWx(FigureCanvasBase, wxPanel):
         DEBUG_MSG("gui_repaint()", 1, self)
         drawDC=wxClientDC(self)
         drawDC.BeginDrawing()
-        drawDC.Clear()
+        #drawDC.Clear()
         drawDC.DrawBitmap(self.bitmap, 0, 0)
         drawDC.EndDrawing()        
         
@@ -928,10 +824,21 @@ class FigureCanvasWx(FigureCanvasBase, wxPanel):
             origDPI = self.figure.dpi.get()
             ps = self.switch_backends(FigureCanvasPS)
             ps.figure.dpi.set(72)
-            for a in ps.figure.get_axes(): a.resize()
+
             ps.print_figure(filename, 72, facecolor, edgecolor)
             self.figure.dpi.set(origDPI)
-            for a in self.figure.axes: a.resize()
+            return
+        elif ext.find('svg')>=0:
+            # enable svg save from WX backend only import this if we
+            # need it since it parse afm files on import
+            from backend_svg import FigureCanvasSVG
+
+            DEBUG_MSG("print_figure() saving SVG", 1, self)
+            origDPI = self.figure.dpi.get()
+            svg = self.switch_backends(FigureCanvasSVG)
+            svg.figure.dpi.set(72)
+            svg.print_figure(filename, 72, facecolor, edgecolor)
+            self.figure.dpi.set(origDPI)                                       
             return
 
         if not self._isRealized:
@@ -1063,16 +970,40 @@ class FigureCanvasWx(FigureCanvasBase, wxPanel):
             self.draw()
         evt.Skip()
         
+
+    def _onKeyDown(self, evt):
+        """Capture key press."""
+        evt.Skip()
+        keyval = evt.m_keyCode
+        if self.keyvald.has_key(keyval):
+            key = self.keyvald[keyval]
+        elif keyval <256:
+            key = chr(keyval)
+        else:
+            key = None
+            
+        self._key = key.lower()
+
+        
+        
+    def _onKeyUp(self, evt):
+        """Release key."""
+        evt.Skip()
+        self._key = None
+
     def _onRightButtonDown(self, evt):
-        # TODO: implement right button down handler - show motion menu
-        pass
+        """Start measuring on an axis."""
+        #print 'left down', evt, dir(evt)
+        evt.Skip()
         
     def _onRightButtonUp(self, evt):
-        # TODO: implement right button up handler - remove motion menu?
-        pass
-        
+        """End measuring on an axis."""
+        evt.Skip()
+
+
     def _onLeftButtonDown(self, evt):
         """Start measuring on an axis."""
+
         evt.Skip()
         
     def _onLeftButtonUp(self, evt):
@@ -1105,7 +1036,7 @@ def draw_if_interactive():
         figManager = Gcf.get_active()
         if figManager is not None:
             figManager.canvas.draw()
-            figManager.canvas.gui_repaint()            
+            #figManager.canvas.gui_repaint()            
 def show():
     """
     Current implementation assumes that matplotlib is executed in a PyCrust
@@ -1165,23 +1096,32 @@ class FigureFrameWx(wxFrame):
         # By adding toolbar in sizer, we are able to put it at the bottom
         # of the frame - so appearance is closer to GTK version
 
-        self.toolbar = NavigationToolbarWx(self.canvas, True)
-        self.toolbar.Realize()
-        if wxPlatform == '__WXMAC__':
-            # Mac platform (OSX 10.3, MacPython) does not seem to cope with
-            # having a toolbar in a sizer. This work-around gets the buttons
-            # back, but at the expense of having the toolbar at the top
-            self.SetToolBar(self.toolbar)
+
+        if matplotlib.rcParams['toolbar']=='classic':
+            self.toolbar = NavigationToolbarWx(self.canvas, True)
+        elif matplotlib.rcParams['toolbar']=='toolbar2':
+            self.toolbar = NavigationToolbar2Wx(self.canvas)            
         else:
-            # On Windows platform, default window size is incorrect, so set
-            # toolbar width to figure width.
-            tw, th = self.toolbar.GetSizeTuple()
-            fw, fh = self.canvas.GetSizeTuple()
-            # By adding toolbar in sizer, we are able to put it at the bottom
-            # of the frame - so appearance is closer to GTK version.
-            # As noted above, doesn't work for Mac.
-            self.toolbar.SetSize(wxSize(fw, th))
-            self.sizer.Add(self.toolbar, 0, wxLEFT | wxEXPAND)
+            self.toolbar = None
+        
+
+        if self.toolbar is not None:
+            self.toolbar.Realize()
+            if wxPlatform == '__WXMAC__':
+                # Mac platform (OSX 10.3, MacPython) does not seem to cope with
+                # having a toolbar in a sizer. This work-around gets the buttons
+                # back, but at the expense of having the toolbar at the top
+                self.SetToolBar(self.toolbar)
+            else:
+                # On Windows platform, default window size is incorrect, so set
+                # toolbar width to figure width.
+                tw, th = self.toolbar.GetSizeTuple()
+                fw, fh = self.canvas.GetSizeTuple()
+                # By adding toolbar in sizer, we are able to put it at the bottom
+                # of the frame - so appearance is closer to GTK version.
+                # As noted above, doesn't work for Mac.
+                self.toolbar.SetSize(wxSize(fw, th))
+                self.sizer.Add(self.toolbar, 0, wxLEFT | wxEXPAND)
         self.SetSizer(self.sizer)
         self.Fit()
 
@@ -1224,13 +1164,13 @@ class FigureManagerWx(FigureManagerBase):
     def add_subplot(self, *args, **kwargs):
         DEBUG_MSG("add_subplot()", 1, self)
         a = FigureManagerBase.add_subplot(self, *args, **kwargs)
-        self.tb.update()
+        if self.tb is not None: self.tb.update()
         return a
         
     def add_axes(self, rect, **kwargs):
         DEBUG_MSG("add_axes()", 1, self)
         a = FigureManagerBase.add_axes(self, rect, **kwargs)
-        self.tb.update()
+        if self.tb is not None: self.tb.update()
         return a
     
     def set_current_axes(self, a):
@@ -1280,7 +1220,7 @@ def _load_bitmap(filename):
         print >>sys.stderr, 'Could not find bitmap file; dying', bmpFilename
         sys.exit()
     bmp = wxBitmap(bmpFilename, wxBITMAP_TYPE_XPM)
-    return  bmp
+    return bmp
 
 class MenuButtonWx(wxButton):
     """
@@ -1376,6 +1316,133 @@ class MenuButtonWx(wxButton):
         # remove trailing ',' and add to button string
         self.SetLabel("Axes: %s" % axis_txt[:-1])
 
+
+
+        
+cursord = {
+    cursors.MOVE : wxCURSOR_HAND,    
+    cursors.HAND : wxCURSOR_HAND,
+    cursors.POINTER : wxCURSOR_ARROW,
+    cursors.SELECT_REGION : wxCURSOR_CROSS,
+    }
+
+class NavigationToolbar2Wx(NavigationToolbar2, wxToolBar):
+    def __init__(self, canvas):
+        wxToolBar.__init__(self, canvas.GetParent(), -1)
+        NavigationToolbar2.__init__(self, canvas)
+        self.canvas = canvas
+        self._idle = True
+        
+    def _init_toolbar(self):
+        DEBUG_MSG("_init_toolbar", 1, self)
+
+        self._parent = self.canvas.GetParent()
+        _NTB2_HOME    = wxNewId()
+        _NTB2_BACK    = wxNewId()
+        _NTB2_FORWARD = wxNewId()
+        _NTB2_PAN     = wxNewId()
+        _NTB2_ZOOM    = wxNewId()
+        _NTB2_SAVE    = wxNewId()        
+        
+        self.SetToolBitmapSize(wxSize(24,24))
+
+        self.AddSimpleTool(_NTB2_HOME, _load_bitmap('home.xpm'),
+                           '', 'Reset original view')
+        self.AddSimpleTool(_NTB2_BACK, _load_bitmap('back.xpm'),
+                           '', 'Back navigation view')
+        self.AddSimpleTool(_NTB2_FORWARD, _load_bitmap('forward.xpm'),
+                           '', 'Forward navigation view')
+        # todo: get new bitmap
+        self.AddSimpleTool(_NTB2_PAN, _load_bitmap('move.xpm'),
+                           '', 'Pan with left, zoom with right')
+        self.AddSimpleTool(_NTB2_ZOOM, _load_bitmap('zoom_to_rect.xpm'),
+                           '', 'Zoom to rectangle')
+        
+        self.AddSeparator()
+        self.AddSimpleTool(_NTB2_SAVE, _load_bitmap('filesave.xpm'),
+                           '', 'Save plot contents to file')
+
+        
+        EVT_TOOL(self, _NTB2_HOME, self.home)
+        EVT_TOOL(self, _NTB2_FORWARD, self.forward)
+        EVT_TOOL(self, _NTB2_BACK, self.back)
+        EVT_TOOL(self, _NTB2_ZOOM, self.zoom)
+        EVT_TOOL(self, _NTB2_PAN, self.pan)
+        EVT_TOOL(self, _NTB2_SAVE, self.save)
+
+        self.Realize()
+
+
+    def save(self, evt):
+        # Fetch the required filename and file type.
+        filetypes = self.canvas._get_imagesave_wildcards()
+        dlg = wxFileDialog(self._parent, "Save to file", "", "", filetypes,
+                           wxSAVE|wxOVERWRITE_PROMPT|wxCHANGE_DIR)
+        if dlg.ShowModal() == wxID_OK:
+            dirname  = dlg.GetDirectory()
+            filename = dlg.GetFilename()
+            DEBUG_MSG('Save file dir:%s name:%s' % (dirname, filename), 3, self)
+            self.canvas.print_figure(os.path.join(dirname, filename))
+
+
+    def set_cursor(self, cursor):
+        cursor = wxStockCursor(cursord[cursor])
+        self.canvas.SetCursor( cursor )
+
+    def release(self, event):
+        try: del self.lastrect
+        except AttributeError: pass
+
+    def dynamic_update(self):
+        d = self._idle
+        self._idle = False
+        if d:
+            self.canvas.draw()
+            self._idle = True
+        
+    def draw_rubberband(self, event, x0, y0, x1, y1):
+        'adapted from http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/189744'
+        canvas = self.canvas
+        dc = wxClientDC(canvas)
+            
+        # Set logical function to XOR for rubberbanding
+        dc.SetLogicalFunction(wxXOR)
+            
+        # Set dc brush and pen
+        # Here I set brush and pen to white and grey respectively
+        # You can set it to your own choices
+            
+        # The brush setting is not really needed since we
+        # dont do any filling of the dc. It is set just for 
+        # the sake of completion.
+
+        wbrush = wxBrush(wxColour(255,255,255), wxTRANSPARENT)
+        wpen = wxPen(wxColour(200, 200, 200), 1, wxSOLID)
+        dc.SetBrush(wbrush)
+        dc.SetPen(wpen)
+
+            
+        dc.ResetBoundingBox()
+        dc.BeginDrawing()
+        height = self.canvas.figure.bbox.height()
+        y1 = height - y1
+        y0 = height - y0
+        
+        if y1<y0: y0, y1 = y1, y0
+        if x1<y0: x0, x1 = x1, x0
+        
+        w = x1 - x0
+        h = y1 - y0
+
+        rect = int(x0), int(y0), int(w), int(h)
+        try: lastrect = self.lastrect
+        except AttributeError: pass
+        else: dc.DrawRectangle(*lastrect)  #erase last
+        self.lastrect = rect
+        dc.DrawRectangle(*rect)
+        dc.EndDrawing()
+               
+
 class NavigationToolbarWx(wxToolBar):
     def __init__(self, canvas, can_kill=False):
         """
@@ -1576,12 +1643,7 @@ class NavigationToolbarWx(wxToolBar):
 
     def _onSave(self, evt):
         # Fetch the required filename and file type.
-        filetypes = "JPEG (*.jpg)|*.jpg|"   \
-                    "BMP (*.bmp)|*.bmp|"    \
-                    "PCX (*.pcx)|*.pcx|"     \
-                    "PNG (*.png)|*.png|"    \
-                    "TIFF (*.tif)|*.tif|" \
-                    "XPM (*.xpm)|*.xpm|" 
+        filetypes = self.canvas._get_imagesave_wildcards()
         dlg = wxFileDialog(self._parent, "Save to file", "", "", filetypes,
                            wxSAVE|wxOVERWRITE_PROMPT|wxCHANGE_DIR)
         if dlg.ShowModal() == wxID_OK:
