@@ -1,20 +1,25 @@
 import sys
 from artist import Artist
 from axes import Axes, Subplot
-from cbook import flatten, True, False
+from cbook import flatten, True, False, allequal
+import _image
+from colors import normalize
+from image import FigureImage
 from matplotlib import rcParams
 from patches import Rectangle
 from text import Text, _process_text_args
 from legend import Legend
 from transforms import Bbox, Value, Point, get_bbox_transform, unit_bbox
 
+
+
 class Figure(Artist):
     
     def __init__(self,
-                 figsize   = rcParams['figure.figsize'],
-                 dpi       = rcParams['figure.dpi'],
-                 facecolor = rcParams['figure.facecolor'],
-                 edgecolor = rcParams['figure.edgecolor'],
+                 figsize   = None, # defaults to rc figure.figsize
+                 dpi       = None, # defaults to rc figure.dpi
+                 facecolor = None, # defaults to rc figure.facecolor
+                 edgecolor = None, # defaults to rc figure.edgecolor
                  frameon = True,
                  ):
         """
@@ -22,7 +27,12 @@ class Figure(Artist):
         DPI is dots per inch 
         """
         Artist.__init__(self)
-        self.set_figure(self)
+        #self.set_figure(self)
+
+        if figsize is None  : figsize   = rcParams['figure.figsize']
+        if dpi is None      : dpi       = rcParams['figure.dpi']
+        if facecolor is None: facecolor = rcParams['figure.facecolor']
+        if edgecolor is None: edgecolor = rcParams['figure.edgecolor']
         
         self.dpi = Value(dpi)
         self.figwidth = Value(figsize[0])
@@ -35,9 +45,6 @@ class Figure(Artist):
         
         self.transFigure = get_bbox_transform( unit_bbox(), self.bbox) 
 
-        self.axes = []
-        self._text=[]
-        self._legends = []
 
         
         self._figurePatch = Rectangle(
@@ -46,6 +53,85 @@ class Figure(Artist):
             )
         self._set_artist_props(self._figurePatch)
 
+        self._hold = rcParams['axes.hold']
+        self.clf()
+
+
+    def hold(self, b=None):
+        """
+        Set the hold state.  If hold is None (default), toggle the
+        hold state.  Else set the hold state to boolean value b.
+
+        Eg
+        hold()      # toggle hold
+        hold(True)  # hold is on
+        hold(False) # hold is off
+        """
+        if b is None: self._hold = not self._hold
+        else: self._hold = b
+
+    def figimage(self, X,
+                 xo=0,
+                 yo=0,
+                 alpha=1.0,
+                 norm=None,
+                 cmap=None, 
+                 vmin=None,
+                 vmax=None,
+                 origin=None):
+        """\
+FIGIMAGE(X) # add non-resampled array to figure
+
+FIGIMAGE(X, xo, yo) # with pixel offsets
+
+FIGIMAGE(X, **kwargs) # control interpolation ,scaling, etc
+
+Add a nonresampled figure to the figure from array X.  xo and yo are
+offsets in pixels
+
+X must be a float array
+
+    If X is MxN, assume luminance (grayscale)
+    If X is MxNx3, assume RGB
+    If X is MxNx4, assume RGBA
+
+The following kwargs are allowed: 
+
+  * cmap is a cm colormap instance, eg cm.jet.  If None, default to
+    the rc image.cmap valuex
+
+  * norm is a matplotlib.colors.normalize instance; default is
+    normalization().  This scales luminance -> 0-1
+
+  * vmin and vmax are used to scale a luminance image to 0-1.  If
+    either is None, the min and max of the luminance values will be
+    used.  Note if you pass a norm instance, the settings for vmin and
+    vmax will be ignored.
+
+  * alpha = 1.0 : the alpha blending value
+
+  * origin is either 'upper' or 'lower', which indicates where the [0,0]
+    index of the array is in the upper left or lower left corner of
+    the axes.  Defaults to the rc image.origin value
+
+This complements the axes image which will be resampled to fit the
+current axes.  If you want a resampled image to fill the entire
+figure, you can define an Axes with size [0,1,0,1].
+
+A image.FigureImage instance is returned.
+"""        
+
+        if not self._hold: self.clf()
+
+        im = FigureImage(self, cmap, norm, xo, yo, origin)
+        im.set_array(X)
+        im.set_alpha(alpha)
+        if norm is None:
+            im.set_clim(vmin, vmax)
+        self._images.append(im )
+        return im
+
+        
     def set_figsize_inches(self, w, h):
         'set the figure size in inches'
         self.figwidth.set(w)
@@ -79,7 +165,7 @@ instructions on how to port your code.
 
         sys.exit()
         
-    def add_axes(self, rect, axisbg=rcParams['axes.facecolor'], frameon=True):
+    def add_axes(self, rect, axisbg=None, frameon=True):
         """
         Add an a axes with axes rect [left, bottom, width, height]
         where all quantities are in fractions of figure width and
@@ -87,7 +173,7 @@ instructions on how to port your code.
 
         The Axes instance will be returned
         """
-        
+        if axisbg is None: axisbg=rcParams['axes.facecolor']
         a = Axes(self, rect, axisbg, frameon)
         self.axes.append(a)
         return a
@@ -108,7 +194,9 @@ instructions on how to port your code.
         Clear the figure
         """
         self.axes = []
-        self._text = []
+        self._text=[]
+        self._legends = []
+        self._images = []
 
     def clear(self):
         """
@@ -122,9 +210,27 @@ instructions on how to port your code.
         """
         # draw the figure bounding box, perhaps none for white figure
 
-
+        renderer.open_group('figure')
         self.transFigure.freeze()  # eval the lazy objects
         if self.frameon: self._figurePatch.draw(renderer)
+        if len(self._images)==1:
+            im = self._images[0]
+            im.draw(renderer)
+        elif len(self._images)>1:
+            # make a composite image blending alpha
+            # list of (_image.Image, ox, oy)
+            if not allequal([im.origin for im in self._images]):
+                raise ValueError('Composite images with different origins not supported')
+            else:
+                origin = self._images[0].origin
+
+            ims = [(im.make_image(), im.ox, im.oy) for im in self._images]
+            im = _image.from_images(self.bbox.height(), self.bbox.width(), ims)
+            im.is_grayscale = False
+            l, b, w, h = self.bbox.get_bounds()
+            renderer.draw_image(0, 0, im, origin, self.bbox)
+
+
 
         # render the axes
         for a in self.axes: a.draw(renderer)
@@ -136,6 +242,7 @@ instructions on how to port your code.
             legend.draw(renderer)
 
         self.transFigure.thaw()  # release the lazy objects
+        renderer.close_group('figure')
 
     def get_axes(self):
         return self.axes
@@ -171,7 +278,7 @@ instructions on how to port your code.
 
 
         handles = flatten(handles)
-        l = Legend(handles, labels, loc)
+        l = Legend(self, handles, labels, loc, isaxes=False)
         self._set_artist_props(l)
         self._legends.append(l)
         return l
@@ -193,6 +300,12 @@ instructions on how to port your code.
         return t
 
     def _set_artist_props(self, a):
-        a.set_figure(self)
+        if a!= self:
+            a.set_figure(self)
         a.set_transform(self.transFigure)
 
+    def get_width_height(self):
+        'return the figure width and height in pixels'
+        w = self.bbox.width()
+        h = self.bbox.height()
+        return w, h
