@@ -9,10 +9,10 @@ from __future__ import division
 import sys
 
 from numerix import Float, alltrue, arange, array, logical_and,\
-     nonzero, searchsorted, take, asarray, ones, where, less
+     nonzero, searchsorted, take, asarray, ones, where, less, ravel
 
 from artist import Artist
-from cbook import True, False
+from cbook import True, False, iterable
 from collections import RegularPolyCollection, PolyCollection
 from colors import colorConverter
 from patches import bbox_artist
@@ -20,7 +20,7 @@ from transforms import lbwh_to_bbox
 from matplotlib import rcParams
 
 TICKLEFT, TICKRIGHT, TICKUP, TICKDOWN = range(4)
-lineStyles  = {'-':1, '--':1, '-.':1, ':':1}
+lineStyles  = {'-':1, '--':1, '-.':1, ':':1, 'None':1}
 lineMarkers =    {'.':1, ',':1, 'o':1, '^':1, 'v':1, '<':1, '>':1, 's':1,
                   '+':1, 'x':1, 'd':1, 'D':1, '|':1, '_':1, 'h':1, 'H':1,
                   'p':1, '1':1, '2':1, '3':1, '4':1,
@@ -28,18 +28,68 @@ lineMarkers =    {'.':1, ',':1, 'o':1, '^':1, 'v':1, '<':1, '>':1, 's':1,
                   TICKRIGHT:1,
                   TICKUP:1,
                   TICKDOWN:1,
+                  'None':1
                   }
 
 class Line2D(Artist):
+    _lineStyles =  {
+        '-'    : '_draw_solid',
+        '--'   : '_draw_dashed', 
+        '-.'   : '_draw_dash_dot', 
+        ':'    : '_draw_dotted',
+        'None' : '_draw_nothing'}
+    
+    _markers =  {
+        '.'  : '_draw_point',
+        ','  : '_draw_pixel', 
+        'o'  : '_draw_circle', 
+        'v'  : '_draw_triangle_down', 
+        '^'  : '_draw_triangle_up', 
+        '<'  : '_draw_triangle_left', 
+        '>'  : '_draw_triangle_right', 
+        '1'  : '_draw_tri_down', 
+        '2'  : '_draw_tri_up', 
+        '3'  : '_draw_tri_left', 
+        '4'  : '_draw_tri_right', 
+        's'  : '_draw_square',
+        'p'  : '_draw_pentagon',
+        'h'  : '_draw_hexagon1',
+        'H'  : '_draw_hexagon2',
+        '+'  : '_draw_plus',
+        'x'  : '_draw_x',
+        'D'  : '_draw_diamond',
+        'd'  : '_draw_thin_diamond',
+        '|'  : '_draw_vline',
+        '_'  : '_draw_hline',
+        TICKLEFT    : '_draw_tickleft', 
+        TICKRIGHT   : '_draw_tickright',
+        TICKUP      : '_draw_tickup',
+        TICKDOWN    : '_draw_tickdown',
+        'None' : '_draw_nothing'
+        }
 
+    _aliases  = (
+        ('antialiased', 'aa'),          
+        ('color', 'c'),  
+        ('linestyle', 'ls'),                                
+        ('linewidth', 'lw'),
+        ('markeredgecolor', 'mec'),
+        ('markeredgewidth', 'mew'),
+        ('markerfacecolor', 'mfc'),
+        ('markersize', 'ms'),
+        )
+        
+        
     def __init__(self, xdata, ydata,
-                 linewidth=rcParams['lines.linewidth'],
-                 linestyle=rcParams['lines.linestyle'],
-                 color=rcParams['lines.color'], 
-                 marker=None,
-                 markersize=rcParams['lines.markersize'],
-                 markeredgecolor=None, markerfacecolor=None,
-                 antialiased=rcParams['lines.antialiased'],
+                 linewidth       = None, # default to rc
+                 linestyle       = None, # default to rc
+                 color           = None, # default to rc
+                 marker          = None, # default to rc
+                 markersize      = None, # default to rc
+                 markeredgewidth = None, # default to rc
+                 markeredgecolor = None, # default to rc
+                 markerfacecolor = None, # default to rc
+                 antialiased     = None, # default to rc
                  ):
         """
         xdata is a sequence of x data
@@ -49,10 +99,30 @@ class Line2D(Artist):
         marker is one of lineMarkers or None
         markersize is the size of the marker in points
         markeredgecolor  and markerfacecolor can be any color arg 
+        markeredgewidth is the size of the markter edge in points
 
         """
         Artist.__init__(self)
         #convert sequences to numeric arrays
+        if not iterable(xdata):
+            raise RuntimeError('xdata must be a sequence')
+        if not iterable(ydata):
+            raise RuntimeError('ydata must be a sequence')
+
+        if linewidth is None   : linewidth=rcParams['lines.linewidth']
+        if linestyle is None   : linestyle=rcParams['lines.linestyle']
+        if marker is None      : marker=rcParams['lines.marker']
+        if color is None       : color=rcParams['lines.color']
+        if markeredgecolor is None :
+            markeredgecolor=rcParams['lines.markeredgecolor']
+        if markerfacecolor is None :
+            markerfacecolor=rcParams['lines.markerfacecolor']
+        if markeredgewidth is None :
+            markeredgewidth=rcParams['lines.markeredgewidth']
+        
+        if markersize is None  : markersize=rcParams['lines.markersize']
+        if antialiased is None : antialiased=rcParams['lines.antialiased']
+
 
         self._linestyle = linestyle
         self._linewidth = linewidth
@@ -61,64 +131,33 @@ class Line2D(Artist):
         self._markersize = markersize
         self._label = ''
         self._dashSeq = None
-        
-        if markeredgecolor is None: markeredgecolor=color
-        
+
+
         self._markerfacecolor = markerfacecolor
         self._markeredgecolor = markeredgecolor
+        self._markeredgewidth = markeredgewidth
 
         self.verticalOffset = None        
         self._useDataClipping = rcParams['lines.data_clipping']
         self.set_data(xdata, ydata)
-        self._lineStyles =  {
-            '-'  : self._draw_solid,
-            '--' : self._draw_dashed, 
-            '-.' : self._draw_dash_dot, 
-            ':'  : self._draw_dotted,
-            None : self._draw_nothing}
-
-        self._markers =  {
-            '.'  : self._draw_point,
-            ','  : self._draw_pixel, 
-            'o'  : self._draw_circle, 
-            'v'  : self._draw_triangle_down, 
-            '^'  : self._draw_triangle_up, 
-            '<'  : self._draw_triangle_left, 
-            '>'  : self._draw_triangle_right, 
-            '1'  : self._draw_tri_down, 
-            '2'  : self._draw_tri_up, 
-            '3'  : self._draw_tri_left, 
-            '4'  : self._draw_tri_right, 
-            's'  : self._draw_square,
-            'p'  : self._draw_pentagon,
-            'h'  : self._draw_hexagon1,
-            'H'  : self._draw_hexagon2,
-            '+'  : self._draw_plus,
-            'x'  : self._draw_x,
-            'D'  : self._draw_diamond,
-            'd'  : self._draw_thin_diamond,
-            '|'  : self._draw_vline,
-            '_'  : self._draw_hline,
-            TICKLEFT    : self._draw_tickleft, 
-            TICKRIGHT   : self._draw_tickright,
-            TICKUP      : self._draw_tickup,
-            TICKDOWN    : self._draw_tickdown,
-            None : self._draw_nothing
-            }
 
         if not self._lineStyles.has_key(linestyle):
-            print >>sys.stderr, 'Unrecognized line style', linestyle
+            print >>sys.stderr, 'Unrecognized line style', linestyle, type(linestyle)
         if not self._markers.has_key(marker):
-            print >>sys.stderr, 'Unrecognized marker style', marker
- 
+            print >>sys.stderr, 'Unrecognized marker style', marker, type(marker)
+
         self.set_marker(marker)
         self._lineFunc = self._lineStyles.get(linestyle, self._draw_nothing)
         self._markerFunc = self._markers.get(marker, self._draw_nothing)
 
-        
+        # set up some abbreviations for easier interactive use
+        for func, abbrev in self._aliases:
+            self.__dict__['set_%s'%abbrev] = getattr(self, 'set_%s'%func)
+            self.__dict__['get_%s'%abbrev] = getattr(self, 'get_%s'%func)        
     def get_window_extent(self, renderer):
         x, y = self._get_numeric_clipped_data_in_range()
 
+
         x, y = self._transform.numerix_x_y(x, y)
         #x, y = self._transform.seq_x_y(x, y)
 
@@ -144,11 +183,18 @@ class Line2D(Artist):
         self._x = asarray(x, Float)
         self._y = asarray(y, Float)
 
+        if len(self._x.shape)>1: self._x = ravel(self._x)
+        if len(self._y.shape)>1: self._y = ravel(self._y)
+
+
         if len(self._y)==1 and len(self._x)>1:
             self._y = self._y*ones(self._x.shape, Float)
 
-        if self._useDataClipping: self._xsorted = self._is_sorted(self._x)
+        if len(self._x) != len(self._y):
+            raise RuntimeError('xdata and ydata must be the same length')
 
+        if self._useDataClipping: self._xsorted = self._is_sorted(self._x)
+        
     def set_data_clipping(self, b):
         'b is a boolean that sets whether data clipping is on'
         self._useDataClipping = b
@@ -177,14 +223,15 @@ class Line2D(Artist):
         return x, y
 
     def draw(self, renderer):
-
+        #renderer.open_group('line2d')
         x, y = self._get_numeric_clipped_data_in_range()
         if len(x)==0: return 
 
+        #xys = self._transform.seq_xy_tups(zip(x,y))
+        #xy, yt = zip(*xys)
+
         xt, yt = self._transform.numerix_x_y(x, y)
-        #xt, yt = self._transform.seq_x_y(x, y)
-
-
+        #if len(xt)>1000: print xt[-5:], yt[-5:]
         gc = renderer.new_gc()
         gc.set_foreground(self._color)
         gc.set_antialiased(self._antialiased)
@@ -194,18 +241,21 @@ class Line2D(Artist):
             gc.set_clip_rectangle(self.clipbox.get_bounds())
 
 
-        self._lineFunc(renderer, gc, xt, yt)
+        lineFunc = getattr(self, self._lineFunc)
+        lineFunc(renderer, gc, xt, yt)
 
 
         if self._marker is not None:
             gc = renderer.new_gc()
             gc.set_foreground(self._markeredgecolor)
-            gc.set_linewidth(self._linewidth)
+            gc.set_linewidth(self._markeredgewidth)
             if self.get_clip_on():
                 gc.set_clip_rectangle(self.clipbox.get_bounds())
-            self._markerFunc(renderer, gc, xt, yt)
+            markerFunc = getattr(self, self._markerFunc)
+            markerFunc(renderer, gc, xt, yt)
 
         #if 1: bbox_artist(self, renderer)
+        #renderer.close_group('line2d')
         
     def get_antialiased(self): return self._antialiased
     def get_color(self): return self._color
@@ -214,6 +264,7 @@ class Line2D(Artist):
     def get_linewidth(self): return self._linewidth
     def get_marker(self): return self._marker
     def get_markeredgecolor(self): return self._markeredgecolor
+    def get_markeredgewidth(self): return self._markeredgewidth
     def get_markerfacecolor(self): return self._markerfacecolor
     def get_markersize(self): return self._markersize
     def get_xdata(self): return self._x
@@ -226,6 +277,8 @@ class Line2D(Artist):
         try: self._xmin, self._xmax
         except AttributeError: indx = arange(len(self._x))
         else:
+            if not hasattr(self, '_xsorted'):
+                self._xsorted = self._is_sorted(self._x)
             if len(self._x)==1:
                 indx = [0]
             elif self._xsorted:
@@ -291,6 +344,9 @@ class Line2D(Artist):
     def set_markeredgecolor(self, ec):
         self._markeredgecolor = ec
 
+    def set_markeredgewidth(self, ew):
+        self._markeredgewidth = ew
+
     def set_markerfacecolor(self, fc):
         self._markerfacecolor = fc
 
@@ -298,6 +354,9 @@ class Line2D(Artist):
         self._markersize = sz
 
     def set_xdata(self, x):
+        try: del self._xsorted
+        except AttributeError: pass
+            
         self.set_data(x, self._y)
 
     def set_ydata(self, y):
@@ -305,6 +364,7 @@ class Line2D(Artist):
         self.set_data(self._x, y)
         
     def set_xclip(self, xmin, xmax):
+        
         if xmax<xmin: xmax, xmin = xmin, xmax
         self._xmin, self._xmax = xmin, xmax
         self._set_clip()
@@ -589,6 +649,7 @@ class Line2D(Artist):
         self._markersize = line._markersize        
         self._markerfacecolor = line._markerfacecolor
         self._markeredgecolor = line._markeredgecolor
+        self._markeredgewidth = line._markeredgewidth
         self._dashSeq = line._dashSeq
 
         self._linestyle = line._linestyle
@@ -598,6 +659,7 @@ class Line2D(Artist):
         self._useDataClipping = line._useDataClipping
 
     def _get_rgb_face(self):
-        if self._markerfacecolor is None: rgbFace = None
+        if (self._markerfacecolor is None or
+            self._markerfacecolor.lower()=='none'): rgbFace = None
         else: rgbFace = colorConverter.to_rgb(self._markerfacecolor)
         return rgbFace
