@@ -11,23 +11,32 @@ Plotting commands
   cla      - clear current axes
   clf      - clear a figure window
   close    - close a figure window
+  colorbar - add a colorbar to the current figure
   cohere   - make a plot of coherence
   csd      - make a plot of cross spectral density
+  draw     - Force a redraw of the current figure
   errorbar - make an errorbar graph
   figlegend - make legend on the figure rather than the axes
+  figimage  - make a figure image
+  figtext   - add text in figure coords
   figure   - create or change active figure
   fill     - make filled polygons
   gca      - return the current axes
   gcf      - return the current figure
+  gci      - get the current image, or None
   get      - get a handle graphics property
+  gray     - set the current colormap to gray
+  jet      - set the current colormap to jet
   hist     - make a histogram
   hold     - set the axes hold state
   legend   - make an axes legend
   loglog   - a log log plot
+  imread   - load image file into array
   imshow   - plot image data
   pcolor   - make a pseudocolor plot
   plot     - make a line plot
   psd      - make a plot of power spectral density
+  rc       - control the default params
   savefig  - save the current figure
   scatter  - make a scatter plot
   set      - set a handle graphics property
@@ -127,6 +136,7 @@ the exception of those in mlab.py provided by matplotlib.
 from numerix import *
 from mlab import *
 
+import cm
 import _matlab_helpers
 import mlab  #so I can override hist, psd, etc...
 
@@ -134,9 +144,14 @@ from axes import Axes
 from backends import new_figure_manager, error_msg, \
      draw_if_interactive, show
 from cbook import flatten, is_string_like, exception_to_str
-from colors import ColormapJet
+from colors import normalize
+from cm import ColormapJet, Grayscale, get_cmap
 from figure import Figure
-from matplotlib import rcParams, get_backend
+import image
+from matplotlib import rcParams, rcParamsDefault, get_backend
+from backend_bases import FigureCanvasBase
+from image import imread as _imread
+import matplotlib
 
 def colors():
     """
@@ -184,23 +199,32 @@ def plotting():
     cla      - clear current axes
     clf      - clear a figure window
     close    - close a figure window
+    colorbar - add a colorbar to the current figure
     cohere   - make a plot of coherence
     csd      - make a plot of cross spectral density
+    draw     - force a redraw of the current figure
     errorbar - make an errorbar graph
     figlegend - add a legend to the figure
+    figimage - add an image to the figure, w/o resampling
+    figtext  - add text in figure coords
     figure   - create or change active figure
     fill     - make filled polygons
     gca      - return the current axes
     gcf      - return the current figure
+    gci      - get the current image, or None
     get      - get a handle graphics property
+    gray     - set the current colormap to gray
+    jet      - set the current colormap to jet
     hist     - make a histogram
     hold     - set the hold state on current axes
     legend   - add a legend to the axes
     loglog   - a log log plot
+    imread   - load image file into array
     imshow   - plot image data
     pcolor   - make a pseudocolor plot
     plot     - make a line plot
     psd      - make a plot of power spectral density
+    rc       - control the default params
     savefig  - save the current figure
     scatter  - make a scatter plot
     set      - set a handle graphics property
@@ -228,12 +252,22 @@ def get_current_fig_manager():
         figManager = _matlab_helpers.Gcf.get_active()
     return figManager
 
-def get_plot_commands(): return ('axes', 'axis', 'bar', 'cla', 'clf',
-'close', 'cohere', 'csd', 'errorbar', 'figlegend', 'figure', 'fill',
-'gca', 'gcf', 'get', 'hist', 'hold', 'imshow', 'legend', 'loglog', 'pcolor',
-'plot', 'psd', 'savefig', 'scatter', 'set', 'semilogx', 'semilogy',
-'show', 'specgram', 'stem', 'subplot', 'table', 'text', 'title',
-'xlabel', 'ylabel')
+def mpl_connect(s, func):
+    return get_current_fig_manager().canvas.mpl_connect(s, func)
+mpl_connect.__doc__ = FigureCanvasBase.mpl_connect.__doc__
+
+def mpl_disconnect(cid):
+    return get_current_fig_manager().canvas.mpl_disconnect(cid)
+mpl_disconnect.__doc__ = FigureCanvasBase.mpl_disconnect.__doc__
+
+def get_plot_commands(): return ( 'axes', 'axis', 'bar', 'cla', 'clf',
+    'close', 'colorbar', 'cohere', 'csd', 'draw', 'errorbar',
+    'figlegend', 'figtext', 'figimage', 'figure', 'fill', 'gca',
+    'gcf', 'gci', 'get', 'gray', 'jet', 'hist', 'hold', 'imread',
+    'imshow', 'legend', 'loglog', 'rc', 'pcolor', 'plot', 'psd',
+    'savefig', 'scatter', 'set', 'semilogx', 'semilogy', 'show',
+    'specgram', 'stem', 'subplot', 'table', 'text', 'title', 'xlabel',
+    'ylabel')
 
 def raise_msg_to_str(msg):
     """msg is a return arg from a raise.  Join with new lines"""
@@ -281,8 +315,8 @@ def axes(*args, **kwargs):
 
       axes() by itself creates a default full subplot(111) window axis
 
-      axes(rect, axisbg='w') where rect=[left, bottom, width, height] in
-        normalized (0,1) units background is the background color for
+      axes(rect, axisbg='w') where rect=[left, bottom, width, height]
+        in normalized (0,1) units.  axisbg is the background color for
         the axis, default white
 
       axes(h) where h is an axes instance makes h the
@@ -317,34 +351,6 @@ def axes(*args, **kwargs):
     return ret
 
 def bar(*args, **kwargs):
-    """
-    BAR(left, height)
-
-    Make a bar plot with rectangles at
-      left, left+width, 0, height
-    left and height are Numeric arrays
-
-    Return value is a list of Rectangle patch instances
-
-    BAR(left, height, width, bottom,
-        color, yerr, xerr, capsize, yoff)
-
-    xerr and yerr, if not None, will be used to generate errorbars
-    on the bar chart
-
-    color specifies the color of the bar
-    ecolor specifies the color of any errorbar
-
-    capsize determines the length in points of the error bar caps
-
-
-    The optional arguments color, width and bottom can be either
-    scalars or len(x) sequences
-
-    This enables you to use bar as the basis for stacked bar
-    charts, or candlestick plots
-    """
-    
     try: patches =  gca().bar(*args, **kwargs)
     except Exception, msg:
         s = exception_to_str(msg)
@@ -352,7 +358,47 @@ def bar(*args, **kwargs):
         raise RuntimeError(msg)
     draw_if_interactive()
     return patches
+bar.__doc__ = Axes.bar.__doc__
 
+def _get_target_images(target=None):
+    if target is None:
+        ax = gca()
+        all = ax.get_images()
+
+        if not len(all):
+            error_msg('You must first define an image, eg with imshow')
+            return
+
+        images = [all[-1]]
+    else:
+        if iterable(target): images = target
+        else: images = [target]
+
+    return images
+
+def clim(vmin=None, vmax=None):
+    """
+    Set the color limits of the current image
+
+    To apply clim to all axes images do
+
+    clim(0, 0.5)
+
+    If either vmin or vmax is None, the image min/max respectively
+    will be used for color scaling.
+
+    If you want to set the clim of multiple images,
+    use, for example for im in gca().get_images(): im.set_clim(0,
+    0.05)
+    
+    """  
+    im = gci._current
+    if im is None:
+        error_msg('You must first define an image, eg with imshow')
+        return
+    im.set_clim(vmin, vmax)
+    draw_if_interactive()
+    
 def close(*args):
     """
     Close a figure window
@@ -389,6 +435,35 @@ def close(*args):
         popup_err(msg)
 
 
+
+def errorbar(x, y, yerr=None, xerr=None, fmt='b-', ecolor='k', capsize=3):
+    """
+    Plot x versus y with error deltas in yerr and xerr.
+    Vertical errorbars are plotted if yerr is not None
+    Horizontal errorbars are plotted if xerr is not None
+
+    xerr and yerr may be any of:
+        a rank-0, Nx1 Numpy array  - symmetric errorbars +/- value
+        an N-element list or tuple - symmetric errorbars +/- value
+        a rank-1, Nx2 Numpy array  - asymmetric errorbars -column1/+column2
+        
+    fmt is the plot format symbol for y
+    ecolor is the errorbar color specifier
+
+    Return value is a length 2 tuple.  The first element is a list of
+    y symbol lines.  The second element is a list of error bar lines.
+
+    capsize is the size of the error bar caps in points """
+
+    try: ret =  gca().errorbar(x, y, yerr, xerr, fmt, ecolor, capsize)
+    except ValueError, msg:
+        msg = raise_msg_to_str(msg)
+        error_msg(msg)
+    else:
+        draw_if_interactive()
+        return ret
+
+    
 def cla():
     """
     Clear the current axes
@@ -404,6 +479,57 @@ def clf():
     manager.clf()
     draw_if_interactive()
 
+def colorbar(tickfmt='%1.1f'):
+    """
+    Create a colorbar for current mappable image (see gci)
+
+    tickfmt is a format string to format the colorbar ticks
+
+    return value is the colorbar axes instance
+    """
+
+    mappable = gci()
+    if mappable is None:
+        error_msg('First define a mappable image (eg imshow, figimage, pcolor, scatter')
+        return
+
+    if isinstance(mappable, image.FigureImage):
+        error_msg('Colorbars for figure images currently not supported')
+        return
+        
+    ax = gca()
+
+    cmap = mappable.cmap
+    norm = mappable.norm
+    
+    if norm.vmin is None or norm.vmax is None:
+        mappable.autoscale()
+    cmin = norm.vmin
+    cmax = norm.vmax
+    l,b,w,h = ax.get_position()
+
+    neww = 0.8*w
+    ax.set_position((l,b,neww,h))
+    cax = axes([l + 0.9*w, b, 0.1*w, h])
+    N = 200
+    c = linspace(cmin, cmax,N)
+    C = array([c,c])
+    coll = cax.imshow(transpose(C), interpolation='nearest',
+                      origin='lower',
+                      cmap=cmap, norm=norm)
+    mappable.add_observer(coll)
+    
+    cax.set_xticks([])
+    cax.yaxis.tick_right()
+    ticks = arange(0,N+1,20)
+    cax.set_yticks(ticks)
+
+    cticks = linspace(cmin, cmax, len(ticks))
+    cax.set_yticklabels([tickfmt%tick for tick in cticks])
+
+    # restore the current axes
+    axes(ax)
+    return cax
 
 def cohere(x, y, NFFT=256, Fs=2, detrend=mlab.detrend_none,
            window=mlab.window_hanning, noverlap=0):
@@ -470,33 +596,36 @@ def csd(x, y, NFFT=256, Fs=2, detrend=mlab.detrend_none,
         draw_if_interactive()
         return ret
 
-def errorbar(x, y, yerr=None, xerr=None, fmt='b-', ecolor='k', capsize=3):
-    """
-    Plot x versus y with error deltas in yerr and xerr.
-    Vertical errorbars are plotted if yerr is not None
-    Horizontal errorbars are plotted if xerr is not None
 
-    xerr and yerr may be any of:
-        a rank-0, Nx1 Numpy array  - symmetric errorbars +/- value
-        an N-element list or tuple - symmetric errorbars +/- value
-        a rank-1, Nx2 Numpy array  - asymmetric errorbars -column1/+column2
-        
-    fmt is the plot format symbol for y
-    ecolor is the errorbar color specifier
+def draw():
+    'redraw the current figure'
+    get_current_fig_manager().canvas.draw()
+    
+def figtext(*args, **kwargs):    
+    try: t =  gcf().text(*args, **kwargs)
+    except RuntimeError, msg:
+        msg = raise_msg_to_str(msg)
+        error_msg(msg)
+        raise RuntimeError(msg)
+    else:
+        draw_if_interactive()
+        return t
+figtext.__doc__ = Figure.text.__doc__
 
-    Return value is a length 2 tuple.  The first element is a list of
-    y symbol lines.  The second element is a list of error bar lines.
-
-    capsize is the size of the error bar caps in points
-    """
-    try: ret =  gca().errorbar(x, y, yerr, xerr, fmt, ecolor, capsize)
+def figimage(*args, **kwargs):    
+    try: im =  gcf().figimage(*args, **kwargs)
     except ValueError, msg:
         msg = raise_msg_to_str(msg)
         error_msg(msg)
-    else:
-        draw_if_interactive()
-        return ret
-
+        raise RuntimeError(msg)
+    except RuntimeError, msg:
+        msg = raise_msg_to_str(msg)
+        error_msg(msg)
+        raise RuntimeError(msg)
+    draw_if_interactive()
+    gci._current = im
+    return im
+figimage.__doc__ = Figure.figimage.__doc__
     
 def figlegend(handles, labels, loc):
     """
@@ -511,6 +640,8 @@ def figlegend(handles, labels, loc):
               'upper right')
 
     See help(legend) for information about the location codes
+
+    A matplotlib.legend.Legend instance is returned
     """
     l=  gcf().legend(handles, labels, loc)
     draw_if_interactive()
@@ -518,10 +649,10 @@ def figlegend(handles, labels, loc):
     
 
 def figure(num=1,
-           figsize   = rcParams['figure.figsize'],
-           dpi       = rcParams['figure.dpi'],
-           facecolor = rcParams['figure.facecolor'],
-           edgecolor = rcParams['figure.edgecolor'],
+           figsize   = None, # defaults to rc figure.figsize
+           dpi       = None, # defaults to rc figure.dpi
+           facecolor = None, # defaults to rc figure.facecolor
+           edgecolor = None, # defaults to rc figure.edgecolor
            frameon = True,
            ):
     """
@@ -535,14 +666,21 @@ def figure(num=1,
 
       figure(1)
 
-    figsize - width in height x inches
-    dpi     - resolution
-    facecolor - the background color    
-    edgecolor - the border color
+    figsize - width in height x inches; defaults to rc figure.figsize
+    dpi     - resolution; defaults to rc figure.dpi
+    facecolor - the background color; defaults to rc figure.facecolor
+    edgecolor - the border color; defaults to rc figure.edgecolor
 
     rcParams gives the default values from the .matplotlibrc file
 
     """
+
+    if figsize is None   : figsize   = rcParams['figure.figsize']
+    if dpi is None       : dpi       = rcParams['figure.dpi']
+    if facecolor is None : facecolor = rcParams['figure.facecolor']
+    if edgecolor is None : edgecolor = rcParams['figure.edgecolor']
+
+
     if num==0:
         error_msg('Figure number can not be 0.\n' + \
                   'Hey, give me a break, this is matlab compatability')
@@ -632,6 +770,17 @@ def gcf():
     else:
         return figure()
 
+def gci():
+    """
+    get the current ScalarMappable instance (image or patch
+    collection), or None if no images or patch collecitons have been
+    defined.  The commands imshow and figimage create images
+    instances, and the commands pcolor and scatter create patch
+    collection instances
+    """
+    return gci._current
+gci._current = None
+
 def get(o, s):
     """
     Return the value of handle property s
@@ -646,12 +795,30 @@ def get(o, s):
     return eval(func, {}, {'o': o})
     
 
-def grid(b):
+def grid(b=None):
     """
     Set the figure grid to be on or off (b is a boolean)
+
+    if b is None, toggle the grid state
     """
     gca().grid(b)
     draw_if_interactive()
+
+def gray():
+    'set the default colormap to gray and apply to current image if any'
+    rc('image', cmap='gray')
+    im = gci()
+    if im is not None:
+        im.set_cmap(cm.gray)
+        draw_if_interactive()
+        
+def jet():
+    'set the default colormap to jet and apply to current image if any'
+    rc('image', cmap='jet')
+    im = gci()
+    if im is not None:
+        im.set_cmap(cm.jet)
+    draw_if_interactive()
     
 def hist(x, bins=10, noplot=0, normed=0, bottom=0):
     """
@@ -668,6 +835,15 @@ def hist(x, bins=10, noplot=0, normed=0, bottom=0):
     counts normalized to form a probability distribtion, ie,
     n/(len(x)*dbin)
     
+
+    To control the properties of the returned patches, you can can
+    call any of the patch methods on those patches; see
+    matplotlib.patches and matplotlib.artist (the base class for
+    patches).  Eg
+
+    n, bins, patches = hist(x, 50, normed=1)
+    set(patches, 'facecolor', 'g', 'alpha', 0.75)
+
     """
     if noplot: return mlab.hist(x, bins, normed)
     else:
@@ -691,7 +867,10 @@ def hold(b=None):
     hold(True)  # hold is on
     hold(False) # hold is off
     """
-    gca().hold(b)
+    gcf().hold(b)    
+    gca().hold(b)    
+    rc('axes', hold=b)
+
     
 def hlines(*args, **kwargs):    
     """
@@ -713,26 +892,11 @@ def hlines(*args, **kwargs):
     draw_if_interactive()
     return lines
 
+def imread(*args, **kwargs):
+    return _imread(*args, **kwargs)
+imread.__doc__ = _imread.__doc__
 
 def imshow(*args, **kwargs):
-    """
-    Display the image in array X to current axes.  X must be a
-    float array
-
-    Usage: 
-
-     IMSHOW(X)
-    
-     If X is MxN, assume luminance (grayscale)
-     If X is MxNx3, assume RGB
-     If X is MxNx4, assume RGBA
-
-     IMSHOW(X, cmap)
-     cmap is a colors.Colormap instance used to make a pseudo-color plot
-    
-    An Image instance is returned
-    """
-
     try: im =  gca().imshow(*args, **kwargs)
     except ValueError, msg:
         msg = raise_msg_to_str(msg)
@@ -742,10 +906,12 @@ def imshow(*args, **kwargs):
         msg = raise_msg_to_str(msg)
         error_msg(msg)
         raise RuntimeError(msg)
-    draw_if_interactive()
+    else:
+        draw_if_interactive()
+    gci._current = im
 
     return im
-
+imshow.__doc__ = Axes.imshow.__doc__
 
 def legend(*args, **kwargs):
     """
@@ -826,7 +992,7 @@ def load(fname):
 
     if is_string_like(fname):
         fh = file(fname)
-    elif hasattr(fname, seek):
+    elif hasattr(fname, 'seek'):
         fh = fname
     else:
         raise ValueError('fname must be a string or file handle')
@@ -868,48 +1034,11 @@ def loglog(*args, **kwargs):
 
 
 def pcolor(*args, **kwargs):
-    """
-    pcolor_patch(C) - make a pseudocolor plot of matrix C
-
-    pcolor(X, Y, C) - a pseudo color plot of C on the matrices X and Y  
-
-    Shading:
-
-      The optional keyword arg shading ('flat' or 'faceted') will
-      determine whether the black grid is drawn around each pcolor
-      square.  Defaul 'faceteted'
-         e.g.,   
-         pcolor(C, shading='flat')  
-         pcolor(X, Y, C, shading='faceted')
-
-    Return value is a matplotlib.collections.PolyCollection object
-
-    Note, the behavior of meshgrid in matlab is a bit
-    counterintuitive for x and y arrays.  For example,
-
-      x = arange(7)
-      y = arange(5)
-      X, Y = meshgrid(x,y)
-
-      Z = rand( len(x), len(y))
-      pcolor(X, Y, Z)
-
-    will fail in matlab and matplotlib.  You will probably be
-    happy with
-
-      pcolor(X, Y, transpose(Z))
-
-    Likewise, for nonsquare Z,
-
-      pcolor(transpose(Z))
-
-    will make the x and y axes in the plot agree with the numrows
-    and numcols of Z
-
-    """
     ret = gca().pcolor(*args, **kwargs)
+    gci._current = ret    
     draw_if_interactive()
     return ret
+pcolor.__doc__ = Axes.pcolor.__doc__
 
 def pcolor_classic(*args, **kwargs):
     """
@@ -928,6 +1057,18 @@ def pcolor_classic(*args, **kwargs):
 
     returns a list of patch objects.
 
+
+    pcolor(C, cmap=cm.jet) - make a pseudocolor plot of matrix C using
+    rectangle patches using a colormap jet.  Colormaps are avalible in
+    matplotlib.cm.  You must pass this as a kwarg.
+        
+    pcolor(C, norm=normalize()) - the normalization function used to
+    scale your color data to 0-1.  must be passed as a kwarg.
+    normalization functions are derived from matplotlib.colors.Norm
+
+    pcolor(C, alpha=0.5) - set the alpha of the pseudocolor plot.
+    Must be used as a kwarg
+
     Note, the behavior of meshgrid in matlab is a bit
     counterintuitive for x and y arrays.  For example,
 
@@ -1030,8 +1171,9 @@ def plot(*args, **kwargs):
 
     Neither line will be antialiased.
     """
-    
-    try: lines =  gca().plot(*args, **kwargs)
+
+    try:
+        lines =  gca().plot(*args, **kwargs)
     except ValueError, msg:
         msg = raise_msg_to_str(msg)
         error_msg(msg)
@@ -1103,7 +1245,17 @@ def psd(x, NFFT=256, Fs=2, detrend=mlab.detrend_none,
         draw_if_interactive()
         return ret
     
-        
+
+def rc(*args, **kwargs):
+    matplotlib.rc(*args, **kwargs)
+    draw_if_interactive()
+rc.__doc__ =   matplotlib.rc.__doc__
+
+def rcdefaults():
+    matplotlib.rcdefaults()
+    draw_if_interactive()
+rcdefaults.__doc__ =   matplotlib.rcdefaults.__doc__
+
 def save(fname, X, fmt='%1.4f'):
     """
     Save the data in X to file fname using fmt string to convert the
@@ -1122,7 +1274,7 @@ def save(fname, X, fmt='%1.4f'):
 
     if is_string_like(fname):
         fh = file(fname, 'w')
-    elif hasattr(fname, seek):
+    elif hasattr(fname, 'seek'):
         fh = fname
     else:
         raise ValueError('fname must be a string or file handle')
@@ -1162,27 +1314,16 @@ def savefig(*args, **kwargs):
     manager.canvas.print_figure(*args, **kwargs)
 
 def scatter(*args, **kwargs):
-    """
-
-    scatter(self, x, y, s=None, c='b'):
-
-    Make a scatter plot of x versus y.  s is a size (in data
-    coords) and can be either a scalar or an array of the same
-    length as x or y.  c is a color and can be a single color
-    format string or an length(x) array of intensities which will
-    be mapped by the colormap jet.        
-
-    If size is None a default size will be used
-    """
-
-    try: patches =  gca().scatter(*args, **kwargs)
+    try: coll =  gca().scatter(*args, **kwargs)
     except ValueError, msg:
         msg = raise_msg_to_str(msg)
         error_msg(msg)
         raise RuntimeError, msg
+    
     draw_if_interactive()
-    return patches
-
+    gci._current = coll
+    return coll
+scatter.__doc__ = Axes.scatter.__doc__
 
 def scatter_classic(*args, **kwargs):
     """
@@ -1214,16 +1355,13 @@ def semilogx(*args, **kwargs):
     semilog x are the same as the args to plot.  See help plot for
     more info    
     """
-    ret =  gca().semilogx(*args, **kwargs)
-    return ret
-    """
+    try: ret =  gca().semilogx(*args, **kwargs)
     except ValueError, msg:
         msg = raise_msg_to_str(msg)
         error_msg(msg)
     else:
         draw_if_interactive()
         return ret
-    """
 
 def semilogy(*args, **kwargs):
     """
@@ -1286,33 +1424,18 @@ def set(h, *args, **kwargs):
     return [x for x in flatten(ret)]
 
 
-def specgram(x, NFFT=256, Fs=2, detrend=mlab.detrend_none,
-             window=mlab.window_hanning, noverlap=128,
-             cmap = ColormapJet(256)):
-    """
-    Compute a spectrogram of data in x.  Data are split into NFFT
-    length segements and the PSD of each section is computed.  The
-    windowing function window is applied to each segment, and the
-    amount of overlap of each segment is specified with noverlap
+def specgram(*args, **kwargs):
 
-    cmap is a a matplotlib.colors.ColorMap
-    
-    See help(psd) for information on the other arguments
-
-    return value is Pxx, freqs, bins, im
-
-    bins are the time points the spectrogram is calculated over
-    freqs is an array of frequencies
-    Pxx is a len(times) x len(freqs) array of power
-    im is a matplotlib image
-    """
-    try: ret =  gca().specgram(x, NFFT, Fs, detrend, window, noverlap, cmap)
+    try: ret =  gca().specgram(*args, **kwargs)
     except ValueError, msg:
         msg = raise_msg_to_str(msg)
         error_msg(msg)
     else:
+        Pxx, freqs, bins, im = ret
+        gci._current = im
         draw_if_interactive()
         return ret
+specgram.__doc__ = Axes.specgram.__doc__
 
 def stem(*args, **kwargs):
     """
@@ -1471,7 +1594,7 @@ def vlines(*args, **kwargs):
         msg = raise_msg_to_str(msg)
         error_msg(msg)
         raise RuntimeError, msg
-    draw_if_interactive()
+    else:  draw_if_interactive()
     return lines
 
 def xlabel(s, *args, **kwargs):
