@@ -7,6 +7,8 @@ commands with the same names.
 
     * cohere - Coherence (normalized cross spectral density)
 
+    * conv     - convolution
+    
     * corrcoef - The matrix of correlation coefficients
 
     * csd - Cross spectral density uing Welch's average periodogram
@@ -84,6 +86,10 @@ def window_none(x):
     "No window function; simply return x"
     return x
 
+def conv(x, y, mode=2):
+    'convolve x with y'
+    return convolve(x,y,mode)
+
 def detrend(x, key=None):
     if key is None or key=='constant':
         return detrend_mean(x)
@@ -104,7 +110,7 @@ def detrend_linear(x):
     # I'm going to regress x on xx=range(len(x)) and return x -
     # (b*xx+a).  Now that I have polyfit working, I could convert the
     # code here, but if it ain't broke, don't fix it!
-    xx = arange(len(x), typecode=x.typecode())
+    xx = arange(len(x), x.typecode())
     X = transpose(array([xx]+[x]))
     C = cov(X)
     b = C[0,1]/C[0,0]
@@ -573,12 +579,17 @@ def hist(y, bins=10, normed=0):
     If normed is False, return the counts in the first element of the
     return tuple.  If normed is True, return the probability density
     n/(len(y)*dbin)
-    
+
+    If y has rank>1, it will be raveled
     Credits: the Numeric 22 documentation
 
+    
 
     """
-    if not iterable(bins):
+    y = asarray(y)
+    if len(y.shape)>1: y = ravel(y)
+
+    if not iterable(bins):       
         ymin, ymax = min(y), max(y)
         if ymin==ymax:
             ymin -= 0.5
@@ -613,10 +624,10 @@ def levypdf(x, gamma, alpha):
    dx = x[1]-x[0]
 
    Nyq = 1/(2*dx)
-   f = 1/(N*dx)*arange(-N/2, N/2, typecode=Float)
+   f = 1/(N*dx)*arange(-N/2, N/2, Float)
 
-   ind = concatenate([arange(N/2, N, typecode=Int),
-                      arange(N/2,typecode=Int)])
+   ind = concatenate([arange(N/2, N, Int),
+                      arange(N/2,Int)])
    df = f[1]-f[0]
    cfl = exp(-gamma*absolute(2*pi*f)**alpha)
 
@@ -1101,3 +1112,17 @@ def specgram(x, NFFT=256, Fs=2, detrend=detrend_none,
     freqs = Fs/NFFT*arange(numFreqs)
 
     return Pxx, freqs, t
+
+def bivariate_normal(X, Y, sigmax=1.0, sigmay=1.0,
+                     mux=0.0, muy=0.0, sigmaxy=0.0):
+    """
+    Bivariate gaussan distribution for equal shape X, Y
+
+    http://mathworld.wolfram.com/BivariateNormalDistribution.html
+    """
+    Xmu = X-mux
+    Ymu = Y-muy
+
+    rho = sigmaxy/(sigmax*sigmay)
+    z = Xmu**2/sigmax**2 + Ymu**2/sigmay - 2*rho*Xmu*Ymu/(sigmax*sigmay)
+    return 1.0/(2*pi*sigmax*sigmay*(1-rho**2)) * exp( -z/(2*(1-rho**2)))
