@@ -212,7 +212,7 @@ class Table(Artist):
         return self.FONTSIZE/72.0*self.figure.dpi.get()/self._axes.bbox.height() * 1.2
             
     def draw(self, renderer):
-
+        renderer.open_group('table')
         self._update_positions(renderer)
 
         keys = self._cells.keys()
@@ -221,6 +221,7 @@ class Table(Artist):
             self._cells[key].draw(renderer)
         #for c in self._cells.itervalues():
         #    c.draw(renderer)
+        renderer.close_group('table')
 
     def _get_grid_bbox(self, renderer):
         """Get a bbox, in axes co-ordinates for the cells.
