@@ -30,16 +30,20 @@ class Text(Artist):
     cached = {}
     def __init__(self, 
                  x=0, y=0, text='',
-                 color=rcParams['text.color'],
+                 color=None,          # defaults to rc params
                  verticalalignment='bottom',
                  horizontalalignment='left',
                  multialignment=None,
-                 fontproperties=FontProperties(),
+                 fontproperties=None, # defaults to FontProperties()
                  rotation=None,
                  ):
+        # small leak here due to self._transform
         Artist.__init__(self)
         self._x, self._y = x, y
-        
+
+        if color is None: color = rcParams['text.color']
+        if fontproperties is None: fontproperties=FontProperties()
+
         self._color = color
         self._text = text
         self._verticalalignment = verticalalignment
@@ -50,6 +54,7 @@ class Text(Artist):
         
         self._rotation = rotation
 
+        #small leak here
         self._fontproperties = fontproperties
 
     def _get_multialignment(self):
@@ -69,7 +74,8 @@ class Text(Artist):
     
     def copy_properties(self, t):
         'Copy properties from t to self'
-        self._color = t._color,
+
+        self._color = t._color
         self._multialignment = t._multialignment        
         self._verticalalignment = t._verticalalignment
         self._horizontalalignment = t._horizontalalignment
@@ -81,7 +87,7 @@ class Text(Artist):
 
         # layout the xylocs in display coords as if angle = zero and
         # then rotate them around self._x, self._y
-
+        
         key = self.get_prop_tup()
         if self.cached.has_key(key): return self.cached[key]
         
@@ -101,6 +107,11 @@ class Text(Artist):
         for line in lines:
             w,h = renderer.get_text_width_height(
                 line, self._fontproperties, ismath=self.is_math_text())
+            if not len(line) and not self.is_math_text():
+                # approx the height of empty line with tall char
+                tmp, h = renderer.get_text_width_height(
+                'T', self._fontproperties, ismath=False)
+
             whs.append( (w,h) )
             offsety = h+pad
             horizLayout.append((line, thisx, thisy, w, h))
@@ -190,12 +201,13 @@ class Text(Artist):
         gc.set_foreground(self._color)
         gc.set_alpha(self._alpha)
         if self.get_clip_on():
-            gc.set_clip_rectangle(self.bbox)
+            gc.set_clip_rectangle(self.clipbox.get_bounds())
 
 
         if 0: bbox_artist(self, renderer)
         angle = self.get_angle()
         bbox, info = self._get_layout(renderer)
+
         for line, wh, x, y in info:
             x, y = self._transform.xy_tup((x, y))
             #renderer.draw_arc(gc, (1,0,0),
@@ -204,6 +216,7 @@ class Text(Artist):
             if renderer.flipy():
                 canvasw, canvash = renderer.get_canvas_width_height()
                 y = canvash-y
+            
             renderer.draw_text(gc, x, y, line,
                                self._fontproperties, angle,
                                ismath=self.is_math_text())
@@ -272,15 +285,14 @@ class Text(Artist):
         return self._verticalalignment
 
     def get_window_extent(self, renderer):
-
         bbox, info = self._get_layout(renderer)
         return bbox
 
             
 
     def get_rotation_matrix(self, x0, y0):
-        if self.is_math_text(): theta = 0
-        else: theta = -pi/180.0*self.get_angle()
+
+        theta = -pi/180.0*self.get_angle()
         # translate x0,y0 to origin
         Torigin = Matrix([ [1, 0, -x0],
                            [0, 1, -y0],
@@ -316,8 +328,7 @@ class Text(Artist):
         legal = ('center', 'right', 'left')
         if align not in legal:
             raise ValueError('Horizontal alignment must be one of %s' % str(legal))
-        if self._horizontalalignment == align: return     
-        self._horizontalalignment = align
+        self._horizontalalignment = align     
 
     def set_multialignment(self, align):
         """
@@ -334,22 +345,52 @@ class Text(Artist):
 
     def set_family(self, fontname):
         """
-        Set the font name, eg, 'Sans', 'Courier', 'Helvetica'
+        Set the font family, eg, 'sans-serif', 'cursive', 'fantasy'
         """
         self._fontproperties.set_family(fontname)
 
+    def set_variant(self, variant):
+        """
+        Set the font variant, eg, 'normal', 'small-caps'
+        """
+        self._fontproperties.set_variant(variant)
+
+    def set_name(self, fontname):
+        """
+        Set the font name, eg, 'Sans', 'Courier', 'Helvetica'
+        """
+        self._fontproperties.set_name(fontname)
+
     def set_fontname(self, fontname):
         """
         Set the font name, eg, 'Sans', 'Courier', 'Helvetica'
         """
         self._fontproperties.set_name(fontname)
-        
+
+    def set_style(self, fontstyle):
+        """
+        Set the font style, one of 'normal', 'italic', 'oblique'
+        """
+        self._fontproperties.set_style(fontstyle)
+
     def set_fontstyle(self, fontstyle):
         """
         Set the font style, one of 'normal', 'italic', 'oblique'
         """
         self._fontproperties.set_style(fontstyle)
 
+    def size(self, size):
+        """
+        Set the font size, eg, 8, 10, 12, 14...
+        """
+        self._fontproperties.set_size(size)
+
+    def set_size(self, fontsize):
+        """
+        Set the font size, eg, 8, 10, 12, 14...
+        """
+        self._fontproperties.set_size(fontsize)
+
     def set_fontsize(self, fontsize):
         """
         Set the font size, eg, 8, 10, 12, 14...
@@ -362,6 +403,13 @@ class Text(Artist):
         'normal', 'bold', 'heavy', 'light', 'ultrabold',  'ultralight'
         """
         self._fontproperties.set_weight(weight)
+
+    def set_weight(self, weight):
+        """
+        Set the font weight, one of:
+        'normal', 'bold', 'heavy', 'light', 'ultrabold',  'ultralight'
+        """
+        self._fontproperties.set_weight(weight)
         
     def set_fontangle(self, style):
         """
@@ -412,7 +460,8 @@ class Text(Artist):
         legal = ('fontproperties', 'family', 'color',
                  'verticalalignment', 'horizontalalignment', 'multialignment',
                  'fontname', 'fontstyle', 'fontangle', 'fontvariant',
-                 'fontweight', 'fontsize', 'rotation', 'transform',)
+                 'fontweight', 'fontsize', 'rotation', 'transform',
+                 'size', 'weight', 'style', 'name', 'variant', 'x', 'y')
 
         for k in d.keys():
             if k not in legal: raise KeyError(
