@@ -47,7 +47,11 @@ There are a number of locators specialized for date locations
   * HourLocator    - locate hours that are a multiple of base
 
   * DayLocator     - locate a given hour each day
+  
+  * DayMultiLocator - Make ticks on day which are multiples of base
 
+  * WeekMultiLocator - Make ticks on weeks which are multiples of base
+  
   * WeekdayLocator - locate a given weekday each week
 
   * MonthLocator   - locate months that are multiples of base
@@ -205,19 +209,23 @@ class IndexDateFormatter(Formatter):
     """
     Use with IndexLocator to cycle format strings by index.  
     """
-    def __init__(self, t, fmt, converter):
+    def __init__(self, t, fmt, converter=None):
         """
         t is a sequence of epoch dates.  fmt is a strftime format string
+
+        converter is a matplotlib.dates.DateConverter.  If your
+        datetimes are aready epoch, use None
         """
         self.t = t
         self.fmt = fmt
         self.converter = converter
 
     def __call__(self, x, pos):
-        'Return the label for time x'
+        'Return the label for time x at position pos'
         ind = int(x)
-        if ind>=len(t) or ind<=0: return ''
-        e = converter.epoch(t[ind])
+        if ind>=len(self.t) or ind<=0: return ''
+        if self.converter is not None: e = self.converter.epoch(self.t[ind])
+        else: e = self.t[ind]
         return time.strftime(self.fmt, time.localtime(e))
 
 
@@ -251,7 +259,7 @@ class ScalarFormatter(Formatter):
         # use exponential formatting for really big or small numbers,
         # else use float
 
-        if d < 1e-2: fmt = '%1.2e'
+        if d < 1e-2: fmt = '%1.3f'
         elif d < 1e-1: fmt = '%1.3f'
         elif d > 1e5: fmt = '%1.1e'
         elif d > 10 : fmt = '%1.1f'
@@ -282,6 +290,7 @@ class LogFormatter(ScalarFormatter):
         fx = math.log10(x)
         isdecade = abs(fx-long(fx))<1e-10
         if not isdecade: s = ''
+        #if 0: pass
         elif x>10000: s= '%1.0e'%x
         #elif x<1: s = '$10^{%d}$'%fx
         #elif x<1: s =  '10^%d'%fx
@@ -317,10 +326,9 @@ class Locator(TickHelper):
         raise NotImplementedError('Derived must override')
 
     def autoscale(self):
-        'Try to choose the view limits intelligently'
-        raise NotImplementedError('Derived must override')
-
-
+        'autoscale the view limits'
+        self.verify_intervals()
+        return  self.nonsingular(*self.dataInterval.get_bounds())
 
     def pan(self, numsteps):
         'Pan numticks (can be positive or negative)'
@@ -341,6 +349,20 @@ class Locator(TickHelper):
         step = 0.1*interval*direction
         self.viewInterval.set_bounds(vmin + step, vmax - step)
 
+    def nonsingular(self, vmin, vmax):
+        if vmin==vmax:
+            if vmin==0.0:
+                vmin -= 1
+                vmax += 1
+            else:
+                vmin -= 0.001*abs(vmin)
+                vmax += 0.001*abs(vmax)
+        return vmin, vmax
+
+    def refresh(self):
+        'refresh internal information based on current lim'
+        pass
+
 
 class IndexLocator(Locator):
     """
@@ -357,13 +379,8 @@ class IndexLocator(Locator):
     def __call__(self):
         'Return the locations of the ticks'
         dmin, dmax = self.dataInterval.get_bounds()
-        return arange(offset, dmax+1, base)
+        return arange(dmin + self.offset, dmax+1, self.base)
 
-    def autoscale(self):
-        'autoscale the view limits'
-        self.verify_intervals()
-
-        return  self.dataInterval.get_bounds()
 
 class FixedLocator(Locator):
     """
@@ -377,6 +394,7 @@ class FixedLocator(Locator):
         'Return the locations of the ticks'
         return self.locs
 
+
 class NullLocator(Locator):
     """
     No ticks
@@ -453,7 +471,7 @@ class LinearLocator(Locator):
         vmin = math.floor(scale*vmin)/scale
         vmax = math.ceil(scale*vmax)/scale
 
-        return vmin, vmax
+        return self.nonsingular(vmin, vmax)
 
 
 def closeto(x,y):
@@ -532,13 +550,16 @@ class MultipleLocator(Locator):
 
         self.verify_intervals()
         dmin, dmax = self.dataInterval.get_bounds()
+        #print 'data limits', dmin, dmax, (dmax-dmin)/60.0
         vmin = self.base.le(dmin)
-        vmax = self.base.ge(dmax)        
+        vmax = self.base.ge(dmax)
+        #print 'base rounded', vmin, vmax, (vmax-vmin)/60.0
         if vmin==vmax:
             vmin -=1
             vmax +=1
 
-        return vmin, vmax
+        #print 'mult locator returning', vmin, vmax, (vmax-vmin)/60.0
+        return self.nonsingular(vmin, vmax)
 
 
 
@@ -548,6 +569,7 @@ class MultipleLocator(Locator):
 
 def decade_down(x):
     'floor x to the nearest lower decade'
+
     lx = math.floor(math.log10(x))
     return 10**lx
 
@@ -556,11 +578,16 @@ def decade_up(x):
     lx = math.ceil(math.log10(x))
     return 10**lx
 
+def is_decade(x):
+    lx = math.log10(x)
+    return lx==int(lx)
+
 class LogLocator(Locator):
     """
     Determine the tick locations for log axes
     """
 
+    
     def __call__(self):
         'Return the locations of the ticks'
         self.verify_intervals()
@@ -575,9 +602,13 @@ class LogLocator(Locator):
             ticklocs.append(decadeStart)
             ticklocs.extend( arange(2,10)*decadeStart )
             ticklocs.append(10*decadeStart)
+
+            
         ticklocs = array(ticklocs)
         ind = nonzero(logical_and(ticklocs>=10.0**vmin ,
                                   ticklocs<=10.0**vmax))
+
+        
         ticklocs = take(ticklocs,ind)
         return ticklocs
 
@@ -589,12 +620,16 @@ class LogLocator(Locator):
         vmin, vmax = self.dataInterval.get_bounds()
         if vmax<vmin:
             vmin, vmax = vmax, vmin
+
         
+        
+        if not is_decade(vmin): vmin = decade_down(vmin)
+        if not is_decade(vmax): vmax = decade_up(vmax)
         if vmin==vmax:
             vmin = decade_down(vmin)
             vmax = decade_up(vmax)
-
-        return vmin, vmax
+            
+        return self.nonsingular(vmin, vmax)
 
 class AutoLocator(Locator):
     """
@@ -607,9 +642,13 @@ class AutoLocator(Locator):
 
     def __call__(self):
         'Return the locations of the ticks'
+        self.refresh()
+        return self._locator()
+
+    def refresh(self):
+        'refresh internal information based on current lim'
         d = self.viewInterval.span()
         self._locator = self.get_locator(d)
-        return self._locator()
 
     def autoscale(self):
         'Try to choose the view limits intelligently'
@@ -710,8 +749,7 @@ class MonthLocator(Locator):
             dmax = self.epochConverter.add_month(dmax, 1)
         
 
-
-        return dmin, dmax
+        return self.nonsingular(dmin, dmax)
 
 
 
@@ -765,7 +803,7 @@ class YearLocator(Locator):
 
 
 
-        return vmin, vmax
+        return self.nonsingular(vmin, vmax)
 
 
 class MinuteLocator(MultipleLocator):
@@ -798,6 +836,13 @@ class DayLocator(MultipleLocator):
         locs = MultipleLocator.__call__(self)
         return asarray(locs) + self.hour*SEC_PER_HOUR
 
+class DayMultiLocator(MultipleLocator):
+    """
+    Make ticks on day which are multiples of base
+    """
+    def __init__(self, base):
+        MultipleLocator.__init__(self, base*SEC_PER_DAY)
+
 class WeekdayLocator(Locator):
 
     def __init__(self, day):
@@ -853,7 +898,7 @@ class WeekdayLocator(Locator):
         dmax += offset
 
 
-        return dmin, dmax        
+        return self.nonsingular(dmin, dmax)
 
     def get_ticklocs(self):
         'return the tick locations in seconds since epoch'
@@ -861,3 +906,10 @@ class WeekdayLocator(Locator):
             raise RuntimeError('You must first set the tlim')
         return self.ticks
         
+
+class WeekMultiLocator(MultipleLocator):
+    """
+    Make ticks on weeks which are multiples of base
+    """
+    def __init__(self, base):
+        MultipleLocator.__init__(self, base*SEC_PER_WEEK)
