@@ -32,11 +32,11 @@ WIN32
 import os
 
 basedir = {
-
     'win32'  : ['win32_static',],
     'linux2' : ['/usr/local', '/usr',],
     'linux'  : ['/usr/local', '/usr',],
-    'darwin' : ['/usr/local', '/usr', '/sw'],
+    'darwin' : [os.getenv('MPLIB_BASE') or '/usr/local', '/usr', '/sw'],
+    'freebsd4' : [os.getenv('MPLIB_BASE') or '/usr/local', '/usr'],
     'sunos5' : [os.getenv('MPLIB_BASE') or '/usr/local',],
 }
 
@@ -59,7 +59,6 @@ BUILT_IMAGE     = False
 BUILT_TKAGG     = False
 BUILT_WINDOWING = False
 
-
 def add_base_flags(module):
     incdirs = [os.path.join(p, 'include') for p in basedir[sys.platform]
                if os.path.exists(p)]
@@ -96,7 +95,7 @@ def add_gd_flags(module):
 
 def add_ft2font_flags(module):
     'Add the module flags to build extensions which use gd'
-    module.libraries.extend(['freetype', 'z'])
+    module.libraries.extend(['freetype', 'z' ])
     add_base_flags(module)
 
     basedirs = module.include_dirs[:]  # copy the list to avoid inf loop!
@@ -116,7 +115,7 @@ def add_ft2font_flags(module):
         module.libraries.append('gw32c')
 
     # put this last for library link order     
-    module.libraries.append('m')
+    module.libraries.extend(['stdc++', 'm'])
 
     
 
@@ -184,6 +183,7 @@ def find_tcltk():
 	o.tcl_lib = "/usr/local/lib"
 	o.tcl_inc = "/usr/local/include"
 	o.tk_lib = "/usr/local/lib"
+        o.tk_inc = "/usr/local/include"
 	o.tkv = ""
     else:
 	tk.withdraw()
@@ -212,14 +212,60 @@ def add_tk_flags(module):
         module.include_dirs.extend(['win32_static/include/tcl'])
         module.library_dirs.extend(['C:/Python23/dlls'])
         module.libraries.extend(['tk84', 'tcl84'])
-    elif sys.platform == 'darwin':
-        module.extra_link_args.extend(['-framework','Tcl'])
-        module.extra_link_args.extend(['-framework','Tk'])
-    else:
-	o = find_tcltk()
-	module.include_dirs.extend([o.tcl_inc, o.tk_inc])
-	module.library_dirs.extend([o.tcl_lib, o.tk_lib])
-        module.libraries.extend(['tk'+o.tkv, 'tcl'+o.tkv])
+        return
+
+    elif sys.platform == 'darwin' :
+        # this config section lifted directly from Imaging - thanks to
+        # the effbot!
+
+        # First test for a MacOSX/darwin framework install
+        from os.path import join, exists
+        framework_dirs = [
+            '/System/Library/Frameworks/',
+            '/Library/Frameworks',
+            join(os.getenv('HOME'), '/Library/Frameworks')
+        ]
+
+        # Find the directory that contains the Tcl.framwork and Tk.framework
+        # bundles.
+        # XXX distutils should support -F!
+        for F in framework_dirs:
+            # both Tcl.framework and Tk.framework should be present
+            for fw in 'Tcl', 'Tk':
+                if not exists(join(F, fw + '.framework')):
+                    break
+            else:
+                # ok, F is now directory with both frameworks. Continure
+                # building
+                tk_framework_found = 1
+                break
+        if tk_framework_found:
+            # For 8.4a2, we must add -I options that point inside the Tcl and Tk
+            # frameworks. In later release we should hopefully be able to pass
+            # the -F option to gcc, which specifies a framework lookup path.
+            #
+            tk_include_dirs = [
+                join(F, fw + '.framework', H)
+                for fw in 'Tcl', 'Tk'
+                for H in 'Headers', 'Versions/Current/PrivateHeaders'
+            ]
+
+            # For 8.4a2, the X11 headers are not included. Rather than include a
+            # complicated search, this is a hard-coded path. It could bail out
+            # if X11 libs are not found...
+            # tk_include_dirs.append('/usr/X11R6/include')
+            frameworks = ['-framework', 'Tcl', '-framework', 'Tk']
+            module.include_dirs.extend(tk_include_dirs)
+            module.extra_link_args.extend(frameworks)
+            module.extra_compile_args.extend(frameworks)
+            return
+
+    # you're still here? ok we'll try it this way
+    o = find_tcltk()
+    module.include_dirs.extend([o.tcl_inc, o.tk_inc])
+    module.library_dirs.extend([o.tcl_lib, o.tk_lib])
+    module.libraries.extend(['tk'+o.tkv, 'tcl'+o.tkv])
+
 
 def add_windowing_flags(module):
     'Add the module flags to build extensions using windowing api'
@@ -244,10 +290,11 @@ def build_windowing(ext_modules, packages):
 def build_ft2font(ext_modules, packages):
     global BUILT_FT2FONT
     if BUILT_FT2FONT: return # only build it if you you haven't already
-    module = Extension('matplotlib.ft2font',
-                       ['src/ft2font.c',
-                        ],
-                       )
+    deps = ['src/ft2font.cpp', 'src/mplutils.cpp']
+    deps.extend(glob.glob('CXX/*.cxx'))
+    deps.extend(glob.glob('CXX/*.c'))
+
+    module = Extension('matplotlib.ft2font', deps)
     add_ft2font_flags(module)
     ext_modules.append(module)    
     BUILT_FT2FONT = True
@@ -255,7 +302,7 @@ def build_ft2font(ext_modules, packages):
 def build_gtkagg(ext_modules, packages):
     global BUILT_GTKAGG
     if BUILT_GTKAGG: return # only build it if you you haven't already
-    deps = ['src/_gtkagg.cpp']
+    deps = ['src/_gtkagg.cpp', 'src/mplutils.cpp']
     deps.extend(glob.glob('CXX/*.cxx'))
     deps.extend(glob.glob('CXX/*.c'))
 
@@ -300,7 +347,7 @@ def build_agg(ext_modules, packages):
     global BUILT_AGG
     if BUILT_AGG: return # only build it if you you haven't already
     
-    deps = ['src/_backend_agg.cpp', 'src/ft2font.c']
+    deps = ['src/_backend_agg.cpp', 'src/ft2font.cpp', 'src/mplutils.cpp']
     deps.extend(glob.glob('agg2/src/*.cpp'))
     deps.extend(glob.glob('CXX/*.cxx'))
     deps.extend(glob.glob('CXX/*.c'))
@@ -320,8 +367,10 @@ def build_image(ext_modules, packages, numerix):
     global BUILT_IMAGE
     if BUILT_IMAGE: return # only build it if you you haven't already
     
-    deps = ['src/_image.cpp'] 
+    deps = ['src/_image.cpp', 'src/mplutils.cpp'] 
     deps.extend(glob.glob('agg2/src/*.cpp'))
+    deps.extend(glob.glob('CXX/*.cxx'))
+    deps.extend(glob.glob('CXX/*.c'))
 
     module = Extension(
         'matplotlib._image',
