@@ -14,6 +14,7 @@
 #include "_backend_agg.h"
 
 #include "_transforms.h"
+#include "mplutils.h"
 
 /* ------------ RendererAgg methods ------------- */
 
@@ -28,8 +29,7 @@ RendererAgg::RendererAgg(unsigned int width, unsigned int height, double dpi,
   NUMBYTES(width*height*4),
   debug(debug)
 {
-  if (debug)
-    std::cout << "RendererAgg::RendererAgg" << std::endl;
+  _VERBOSE("RendererAgg::RendererAgg");
   unsigned stride(width*4);    
   
   
@@ -53,10 +53,9 @@ RendererAgg::RendererAgg(unsigned int width, unsigned int height, double dpi,
 
 Py::Object
 RendererAgg::draw_rectangle(const Py::Tuple & args) {
-  if (debug)
-    std::cout << "RendererAgg::draw_rectangle" << std::endl;
+  _VERBOSE("RendererAgg::draw_rectangle");
   args.verify_length(6);
-  
+  theRasterizer->reset_clipping();
   Py::Object gcEdge( args[0] );
   Py::Object rgbFaceMaybeNone( args[1] );
   
@@ -107,9 +106,8 @@ RendererAgg::draw_rectangle(const Py::Tuple & args) {
 
 Py::Object
 RendererAgg::draw_ellipse(const Py::Tuple& args) {
-  if (debug)
-    std::cout << "RendererAgg::draw_ellipse" << std::endl;
-  
+  _VERBOSE("RendererAgg::draw_ellipse");
+  theRasterizer->reset_clipping();
   args.verify_length(6);  
   Py::Object gcEdge = args[0];
   Py::Object rgbFaceMaybeNone = args[1];
@@ -153,27 +151,26 @@ RendererAgg::draw_ellipse(const Py::Tuple& args) {
 
 Py::Object
 RendererAgg::draw_polygon(const Py::Tuple& args) {
-  if (debug)
-    std::cout << "RendererAgg::draw_polygon" << std::endl;
-  
+  _VERBOSE("RendererAgg::draw_polygon");
+  theRasterizer->reset_clipping();  
   args.verify_length(3);  
-  
+
   Py::Object gcEdge( args[0] );
   Py::Object rgbFaceMaybeNone( args[1] );
   Py::SeqBase<Py::Object> points( args[2] );
-  
+
   
   set_clip_rectangle(gcEdge);
   agg::gen_stroke::line_cap_e cap = get_linecap(gcEdge);
   agg::gen_stroke::line_join_e join = get_joinstyle(gcEdge);
   
   double lw = points_to_pixels ( gcEdge.getAttr("_linewidth") ) ;
-  
+
   size_t Npoints = points.length();
   if (Npoints<=0)
     return Py::Object();
-  
-  
+
+
   // dump the x.y vertices into a double array for faster look ahread
   // and behind access
   double xs[Npoints];
@@ -187,6 +184,7 @@ RendererAgg::draw_polygon(const Py::Tuple& args) {
 
   }
   
+
   agg::path_storage path;  
   for (size_t j=0; j<Npoints; ++j) {
 
@@ -197,7 +195,7 @@ RendererAgg::draw_polygon(const Py::Tuple& args) {
     else path.line_to(x,y); 
   }
   path.close_polygon();
-  
+
   agg::rgba edgecolor = get_color(gcEdge);
 
   
@@ -209,7 +207,7 @@ RendererAgg::draw_polygon(const Py::Tuple& args) {
     theRasterizer->add_path(path);    
     theRasterizer->render(*slineP8, *theRenderer);  
   }
-  
+
   //now fill the edge
   agg::conv_stroke<agg::path_storage> stroke(path);
   stroke.width(lw);
@@ -226,10 +224,10 @@ RendererAgg::draw_polygon(const Py::Tuple& args) {
 
 Py::Object
 RendererAgg::draw_line_collection(const Py::Tuple& args) {
+  theRasterizer->reset_clipping();
   
-  
-  if (debug)
-    std::cout << "RendererAgg::draw_line_collection" << std::endl;
+  _VERBOSE("RendererAgg::draw_line_collection");
+
   args.verify_length(8);  
   
   
@@ -255,10 +253,6 @@ RendererAgg::draw_line_collection(const Py::Tuple& args) {
   Py::SeqBase<Py::Object> offsets;
   Transformation* transOffset=NULL;
   if  (usingOffsets) {
-    /* this line is broken, mysteriously
-    if (!Transformation::check(args[7])) 
-      throw Py::TypeError("RendererAgg::draw_line_collection expected a Transformation instance for transOffset");
-    */
     offsets = Py::SeqBase<Py::Object>(args[6]);        
     transOffset = static_cast<Transformation*>(args[7].ptr());
   }
@@ -274,53 +268,42 @@ RendererAgg::draw_line_collection(const Py::Tuple& args) {
     Noffsets = offsets.length();
     if (Noffsets>Nsegments) N = Noffsets;
   }
-  
 
-  Py::Tuple xyo, pos;
+  double xo(0.0), yo(0.0), thisx(0.0), thisy(0.0);
+  std::pair<double, double> xy;  
+  Py::Tuple xyo;
+  Py::SeqBase<Py::Object> xys;
   for (size_t i=0; i<N; ++i) {
-    
-    pos = Py::Tuple(segments[i%Nsegments]);
-    double x0 = Py::Float(pos[0]);
-    double y0 = Py::Float(pos[1]);
-    double x1 = Py::Float(pos[2]);
-    double y1 = Py::Float(pos[3]);
-
-
-    std::pair<double, double> xy = transform->operator()(x0,y0);
-    x0 = xy.first;
-    y0 = xy.second;
-
-    xy = transform->operator()(x1,y1);
-    x1 = xy.first;
-    y1 = xy.second;
-
     if (usingOffsets) {
       xyo = Py::Tuple(offsets[i%Noffsets]);
-      double xo = Py::Float(xyo[0]);
-      double yo = Py::Float(xyo[1]);
-      std::pair<double, double> xy = transOffset->operator()(xo,yo);
-      x0 += xy.first;
-      y0 += xy.second;
-      x1 += xy.first;
-      y1 += xy.second;
-      
-    }
-    //snap x to pixel for verical lines
-    if (x0==x1) {
-      x0 = (int)x0 + 0.5;
-      x1 = (int)x1 + 0.5;
-    }
-    
-    //snap y to pixel for horizontal lines
-    if (y0==y1) {
-      y0 = (int)y0 + 0.5;
-      y1 = (int)y1 + 0.5;
+      xo = Py::Float(xyo[0]);
+      yo = Py::Float(xyo[1]);
+      xy = transOffset->operator()(xo,yo);
+      xo = xy.first;
+      yo = xy.second;
     }
 
+    xys = segments[i%Nsegments];
+    size_t numtups = xys.length();
+    if (numtups<2) continue;
       
     agg::path_storage path;
-    path.move_to(x0, height-y0);
-    path.line_to(x1, height-y1);
+
+    for (size_t j=0; j<numtups; j++) {
+      xyo = xys[j];
+      thisx = Py::Float(xyo[0]);
+      thisy = Py::Float(xyo[1]);
+      xy = transform->operator()(thisx,thisy);
+      thisx = xy.first;
+      thisy = xy.second;
+
+      if (usingOffsets) {
+	thisx += xo;
+	thisy += yo;
+      }      
+      if (j==0)  path.move_to(thisx, height-thisy);
+      else       path.line_to(thisx, height-thisy);
+    }
 
     agg::conv_stroke<agg::path_storage> stroke(path);
     //stroke.line_cap(cap);
@@ -359,6 +342,7 @@ RendererAgg::set_clip_from_bbox(const Py::Object& o) {
     // Bbox::check(args[0]) failing; something about cross module?
     // set the clip rectangle
     // flipy
+
     Bbox* clipbox = static_cast<Bbox*>(o.ptr());
     double l = clipbox->ll_api()->x_api()->val() ; 
     double b = clipbox->ll_api()->y_api()->val();
@@ -373,10 +357,9 @@ RendererAgg::set_clip_from_bbox(const Py::Object& o) {
 
 Py::Object
 RendererAgg::draw_poly_collection(const Py::Tuple& args) {
+  theRasterizer->reset_clipping();
   
-  
-  if (debug)
-    std::cout << "RendererAgg::draw_poly_collection" << std::endl;
+  _VERBOSE("RendererAgg::draw_poly_collection");
   args.verify_length(9);  
   
 
@@ -535,10 +518,9 @@ RendererAgg::draw_poly_collection(const Py::Tuple& args) {
 
 Py::Object
 RendererAgg::draw_regpoly_collection(const Py::Tuple& args) {
+  theRasterizer->reset_clipping();
   
-  
-  if (debug)
-    std::cout << "RendererAgg::draw_regpoly_collection" << std::endl;
+  _VERBOSE("RendererAgg::draw_regpoly_collection");
   args.verify_length(9);  
   
   
@@ -572,7 +554,7 @@ RendererAgg::draw_regpoly_collection(const Py::Tuple& args) {
   
   double thisx, thisy;
   
-  // dump the x.y vertices into a double array for paster access
+  // dump the x.y vertices into a double array for faster access
   double xverts[Nverts];
   double yverts[Nverts];
   Py::Tuple xy;
@@ -648,7 +630,7 @@ RendererAgg::draw_regpoly_collection(const Py::Tuple& args) {
 	theRenderer->color(edgecolor);    
 	theRasterizer->render(*slineP8, *theRenderer); 
       }
-      else {
+      else {      
 	rendererBin->color(edgecolor);    
 	theRasterizer->render(*slineBin, *rendererBin); 
       }
@@ -661,9 +643,8 @@ RendererAgg::draw_regpoly_collection(const Py::Tuple& args) {
 Py::Object
 RendererAgg::draw_lines(const Py::Tuple& args) {
   
-  
-  if (debug)
-    std::cout << "RendererAgg::draw_lines" << std::endl;
+  theRasterizer->reset_clipping();
+  _VERBOSE("RendererAgg::draw_lines");
   args.verify_length(3);  
   Py::Object gc = args[0];
   Py::SeqBase<Py::Object> x = args[1];  //todo: use numerix for efficiency
@@ -686,7 +667,7 @@ RendererAgg::draw_lines(const Py::Tuple& args) {
   
   
   double lw = points_to_pixels ( gc.getAttr("_linewidth") ) ;
-  
+  //std::cout << "agg lw " << lw << std::endl;
   agg::rgba color = get_color(gc);
   
   
@@ -708,7 +689,7 @@ RendererAgg::draw_lines(const Py::Tuple& args) {
   
   int isaa = antialiased(gc);
 
-
+  double heightd = double(height);
   if (Nx==2) { 
     // this is a little hack - len(2) lines are probably grid and
     // ticks so I'm going to snap to pixel
@@ -717,7 +698,7 @@ RendererAgg::draw_lines(const Py::Tuple& args) {
     double y0 = Py::Float(y[0]);
     double x1 = Py::Float(x[1]);
     double y1 = Py::Float(y[1]);
-
+    
     if (x0==x1) {
       x0 = (int)x0 + 0.5;
       x1 = (int)x1 + 0.5;
@@ -728,8 +709,8 @@ RendererAgg::draw_lines(const Py::Tuple& args) {
       y1 = (int)y1 + 0.5;
     }
 
-    y0 = height-y0;
-    y1 = height-y1;
+    y0 = heightd-y0;
+    y1 = heightd-y1;
 
     path.move_to(x0, y0);
     path.line_to(x1, y1);
@@ -738,17 +719,19 @@ RendererAgg::draw_lines(const Py::Tuple& args) {
   else {
     double thisX = Py::Float( x[0] );
     double thisY = Py::Float( y[0] );
-    thisY = height - thisY; //flipy
+    thisY = heightd - thisY; //flipy
     path.move_to(thisX, thisY);
     for (size_t i=1; i<Nx; ++i) {
       thisX = Py::Float( x[i] );
       thisY = Py::Float( y[i] );
-      thisY = height - thisY;  //flipy
+      thisY = heightd - thisY;  //flipy
+      //if ((i<10) || i>=19990)
+      //std::cout << i << " " << Nx << " " << thisX << " " << thisY << std::endl;
       path.line_to(thisX, thisY);
     }
   }  
   
-  
+  //std::cout << width << " " << height << std::endl;
   if (! useDashes ) {
     
     agg::conv_stroke<agg::path_storage> stroke(path);
@@ -798,26 +781,40 @@ RendererAgg::draw_lines(const Py::Tuple& args) {
 
 Py::Object
 RendererAgg::draw_text(const Py::Tuple& args) {
-  if (debug)
-    std::cout << "RendererAgg::draw_text" << std::endl;
-  
+  _VERBOSE("RendererAgg::draw_text");
+  theRasterizer->reset_clipping();
   args.verify_length(4);
   
   
-  FT2FontObject *font = (FT2FontObject *)args[0].ptr();
+  FT2Font *font = static_cast<FT2Font*>(args[0].ptr());
   
   int x = Py::Int( args[1] );
   int y = Py::Int( args[2] );
-  Py::SeqBase<Py::Object> rgba = Py::SeqBase<Py::Object>( args[3] );
-  
-  double r = Py::Float( rgba[0] );
-  double g = Py::Float( rgba[1] );
-  double b = Py::Float( rgba[2] );
-  double a = Py::Float( rgba[3] );
-  
+  Py::Object gc = args[3];
+
+  Py::Object o ( gc.getAttr( "_cliprect" ) );
+
+  bool useClip = o.ptr()!=Py_None;
+  double l = 0;
+  double b = 0;
+  double r = width;
+  double t = height;
+  if (useClip) {
+    Py::SeqBase<Py::Object> rect( o );
   
+    l = Py::Float(rect[0]) ; 
+    b = Py::Float(rect[1]) ; 
+    double w = Py::Float(rect[2]) ; 
+    double h = Py::Float(rect[3]) ; 
+    r = l+w;
+    t = b+h;
+    //std::cout << b << " " << h << " " << " " << t << std::endl;
+  }
+
+  agg::rgba color = get_color(gc);  
   pixfmt::color_type p;
-  p.r = int(255*r); p.b = int(255*b); p.g = int(255*g); p.a = int(255*a);
+  p.r = int(255*color.r); p.b = int(255*color.b); 
+  p.g = int(255*color.g); p.a = int(255*color.a);
    
   //y = y-font->image.height;
   unsigned thisx, thisy;
@@ -826,8 +823,8 @@ RendererAgg::draw_text(const Py::Tuple& args) {
     for (size_t j=0; j<font->image.height; ++j) {
       thisx = i+x+font->image.offsetx; 
       thisy = j+y+font->image.offsety; 
-      if (thisx<0 || thisx>=width)  continue;
-      if (thisy<0 || thisy>=height) continue;
+      if (thisx<l || thisx>=r)  continue;
+      if (thisy<height-t || thisy>=height-b) continue;
       pixFmt->blend_pixel
 	(thisx, thisy, p, font->image.buffer[i + j*font->image.width]);
     }
@@ -861,20 +858,60 @@ RendererAgg::draw_text(const Py::Tuple& args) {
 
 Py::Object 
 RendererAgg::draw_image(const Py::Tuple& args) {
-  if (debug)
-    std::cout << "RendererAgg::draw_image" << std::endl;
-  args.verify_length(3);
+  _VERBOSE("RendererAgg::draw_image");
+  theRasterizer->reset_clipping();
+  args.verify_length(5);
   
   int x = Py::Int(args[0]);
   int y = Py::Int(args[1]);
-  ImageObject *image = (ImageObject *)args[2].ptr();
+  Image *image = static_cast<Image*>(args[2].ptr());
+  std::string origin = Py::String(args[3]);
   
+  if (origin!="lower" && origin!="upper")
+    throw Py::ValueError("origin must be upper|lower");
   
-  //todo: handle x and y
-  agg::rect r(0, 0, image->rowsOut, image->colsOut);
-  
-  rendererBase->copy_from(*image->rbufOut, &r, x, y);
+  bool isUpper = origin=="upper";
   
+  size_t ind = 0;
+  size_t thisx, thisy;
+  size_t oy = isUpper ? y : height-y;
+
+  size_t minx(0), maxx(width), miny(0), maxy(height);
+
+  if (args[4].ptr() != Py_None) {
+    Bbox* bbox = static_cast<Bbox*>(args[4].ptr());
+    minx = (size_t) (bbox->ll_api()->x_api()->val());
+    maxy = height-(size_t) (bbox->ll_api()->y_api()->val());
+    maxx = (size_t) (bbox->ur_api()->x_api()->val());
+    miny = height-(size_t) (bbox->ur_api()->y_api()->val());
+  }
+
+  //if (isUpper) oy -= image->rowsOut;  //start at top
+  //std::cout << minx << " " << maxx << " " << miny << " " << maxy << std::endl;
+  for (size_t j=0; j<image->rowsOut; j++) {
+    thisy =  isUpper ?  oy+j : oy-j; 
+    if (thisy<miny || thisy>=maxy) {
+     ind += 4*image->colsOut;
+      continue;
+    }
+    for (size_t i=0; i<image->colsOut; i++) {
+      thisx = i+x; 
+      if (thisx<minx || thisx>=maxx) {
+      ind += 4;
+      	continue;
+      }
+	
+      pixfmt::color_type p;
+
+      p.r = *(image->bufferOut+ind++);
+      p.g = *(image->bufferOut+ind++);
+      p.b = *(image->bufferOut+ind++);
+      p.a = *(image->bufferOut+ind++);
+
+      pixFmt->blend_pixel(thisx, thisy, p, p.a);
+    }
+  }
+
   return Py::Object();
   
 }
@@ -882,8 +919,7 @@ RendererAgg::draw_image(const Py::Tuple& args) {
 
 Py::Object 
 RendererAgg::write_rgba(const Py::Tuple& args) {
-  if (debug)
-    std::cout << "RendererAgg::write_rgba" << std::endl;
+  _VERBOSE("RendererAgg::write_rgba");
   
   args.verify_length(1);  
   std::string fname = Py::String( args[0]);
@@ -903,8 +939,8 @@ RendererAgg::write_rgba(const Py::Tuple& args) {
 Py::Object 
 RendererAgg::write_png(const Py::Tuple& args)
 {
-  if (debug)
-    std::cout << "RendererAgg::write_png" << std::endl;
+  //small memory leak in this function - JDH 2004-06-08
+  _VERBOSE("RendererAgg::write_png");
   
   args.verify_length(1);
   
@@ -935,13 +971,13 @@ RendererAgg::write_png(const Py::Tuple& args)
   info_ptr = png_create_info_struct(png_ptr);
   if (info_ptr == NULL) {
     fclose(fp);
-    png_destroy_write_struct(&png_ptr, (png_infopp)NULL);
+    png_destroy_write_struct(&png_ptr, &info_ptr);
     throw Py::RuntimeError("could not create info struct");
   }
   
   if (setjmp(png_ptr->jmpbuf)) {
     fclose(fp);
-    png_destroy_write_struct(&png_ptr, (png_infopp)NULL);
+    png_destroy_write_struct(&png_ptr, &info_ptr);
     throw Py::RuntimeError("error building image");
   }
   
@@ -963,23 +999,34 @@ RendererAgg::write_png(const Py::Tuple& args)
   png_write_info(png_ptr, info_ptr);
   png_write_image(png_ptr, row_pointers);
   png_write_end(png_ptr, info_ptr);
-  png_destroy_write_struct(&png_ptr, (png_infopp)NULL);
+
+  /* Changed calls to png_destroy_write_struct to follow
+     http://www.libpng.org/pub/png/libpng-manual.txt.  
+     This ensures the info_ptr memory is released.  
+  */
+
+  png_destroy_write_struct(&png_ptr, &info_ptr);
+
+ 
   fclose(fp);
   
   return Py::Object();
 }
 
+
 Py::Object 
 RendererAgg::tostring_rgb(const Py::Tuple& args) {
   //"Return the rendered buffer as an RGB string";
   
-  if (debug)
-    std::cout << "RendererAgg::tostring_rgb" << std::endl;
+  _VERBOSE("RendererAgg::tostring_rgb");
   
   args.verify_length(0);    
   int row_len = width*3;
-  unsigned char* buf_tmp = 
-    new unsigned char[row_len * height];
+  unsigned char* buf_tmp = new unsigned char[row_len * height];
+  if (buf_tmp ==NULL) {
+    //todo: also handle allocation throw
+    throw Py::MemoryError("RendererAgg::tostring_rgb could not allocate memory");
+  }
   agg::rendering_buffer renderingBufferTmp;
   renderingBufferTmp.attach(buf_tmp, 
 			    width, 
@@ -990,19 +1037,28 @@ RendererAgg::tostring_rgb(const Py::Tuple& args) {
   
   
   //todo: how to do this with native CXX
-  return Py::Object(Py_BuildValue("s#", 
-				  buf_tmp, 
-				  row_len * height));
-  //len = row_len * height
-  //std::string s(buf_tmp);
-  //return Py::String(buf_tmp, row_len * height);
+  PyObject* o = Py_BuildValue("s#", 
+			      buf_tmp, 
+			      row_len * height);
+  delete [] buf_tmp;
+  return Py::asObject(o);
+}
+
+Py::Object 
+RendererAgg::buffer_rgba(const Py::Tuple& args) {
+  //"expose the rendered buffer as Python buffer object";
+  
+  _VERBOSE("RendererAgg::buffer_rgba");
+  
+  args.verify_length(0);    
+  int row_len = width*4;
+  return Py::asObject(PyBuffer_FromMemory( pixBuffer, row_len*height));
 }
 
 agg::rgba
 RendererAgg::get_color(const Py::Object& gc) {
   
-  if (debug)
-    std::cout << "RendererAgg::get_color" << std::endl;
+  _VERBOSE("RendererAgg::get_color");
   
   Py::Tuple rgb = Py::Tuple( gc.getAttr("_rgb") );
   
@@ -1015,10 +1071,21 @@ RendererAgg::get_color(const Py::Object& gc) {
   
 }
 
+Py::Object
+RendererAgg::clear(const Py::Tuple& args) {
+  //"clear the rendered buffer";
+  
+  _VERBOSE("RendererAgg::clear");
+  
+  args.verify_length(0);    
+  rendererBase->clear(agg::rgba(1, 1, 1, 0));
+  
+  return Py::Object();
+}
+
 agg::gen_stroke::line_cap_e
 RendererAgg::get_linecap(const Py::Object& gc) {
-  if (debug)
-    std::cout << "RendererAgg::get_linecap" << std::endl;
+  _VERBOSE("RendererAgg::get_linecap");
   
   std::string capstyle = Py::String( gc.getAttr( "_capstyle" ) );
   
@@ -1035,8 +1102,7 @@ RendererAgg::get_linecap(const Py::Object& gc) {
 
 agg::gen_stroke::line_join_e
 RendererAgg::get_joinstyle(const Py::Object& gc) {
-  if (debug)
-    std::cout << "RendererAgg::get_joinstyle" << std::endl;
+  _VERBOSE("RendererAgg::get_joinstyle");
   
   std::string joinstyle = Py::String( gc.getAttr("_joinstyle") );
   
@@ -1054,8 +1120,7 @@ RendererAgg::get_joinstyle(const Py::Object& gc) {
 Py::Tuple
 RendererAgg::get_dashes(const Py::Object& gc) {
   //return the dashOffset, dashes sequence tuple.  
-  if (debug)
-    std::cout << "RendererAgg::get_dashes" << std::endl;
+  _VERBOSE("RendererAgg::get_dashes");
   
   Py::Tuple _dashes = gc.getAttr("_dashes");
   
@@ -1070,8 +1135,7 @@ RendererAgg::get_dashes(const Py::Object& gc) {
 
 agg::rgba
 RendererAgg::rgb_to_color(const Py::SeqBase<Py::Object>& rgb, double alpha) {
-  if (debug)
-    std::cout << "RendererAgg::rgb_to_color" << std::endl;
+  _VERBOSE("RendererAgg::rgb_to_color");
   
   double r = Py::Float(rgb[0]);
   double g = Py::Float(rgb[1]);
@@ -1083,9 +1147,8 @@ RendererAgg::rgb_to_color(const Py::SeqBase<Py::Object>& rgb, double alpha) {
 void
 RendererAgg::set_clip_rectangle( const Py::Object& gc) {
   //set the clip rectangle from the gc
-  
-  if (debug)
-    std::cout << "RendererAgg::set_clip_rectangle" << std::endl;
+
+  _VERBOSE("RendererAgg::set_clip_rectangle");
   
   Py::Object o ( gc.getAttr( "_cliprect" ) );
   
@@ -1112,8 +1175,7 @@ RendererAgg::set_clip_rectangle( const Py::Object& gc) {
 int
 RendererAgg::antialiased(const Py::Object& gc) {
   //return 1 if gc is antialiased
-  if (debug)
-    std::cout << "RendererAgg::antialiased" << std::endl;
+  _VERBOSE("RendererAgg::antialiased");
   int isaa = Py::Int( gc.getAttr( "_antialiased") );
   return isaa;
 }
@@ -1123,8 +1185,7 @@ RendererAgg::points_to_pixels_snapto(const Py::Object& points) {
   // convert a value in points to pixels depending on renderer dpi and
   // screen pixels per inch
   // snap return pixels to grid
-  if (debug)
-    std::cout << "RendererAgg::points_to_pixels_snapto" << std::endl;
+  _VERBOSE("RendererAgg::points_to_pixels_snapto");
   double p = Py::Float( points ) ;
   //return (int)(p*PIXELS_PER_INCH/72.0*dpi/72.0)+0.5;
   return (int)(p*dpi/72.0)+0.5;
@@ -1134,8 +1195,7 @@ RendererAgg::points_to_pixels_snapto(const Py::Object& points) {
 
 double
 RendererAgg::points_to_pixels( const Py::Object& points) {
-  if (debug)
-    std::cout << "RendererAgg::points_to_pixels" << std::endl;
+  _VERBOSE("RendererAgg::points_to_pixels");
   double p = Py::Float( points ) ;
   //return p * PIXELS_PER_INCH/72.0*dpi/72.0;
   return p * dpi/72.0;
@@ -1143,13 +1203,13 @@ RendererAgg::points_to_pixels( const Py::Object& points) {
 
 RendererAgg::~RendererAgg() {
   
-  if (debug)
-    std::cout << "RendererAgg::~RendererAgg" << std::endl;
+  _VERBOSE("RendererAgg::~RendererAgg");
   
   
   delete slineP8;
   delete slineBin;
   delete theRasterizer;
+  delete theRenderer;
   delete rendererBin;
   delete rendererBase;
   delete pixFmt;
@@ -1210,6 +1270,10 @@ void RendererAgg::init_type()
 		     "write_png(fname)");
   add_varargs_method("tostring_rgb", &RendererAgg::tostring_rgb, 
 		     "s = tostring_rgb()");
+  add_varargs_method("buffer_rgba", &RendererAgg::buffer_rgba, 
+		     "buffer = buffer_rgba()");
+  add_varargs_method("clear", &RendererAgg::clear, 
+		     "clear()");
   
 }
 
@@ -1222,8 +1286,3 @@ DL_EXPORT(void)
   _backend_agg = new _backend_agg_module;
   
 };
-
-
-
-
-
