@@ -1,15 +1,21 @@
 #include <functional>
-#include <numeric>
 #include "_transforms.h"
- 
+#include "mplutils.h"
+
 #ifdef NUMARRAY
 #include "numarray/arrayobject.h" 
 #else
 #include "Numeric/arrayobject.h" 
 #endif   
 
+Value::~Value() {
+  _VERBOSE("Value::~Value");
+
+}
+
 Py::Object
 Value::set(const Py::Tuple & args) {
+  _VERBOSE("Value::set");
   args.verify_length(1);
 
   _val = Py::Float( args[0] ); 
@@ -18,6 +24,7 @@ Value::set(const Py::Tuple & args) {
 
 Py::Object
 Value::get(const Py::Tuple & args) {
+  _VERBOSE("Value::get");
   args.verify_length(0);
   
   return Py::Float( _val ); 
@@ -40,17 +47,20 @@ LazyValue::compare(const Py::Object &other) {
 
 Py::Object 
 LazyValue::number_add( const Py::Object &o ) {
+  _VERBOSE("LazyValue::number");
   
   
   if (!LazyValue::check(o)) 
     throw Py::TypeError("Can only add LazyValues with other LazyValues");
   
   LazyValue* rhs = static_cast<LazyValue*>(o.ptr());
-  return Py::Object(new BinOp(this, rhs, BinOp::ADD));
+ 
+  return Py::asObject(new BinOp(this, rhs, BinOp::ADD));
 } 
 
 Py::Object 
 LazyValue::number_divide( const Py::Object &o ) {
+  _VERBOSE("LazyValue::number");
   
   //std::cout << "initing divide" << std::endl;
   if (!LazyValue::check(o)) 
@@ -59,41 +69,85 @@ LazyValue::number_divide( const Py::Object &o ) {
   LazyValue* rhs = static_cast<LazyValue*>(o.ptr());
   BinOp* op = new BinOp(this, rhs, BinOp::DIVIDE);
   //std::cout << "initing divide done" << std::endl;
-  return Py::Object(op);
+  return Py::asObject(op);
 } 
 
 Py::Object 
 LazyValue::number_multiply( const Py::Object &o ) {
+  _VERBOSE("LazyValue::number");
   
   
   if (!LazyValue::check(o)) 
     throw Py::TypeError("Can only multiply LazyValues with other LazyValues");
   
   LazyValue* rhs = static_cast<LazyValue*>(o.ptr());
-  return Py::Object(new BinOp(this, rhs, BinOp::MULTIPLY));
+  return Py::asObject(new BinOp(this, rhs, BinOp::MULTIPLY));
 } 
 
 Py::Object 
 LazyValue::number_subtract( const Py::Object &o ) {
+  _VERBOSE("LazyValue::number");
   
   
   if (!LazyValue::check(o)) 
     throw Py::TypeError("Can only subtract LazyValues with other LazyValues");
   
   LazyValue* rhs = static_cast<LazyValue*>(o.ptr());
-  return Py::Object(new BinOp(this, rhs, BinOp::SUBTRACT));
+  return Py::asObject(new BinOp(this, rhs, BinOp::SUBTRACT));
 } 
 
+BinOp::BinOp(LazyValue* lhs, LazyValue* rhs, int opcode) : 
+  _lhs(lhs), _rhs(rhs), _opcode(opcode) {
+  _VERBOSE("BinOp::BinOp");
+  Py_INCREF(lhs);
+  Py_INCREF(rhs);
+}
+
+BinOp::~BinOp() {
+  _VERBOSE("BinOp::~BinOp");
+  Py_DECREF(_lhs);
+  Py_DECREF(_rhs);
+}
+
 Py::Object
 BinOp::get(const Py::Tuple & args) {
+  _VERBOSE("BinOp::get");
   args.verify_length(0);
   double x = val();
   return Py::Float( x ); 
 }
 
+Point::Point(LazyValue* x, LazyValue*  y) : _x(x), _y(y) {
+  _VERBOSE("Point::Point"); 
+  Py_INCREF(x);
+  Py_INCREF(y);
+}
+
+Point::~Point()
+{
+  _VERBOSE("Point::~Point");
+  Py_DECREF(_x);
+  Py_DECREF(_y);
+
+}
+
+Interval::Interval(LazyValue* val1, LazyValue* val2) : 
+  _val1(val1), _val2(val2) {
+  _VERBOSE("Interval::Interval");
+  Py_INCREF(val1);
+  Py_INCREF(val2);
+};
+
+Interval::~Interval() {
+  _VERBOSE("Interval::~Interval");
+  Py_DECREF(_val1);
+  Py_DECREF(_val2);
+
+}
 
 Py::Object 
 Interval::update(const Py::Tuple &args) {
+  _VERBOSE("Interval::update");
   args.verify_length(2);
 
   Py::SeqBase<Py::Object> vals = args[0];
@@ -106,17 +160,19 @@ Interval::update(const Py::Tuple &args) {
   double minx = _val1->val();
   double maxx = _val2->val();
 
+  double thisval;
   if (ignore) {
-    minx = std::numeric_limits<double>::max();
-    maxx = std::numeric_limits<double>::min();
+    thisval = Py::Float(vals[0]);
+    minx = thisval;
+    maxx = thisval;
   }
 
-  double thisval;
+
   for (size_t i=0; i<Nval; ++i) {
     thisval = Py::Float(vals[i]);
 
-    if (thisval<minx) minx=thisval;
-    if (thisval>maxx) maxx=thisval;
+    if (thisval<minx) minx = thisval;
+    if (thisval>maxx) maxx = thisval;
   } 
 
 
@@ -125,9 +181,21 @@ Interval::update(const Py::Tuple &args) {
   return Py::Object();
 }
 
+Bbox::Bbox(Point* ll, Point* ur) : _ll(ll), _ur(ur) {
+  _VERBOSE("Bbox::Bbox");
+  Py_INCREF(ll);
+  Py_INCREF(ur);
+};
+  
+
+Bbox::~Bbox() {
+  _VERBOSE("Bbox::~Bbox");
+  Py_DECREF(_ll);
+  Py_DECREF(_ur);
+}
+
 Py::Object 
-Bbox::deepcopy(const Py::Tuple &args) {
-  args.verify_length(0);
+Bbox::_deepcopy() {
   
   double minx = _ll->xval();
   double miny = _ll->yval();
@@ -135,12 +203,19 @@ Bbox::deepcopy(const Py::Tuple &args) {
   double maxx = _ur->xval();
   double maxy = _ur->yval();
   
-  return Py::Object( new Bbox( new Point(new Value(minx), new Value(miny) ),
-			       new Point(new Value(maxx), new Value(maxy) )));
+  return Py::asObject( new Bbox( new Point(new Value(minx), new Value(miny) ),
+				 new Point(new Value(maxx), new Value(maxy) )));
+}
+Py::Object 
+Bbox::deepcopy(const Py::Tuple &args) {
+  _VERBOSE("Bbox::deepcopy");
+  args.verify_length(0);
+  return _deepcopy();
 }
 
 Py::Object 
 Bbox::scale(const Py::Tuple &args) {
+  _VERBOSE("Bbox::scale");
   args.verify_length(2);
   double sx = Py::Float(args[0]);
   double sy = Py::Float(args[1]);
@@ -168,6 +243,7 @@ Bbox::scale(const Py::Tuple &args) {
 
 Py::Object
 Bbox::get_bounds(const Py::Tuple & args) {
+  _VERBOSE("Bbox::get_bounds");
   args.verify_length(0);
   
   
@@ -190,21 +266,26 @@ Bbox::get_bounds(const Py::Tuple & args) {
 
 Py::Object 
 Bbox::contains(const Py::Tuple &args) {
+  _VERBOSE("Bbox::contains");
   args.verify_length(2);
 
   double x = Py::Float(args[0]);
   double y = Py::Float(args[1]); 
-  
-  Interval *ix = intervalx_api();
-  Interval *iy = intervaly_api();
-  int b = ix->contains_api(x) && iy->contains_api(y);
-  delete ix;
-  delete iy;
-  return Py::Int(b);
+
+  double minx = _ll->xval();
+  double miny = _ll->yval();  
+  double maxx = _ur->xval();
+  double maxy = _ur->yval();
+
+  int inx = ( (x>=minx) && (x<=maxx) || (x>=maxx) && (x<=minx) );
+  if (!inx) return Py::Int(0);
+  int iny = ( (y>=miny) && (y<=maxy) || (y>=maxy) && (y<=miny) );
+  return Py::Int(iny);
 }
 
 Py::Object 
 Bbox::overlaps(const Py::Tuple &args) {
+  _VERBOSE("Bbox::overlaps");
   args.verify_length(1);
 
   if (! check(args[0]))
@@ -217,6 +298,7 @@ Bbox::overlaps(const Py::Tuple &args) {
 
 Py::Object 
 Bbox::overlapsx(const Py::Tuple &args) {
+  _VERBOSE("Bbox::overlapsx");
   args.verify_length(1);
 
   if (! check(args[0]))
@@ -238,6 +320,7 @@ Bbox::overlapsx(const Py::Tuple &args) {
 
 Py::Object 
 Bbox::overlapsy(const Py::Tuple &args) {
+  _VERBOSE("Bbox::overlapsy");
   args.verify_length(1);
 
   if (! check(args[0]))
@@ -262,6 +345,7 @@ Bbox::overlapsy(const Py::Tuple &args) {
 
 Py::Object 
 Bbox::update(const Py::Tuple &args) {
+  _VERBOSE("Bbox::update");
   args.verify_length(2);
 
   Py::SeqBase<Py::Object> xys = args[0];
@@ -276,14 +360,19 @@ Bbox::update(const Py::Tuple &args) {
   double miny = _ll->yval();
   double maxy = _ur->yval();
 
+  Py::Tuple tup;
   if (ignore) {
-    minx = std::numeric_limits<double>::max();
-    maxx = std::numeric_limits<double>::min();
-    miny = std::numeric_limits<double>::max();
-    maxy = std::numeric_limits<double>::min();
+    tup = xys[0];
+    double x = Py::Float(tup[0]);
+    double y = Py::Float(tup[1]);
+
+    minx=x;
+    maxx=x;
+    miny=y;
+    maxy=y;
   }
 
-  Py::Tuple tup;
+
   for (size_t i=0; i<Nx; ++i) {
     tup = xys[i];
     double x = Py::Float(tup[0]);
@@ -293,6 +382,7 @@ Bbox::update(const Py::Tuple &args) {
     if (x>maxx) maxx=x;
     if (y<miny) miny=y;
     if (y>maxy) maxy=y;
+
   } 
 
 
@@ -303,9 +393,14 @@ Bbox::update(const Py::Tuple &args) {
   return Py::Object();
 }
 
+Func::~Func() {
+  _VERBOSE("Func::~Func");
+}
+
 
 Py::Object 
 Func::map(const Py::Tuple &args) {
+  _VERBOSE("Func::map");
   
   args.verify_length(1);
   double xin = Py::Float(args[0]);
@@ -317,17 +412,32 @@ Func::map(const Py::Tuple &args) {
 
 Py::Object 
 Func::inverse(const Py::Tuple &args) {
+  _VERBOSE("Func::inverse");
   
   args.verify_length(1);
   double xin = Py::Float(args[0]);
   
   double xout = this->inverse_api(xin);
   return Py::Float(xout);
-
 };
 
+FuncXY::FuncXY(Func* funcx, Func* funcy) : 
+  _funcx(funcx), _funcy(funcy) {
+  _VERBOSE("FuncXY::FuncXY");
+    Py_INCREF(funcx);
+    Py_INCREF(funcy);
+}
+
+FuncXY::~FuncXY() {
+  _VERBOSE("FuncXY::~FuncXY");
+  Py_DECREF(_funcx);
+  Py_DECREF(_funcy);
+
+}
+
 Py::Object 
 FuncXY::map(const Py::Tuple &args) {
+  _VERBOSE("FuncXY::map");
   
   args.verify_length(2);
   double xin = Py::Float(args[0]);
@@ -347,6 +457,7 @@ FuncXY::map(const Py::Tuple &args) {
 
 Py::Object 
 FuncXY::inverse(const Py::Tuple &args) {
+  _VERBOSE("FuncXY::inverse");
   
   args.verify_length(2);
   double xin = Py::Float(args[0]);
@@ -367,40 +478,59 @@ FuncXY::inverse(const Py::Tuple &args) {
 
 Py::Object
 FuncXY::set_funcx(const Py::Tuple & args) {
+  _VERBOSE("FuncXY::set_funcx");
   args.verify_length(1);
   
   if (!Func::check(args[0])) 
     throw Py::TypeError("set_funcx(func) expected a Func instance");
   
   _funcx = static_cast<Func*>(args[0].ptr());
+  Py_INCREF(_funcx);
   return Py::Object();
 }
 
 Py::Object
 FuncXY::set_funcy(const Py::Tuple & args) {
+  _VERBOSE("FuncXY::set_funcy");
   args.verify_length(1);
   
   if (!Func::check(args[0])) 
     throw Py::TypeError("set_funcy(func) expected a Func instance");
   
   _funcy = static_cast<Func*>(args[0].ptr());
+  Py_INCREF(_funcy);
   return Py::Object();
 }
 
 Py::Object
 FuncXY::get_funcx(const Py::Tuple & args) {
+  _VERBOSE("FuncXY::get_funcx");
   args.verify_length(0);
   return Py::Object(_funcx);
 }
 
 Py::Object
 FuncXY::get_funcy(const Py::Tuple & args) {
+  _VERBOSE("FuncXY::get_funcy");
   args.verify_length(0);
   return Py::Object(_funcy);
 }
 
+PolarXY::~PolarXY() {
+  _VERBOSE("PolarXY::~PolarXY");
+}
+
+Transformation::~Transformation() {
+  _VERBOSE("Transformation::~Transformation");
+  if (_transOffset!=NULL) {
+    Py_DECREF(_transOffset);
+  }
+
+}
+
 Py::Object
 Transformation::as_vec6(const Py::Tuple & args) {
+  _VERBOSE("Transformation::as_vec6");
   throw Py::RuntimeError("This transformation does not support as_vec6");
   return Py::Object();
 }
@@ -408,12 +538,14 @@ Transformation::as_vec6(const Py::Tuple & args) {
 
 Py::Object
 Transformation::get_funcx(const Py::Tuple & args) {
+  _VERBOSE("Transformation::get_funcx");
   throw Py::RuntimeError("This transformation does not support get_funcx");
   return Py::Object();
 }
 
 Py::Object
 Transformation::get_funcy(const Py::Tuple & args) {
+  _VERBOSE("Transformation::get_funcy");
   throw Py::RuntimeError("This transformation does not support get_funcy");
   return Py::Object();
 }
@@ -421,12 +553,14 @@ Transformation::get_funcy(const Py::Tuple & args) {
 
 Py::Object
 Transformation::set_funcx(const Py::Tuple & args) {
+  _VERBOSE("Transformation::set_funcx");
   throw Py::RuntimeError("This transformation does not support set_funcx");
   return Py::Object();
 }
 
 Py::Object
 Transformation::set_funcy(const Py::Tuple & args) {
+  _VERBOSE("Transformation::set_funcy");
   throw Py::RuntimeError("This transformation does not support set_funcy");
   return Py::Object();
 }
@@ -435,12 +569,14 @@ Transformation::set_funcy(const Py::Tuple & args) {
 
 Py::Object
 Transformation::get_bbox1(const Py::Tuple & args) {
+  _VERBOSE("Transformation::get_bbox1");
   throw Py::RuntimeError("This transformation does not support get_bbox1"); 
   return Py::Object();
 }
 
 Py::Object
 Transformation::get_bbox2(const Py::Tuple & args) {
+  _VERBOSE("Transformation::get_bbox2");
   throw Py::RuntimeError("This transformation does not support get_bbox2"); 
   return Py::Object();
 }
@@ -448,29 +584,38 @@ Transformation::get_bbox2(const Py::Tuple & args) {
 
 Py::Object
 Transformation::set_bbox1(const Py::Tuple & args) {
+  _VERBOSE("Transformation::set_bbox1");
   throw Py::RuntimeError("This transformation does not support set_bbox1"); 
   return Py::Object();
 }
 
 Py::Object
 Transformation::set_bbox2(const Py::Tuple & args) {
+  _VERBOSE("Transformation::set_bbox2");
   throw Py::RuntimeError("This transformation does not support set_bbox1"); 
   return Py::Object();
 }
 
 Py::Object
 Transformation::set_offset(const Py::Tuple & args) {
+  _VERBOSE("Transformation::set_offset");
   args.verify_length(2);
 
   Py::SeqBase<Py::Object> xy = args[0];
-
+  //std::cout << "checking args" << std::endl;
   if (!check(args[1]))
     throw Py::TypeError("Transformation::set_offset(xy,trans) requires trans to be a Transformation instance");
-  
+
+  //std::cout << "getting x,y" << std::endl;  
+
   _usingOffset = 1;
   _xo = Py::Float(xy[0]);
   _yo = Py::Float(xy[1]);
+  //std::cout << "casting" << std::endl;  
   _transOffset = static_cast<Transformation*>(args[1].ptr());
+  //std::cout << "increffing" << std::endl;  
+  Py_INCREF(_transOffset);
+  //std::cout << "returning" << std::endl;  
   return Py::Object();
 }
 
@@ -478,6 +623,7 @@ Transformation::set_offset(const Py::Tuple & args) {
 
 Py::Object
 Transformation::inverse_xy_tup(const Py::Tuple & args) {
+  _VERBOSE("Transformation::inverse_xy_tup");
   args.verify_length(1);
 
   Py::Tuple tup = args[0];
@@ -496,6 +642,7 @@ Transformation::inverse_xy_tup(const Py::Tuple & args) {
 
 Py::Object
 Transformation::xy_tup(const Py::Tuple & args) {
+  _VERBOSE("Transformation::xy_tup");
   args.verify_length(1);
 
   if (!_frozen) eval_scalars();
@@ -514,6 +661,7 @@ Transformation::xy_tup(const Py::Tuple & args) {
 
 Py::Object
 Transformation::seq_x_y(const Py::Tuple & args) {
+  _VERBOSE("Transformation::seq_x_y");
   args.verify_length(2);
   
   Py::SeqBase<Py::Object> x = args[0];
@@ -548,6 +696,7 @@ Transformation::seq_x_y(const Py::Tuple & args) {
 
 Py::Object
 Transformation::numerix_x_y(const Py::Tuple & args) {
+  _VERBOSE("Transformation::numerix_x_y");
   args.verify_length(2);
 
 
@@ -559,6 +708,7 @@ Transformation::numerix_x_y(const Py::Tuple & args) {
   if (x==NULL) 
     throw Py::TypeError("Transformation::numerix_x_y expected numerix array");
 
+  
   PyArrayObject *y = (PyArrayObject *) PyArray_ContiguousFromObject(yo.ptr(), PyArray_DOUBLE, 1, 1); 
   
   if (y==NULL) 
@@ -576,13 +726,21 @@ Transformation::numerix_x_y(const Py::Tuple & args) {
 
   int dimensions[1];
   dimensions[0] = Nx;
+
+  
   PyArrayObject *retx = (PyArrayObject *)PyArray_FromDims(1,dimensions,PyArray_DOUBLE);
-  if (retx==NULL)
+  if (retx==NULL) {
+    Py_XDECREF(x);
+    Py_XDECREF(y);
     throw Py::RuntimeError("Could not create return x array");
+  }
 
   PyArrayObject *rety = (PyArrayObject *)PyArray_FromDims(1,dimensions,PyArray_DOUBLE);
-  if (retx==NULL)
+  if (rety==NULL) {
+    Py_XDECREF(x);
+    Py_XDECREF(y);
     throw Py::RuntimeError("Could not create return x array");
+  }
 
   for (size_t i=0; i< Nx; ++i) {
 
@@ -593,15 +751,21 @@ Transformation::numerix_x_y(const Py::Tuple & args) {
     *(double *)(retx->data + i*retx->strides[0]) = xy.first;
     *(double *)(rety->data + i*rety->strides[0]) = xy.second;
   }
+  
+  Py_XDECREF(x);
+  Py_XDECREF(y);
 
   Py::Tuple ret(2);
   ret[0] = Py::Object((PyObject*)retx);
   ret[1] = Py::Object((PyObject*)rety);
+  Py_XDECREF(retx);
+  Py_XDECREF(rety);
   return ret;
 }
 
 Py::Object
 Transformation::seq_xy_tups(const Py::Tuple & args) {
+  _VERBOSE("Transformation::seq_xy_tups");
   args.verify_length(1);
   
   Py::SeqBase<Py::Object> xytups = args[0];
@@ -633,15 +797,36 @@ Transformation::seq_xy_tups(const Py::Tuple & args) {
 }
 
 
+SeparableTransformation::SeparableTransformation(Bbox *b1, Bbox *b2, Func *funcx, Func *funcy) : 
+    Transformation(), 
+    _b1(b1), _b2(b2), _funcx(funcx), _funcy(funcy)  {
+  _VERBOSE("SeparableTransformation::SeparableTransformation");
+  Py_INCREF(b1);
+  Py_INCREF(b2);
+  Py_INCREF(funcx);
+  Py_INCREF(funcy);
+  
+}
+
+
+SeparableTransformation::~SeparableTransformation() {
+  _VERBOSE("SeparableTransformation::~SeparableTransformation");
+  Py_DECREF(_b1);
+  Py_DECREF(_b2);
+  Py_DECREF(_funcx);
+  Py_DECREF(_funcy);
+}
 
 Py::Object
 SeparableTransformation::get_funcx(const Py::Tuple & args) {
+  _VERBOSE("SeparableTransformation::get_funcx");
   args.verify_length(0);
   return Py::Object(_funcx);
 }
 
 Py::Object
 SeparableTransformation::get_funcy(const Py::Tuple & args) {
+  _VERBOSE("SeparableTransformation::get_funcy");
   args.verify_length(0);
   return Py::Object(_funcy);
 }
@@ -649,21 +834,23 @@ SeparableTransformation::get_funcy(const Py::Tuple & args) {
 
 Py::Object
 SeparableTransformation::set_funcx(const Py::Tuple & args) {
+  _VERBOSE("SeparableTransformation::set_funcx");
   args.verify_length(1);
   if (!Func::check(args[0])) 
     throw Py::TypeError("set_funcx(func) expected a func instance");
   _funcx = static_cast<Func*>(args[0].ptr());
-
+  Py_INCREF(_funcx);
   return Py::Object();
 }
 
 Py::Object
 SeparableTransformation::set_funcy(const Py::Tuple & args) {
+  _VERBOSE("SeparableTransformation::set_funcy");
   args.verify_length(1);
   if (!Func::check(args[0])) 
     throw Py::TypeError("set_funcy(func) expected a func instance");
   _funcy = static_cast<Func*>(args[0].ptr());
-
+  Py_INCREF(_funcy);
   return Py::Object();
 }
 
@@ -671,12 +858,14 @@ SeparableTransformation::set_funcy(const Py::Tuple & args) {
 
 Py::Object
 SeparableTransformation::get_bbox1(const Py::Tuple & args) {
+  _VERBOSE("SeparableTransformation::get_bbox1");
   args.verify_length(0);
   return Py::Object(_b1);
 }
 
 Py::Object
 SeparableTransformation::get_bbox2(const Py::Tuple & args) {
+  _VERBOSE("SeparableTransformation::get_bbox2");
   args.verify_length(0);
   return Py::Object(_b2);
 }
@@ -684,27 +873,30 @@ SeparableTransformation::get_bbox2(const Py::Tuple & args) {
 
 Py::Object
 SeparableTransformation::set_bbox1(const Py::Tuple & args) {
+  _VERBOSE("SeparableTransformation::set_bbox1");
   args.verify_length(1);
   if (!Bbox::check(args[0])) 
     throw Py::TypeError("set_bbox1(func) expected a func instance");
   _b1 = static_cast<Bbox*>(args[0].ptr());
-
+  Py_INCREF(_b1);
   return Py::Object();
 }
 
 Py::Object
 SeparableTransformation::set_bbox2(const Py::Tuple & args) {
+  _VERBOSE("SeparableTransformation::set_bbox2");
   args.verify_length(1);
   if (!Bbox::check(args[0])) 
     throw Py::TypeError("set_bbox2(func) expected a func instance");
   _b2 = static_cast<Bbox*>(args[0].ptr());
-
+  Py_INCREF(_b2);
   return Py::Object();
 }
 
 
 std::pair<double, double>&
 SeparableTransformation::operator()(const double& x, const double& y) {
+  _VERBOSE("SeparableTransformation::operator");
 
   // calling function must first call eval_scalars
   double fx = _funcx->operator()(x);
@@ -726,6 +918,7 @@ SeparableTransformation::operator()(const double& x, const double& y) {
 
 std::pair<double, double> &
 SeparableTransformation::inverse_api(const double &x, const double &y) {
+  _VERBOSE("SeparableTransformation::inverse_api");
 
   // calling function must first call eval_scalars_inverse and
   // _transOffset->eval_scalars_inverse()
@@ -751,6 +944,7 @@ SeparableTransformation::inverse_api(const double &x, const double &y) {
 
 void 
 SeparableTransformation::eval_scalars(void) {
+  _VERBOSE("SeparableTransformation::eval_scalars");
   double xminIn  = _funcx->operator()( _b1->ll_api()->xval() );
   double xmaxIn  = _funcx->operator()( _b1->ur_api()->xval() );
   double yminIn  = _funcy->operator()( _b1->ll_api()->yval() );
@@ -803,8 +997,40 @@ SeparableTransformation::eval_scalars(void) {
 }
 
 
+Py::Object 
+SeparableTransformation::deepcopy(const Py::Tuple &args) {
+  _VERBOSE("SeparableTransformation::deepcopy");
+  args.verify_length(0);
+  return Py::asObject( new SeparableTransformation( static_cast<Bbox*>((_b1->_deepcopy()).ptr()),static_cast<Bbox*>((_b2->_deepcopy()).ptr()), _funcx,_funcy ));
+}
+
+Affine::Affine(LazyValue *a, LazyValue *b,  LazyValue *c, 
+	       LazyValue *d, LazyValue *tx, LazyValue *ty) : 
+  _a(a), _b(b), _c(c), _d(d), _tx(tx), _ty(ty) {
+  _VERBOSE("Affine::Affine");
+  Py_INCREF(a);
+  Py_INCREF(b);
+  Py_INCREF(c);
+  Py_INCREF(d);
+  Py_INCREF(tx);
+  Py_INCREF(ty);
+
+}
+
+Affine::~Affine() {
+  _VERBOSE("Affine::~Affine");
+  Py_DECREF(_a);
+  Py_DECREF(_b);
+  Py_DECREF(_c);
+  Py_DECREF(_d);
+  Py_DECREF(_tx);
+  Py_DECREF(_ty);
+}
+
+
 Py::Object 
 Affine::as_vec6(const Py::Tuple &args) {
+  _VERBOSE("Affine::as_vec6");
   //return the affine as length 6 list
   args.verify_length(0);
   Py::List ret(6);
@@ -822,6 +1048,7 @@ Affine::as_vec6(const Py::Tuple &args) {
 
 std::pair<double, double> & 
 Affine::operator()(const double &x, const double &y) {
+  _VERBOSE("Affine::operator");
   xy.first  = _aval*x + _cval*y + _txval;
   xy.second = _bval*x + _dval*y + _tyval;
 
@@ -836,6 +1063,7 @@ Affine::operator()(const double &x, const double &y) {
 
 std::pair<double, double> & 
 Affine::inverse_api(const double &x, const double &y) {
+  _VERBOSE("Affine::inverse_api");
 
   if (!_invertible)
     throw Py::RuntimeError("Transformation is not invertible");
@@ -859,6 +1087,7 @@ Affine::inverse_api(const double &x, const double &y) {
 
 void 
 Affine::eval_scalars(void) {
+  _VERBOSE("Affine::eval_scalars");
   _aval  = _a->val();
   _bval  = _b->val();
   _cval  = _c->val();
@@ -887,19 +1116,32 @@ Affine::eval_scalars(void) {
   }
 }
 
+Py::Object 
+Affine::deepcopy(const Py::Tuple &args) {
+  _VERBOSE("Affine::deepcopy");
+  args.verify_length(0);
+  eval_scalars();
+  return Py::asObject( new Affine( new Value(_aval),new Value(_bval), new Value(_cval),
+                                   new Value(_dval),new Value(_txval),new Value(_tyval) ));
+}
 
+ 
 
 /* ------------ module methods ------------- */
-Py::Object _transforms_module::new_value (const Py::Tuple &args)
+Py::Object 
+_transforms_module::new_value (const Py::Tuple &args)
 {
+  _VERBOSE("_transforms_module::new_value ");
   args.verify_length(1);
   double val = Py::Float(args[0]);
-  return Py::Object( new Value(val) );
+  return Py::asObject( new Value(val) );
 }   
 
 
-Py::Object _transforms_module::new_point (const Py::Tuple &args)
+Py::Object 
+_transforms_module::new_point (const Py::Tuple &args)
 {
+  _VERBOSE("_transforms_module::new_point ");
   args.verify_length(2);
   
   LazyValue *x, *y;
@@ -917,13 +1159,16 @@ Py::Object _transforms_module::new_point (const Py::Tuple &args)
     y = static_cast<Value*>(args[1].ptr());
   else 
     throw Py::TypeError("Can only create points from LazyValues");
-  return Py::Object(new Point(x, y));
+
+  return Py::asObject(new Point(x, y));
   
 }    
 
 
-Py::Object _transforms_module::new_interval (const Py::Tuple &args)
+Py::Object 
+_transforms_module::new_interval (const Py::Tuple &args)
 {
+  _VERBOSE("_transforms_module::new_interval ");
   
   args.verify_length(2);
   
@@ -935,11 +1180,13 @@ Py::Object _transforms_module::new_interval (const Py::Tuple &args)
   
   LazyValue* v1 = static_cast<LazyValue*>(args[0].ptr());
   LazyValue* v2 = static_cast<LazyValue*>(args[1].ptr());
-  return Py::Object(new Interval(v1, v2) );  
+  return Py::asObject(new Interval(v1, v2) );  
 }
 
-Py::Object _transforms_module::new_bbox (const Py::Tuple &args)
+Py::Object 
+_transforms_module::new_bbox (const Py::Tuple &args)
 {
+  _VERBOSE("_transforms_module::new_bbox ");
   
   args.verify_length(2);
   
@@ -950,10 +1197,12 @@ Py::Object _transforms_module::new_bbox (const Py::Tuple &args)
   
   Point* ll = static_cast<Point*>(args[0].ptr());
   Point* ur = static_cast<Point*>(args[1].ptr());
-  return Py::Object(new Bbox(ll, ur) );  
+  return Py::asObject(new Bbox(ll, ur) );  
 }
 
-Py::Object _transforms_module::new_affine (const Py::Tuple &args) {
+Py::Object 
+_transforms_module::new_affine (const Py::Tuple &args) {
+  _VERBOSE("_transforms_module::new_affine ");
 
   args.verify_length(6);
   
@@ -970,26 +1219,25 @@ Py::Object _transforms_module::new_affine (const Py::Tuple &args) {
   LazyValue* d  = static_cast<LazyValue*>(args[3].ptr()); 
   LazyValue* tx = static_cast<LazyValue*>(args[4].ptr());
   LazyValue* ty = static_cast<LazyValue*>(args[5].ptr()); 
-  return Py::Object(new Affine(a, b, c, d, tx, ty));
+  return Py::asObject(new Affine(a, b, c, d, tx, ty));
   
 }    
 
+ 
 
-Py::Object _transforms_module::new_identity (const Py::Tuple &args)
+Py::Object 
+_transforms_module::new_func (const Py::Tuple &args)
 {
-  args.verify_length(0);
-  return Py::Object( new Identity() );
+  _VERBOSE("_transforms_module::new_func ");
+  args.verify_length(1);
+  int typecode = Py::Int(args[0]);
+  return Py::asObject(new Func(typecode));
 }   
 
-Py::Object _transforms_module::new_log (const Py::Tuple &args)
-{
-  args.verify_length(0);
-  return Py::Object( new Log() );  
-
-}   
-
-Py::Object _transforms_module::new_funcxy (const Py::Tuple &args)
+Py::Object 
+_transforms_module::new_funcxy (const Py::Tuple &args)
 {
+  _VERBOSE("_transforms_module::new_funcxy ");
   args.verify_length(2);
   if (!Func::check(args[0]))
     throw Py::TypeError("FuncXY(funcx, funcy) expected a Func instance for funcx)");
@@ -998,17 +1246,22 @@ Py::Object _transforms_module::new_funcxy (const Py::Tuple &args)
   Func* funcx  = static_cast<Func*>(args[0].ptr());
   Func* funcy  = static_cast<Func*>(args[1].ptr());
 
-  return Py::Object(new FuncXY(funcx, funcy));
+  return Py::asObject(new FuncXY(funcx, funcy));
 }   
 
-Py::Object _transforms_module::new_polarxy (const Py::Tuple &args)
+
+Py::Object 
+_transforms_module::new_polarxy (const Py::Tuple &args)
 {
+  _VERBOSE("_transforms_module::new_polarxy ");
   args.verify_length(0);
-  return Py::Object( new PolarXY() );
+  return Py::asObject( new PolarXY() );
 }   
 
-Py::Object _transforms_module::new_separable_transformation (const Py::Tuple &args)
+Py::Object 
+_transforms_module::new_separable_transformation (const Py::Tuple &args)
 {
+  _VERBOSE("_transforms_module::new_separable_transformation ");
   args.verify_length(4);
   if (!Bbox::check(args[0]))
     throw Py::TypeError("SeparableTransform(box1, box2, funcx, funcy) expected a Bbox for box1");
@@ -1025,11 +1278,13 @@ Py::Object _transforms_module::new_separable_transformation (const Py::Tuple &ar
   Func* funcx  = static_cast<Func*>(args[2].ptr()); 
   Func* funcy  = static_cast<Func*>(args[3].ptr()); 
   
-  return Py::Object( new SeparableTransformation(box1, box2, funcx, funcy) );
-}    
+  return Py::asObject( new SeparableTransformation(box1, box2, funcx, funcy) );
+}     
 
-void LazyValue::init_type()
+void 
+LazyValue::init_type()
 {
+  _VERBOSE("LazyValue::init_type");
   behaviors().name("LazyValue");
   behaviors().doc("A lazy evaluation float, with arithmetic");
   behaviors().supportNumberType();
@@ -1038,31 +1293,40 @@ void LazyValue::init_type()
   add_varargs_method("set",    &LazyValue::set,     "set(val)\n");
 }
 
-void Value::init_type()
+void 
+Value::init_type()
 {
+  _VERBOSE("Value::init_type");
   behaviors().name("Value");
   behaviors().doc("A mutable float");
   behaviors().supportNumberType();}
 
 
-void BinOp::init_type()
+void 
+BinOp::init_type()
 {
+  _VERBOSE("BinOp::init_type");
   behaviors().name("BinOp");
   behaviors().doc("A binary operation on lazy values");
   behaviors().supportNumberType();
 } 
 
-void Point::init_type()
+void 
+Point::init_type()
 {
+  _VERBOSE("Point::init_type");
   behaviors().name("Point");
   behaviors().doc("A point x, y");
-  
+    
   add_varargs_method("x",    &Point::x,     "x()\n");
   add_varargs_method("y",    &Point::y,     "y()\n");
+  add_varargs_method("reference_count", &Point::reference_count);
 }
 
-void Interval::init_type()
+void 
+Interval::init_type()
 {
+  _VERBOSE("Interval::init_type");
   behaviors().name("Interval");
   behaviors().doc("A 1D interval");
   
@@ -1077,8 +1341,10 @@ void Interval::init_type()
   add_varargs_method("val2", &Interval::val2, "val2()\n");
 }
 
-void Bbox::init_type()
+void 
+Bbox::init_type()
 {
+  _VERBOSE("Bbox::init_type");
   behaviors().name("Bbox");
   behaviors().doc("A 2D bounding box");
   
@@ -1106,28 +1372,23 @@ void Bbox::init_type()
 
 
 
-void Func::init_type()
+void 
+Func::init_type()
 {
+  _VERBOSE("Func::init_type");
   behaviors().name("Func");
   behaviors().doc("Map double -> double");
+  behaviors().supportRepr();  
   add_varargs_method("map",     &Func::map, "map(x)\n");
   add_varargs_method("inverse", &Func::inverse, "inverse(y)\n");
+  add_varargs_method("set_type", &Func::set_type, "set_type(TYPE)\n");
+  add_varargs_method("get_type", &Func::get_type, "get_type()\n");
 } 
 
-void Identity::init_type()
-{
-  behaviors().name("Identity");
-  behaviors().doc("Map x-> x");
-}
-
-void Log::init_type()
-{
-  behaviors().name("Log");
-  behaviors().doc("Map x-> log10(x)");
-}
-
-void FuncXY::init_type()
+void 
+FuncXY::init_type()
 {
+  _VERBOSE("FuncXY::init_type");
   behaviors().name("FuncXY");
   behaviors().doc("Map double,double -> funcx(double), funcy(double)");
   add_varargs_method("map", &FuncXY::map, "map(x,y)\n");
@@ -1138,15 +1399,19 @@ void FuncXY::init_type()
   add_varargs_method("get_funcy", &FuncXY::get_funcy, "get_funcy(func)\n");
 } 
  
-void PolarXY::init_type()
+void 
+PolarXY::init_type()
 {
+  _VERBOSE("PolarXY::init_type");
   behaviors().name("PolarXY");
   behaviors().doc("map r, theta -> r*cos(theta), r*sin(theta)");
 } 
 
 
-void Transformation::init_type()
+void 
+Transformation::init_type()
 {
+  _VERBOSE("Transformation::init_type");
   behaviors().name("Transformation");
   behaviors().doc("Transformation base class");
 
@@ -1176,18 +1441,23 @@ void Transformation::init_type()
   add_varargs_method("set_offset",   &Transformation::set_offset,  "set_offset(xy, trans)\n");
 
   add_varargs_method("as_vec6", &Transformation::as_vec6, "as_vec6(): return the affine as length 6 list of Values\n");
+  add_varargs_method("deepcopy", &Transformation::deepcopy, "deepcopy()\n");
 
 }
 
-void Affine::init_type()
+void 
+Affine::init_type()
 {
+  _VERBOSE("Affine::init_type");
   behaviors().name("Affine");
   behaviors().doc("A mutable float");
 }
 
 
-void SeparableTransformation::init_type()
+void 
+SeparableTransformation::init_type()
 {
+  _VERBOSE("SeparableTransformation::init_type");
   behaviors().name("SeparableTransformation");
   behaviors().doc("SeparableTransformation(box1, box2, funcx, funcy); x and y transformations are independet");
 
@@ -1201,11 +1471,14 @@ extern "C"
 DL_EXPORT(void)
   init_transforms(void)
 {
-  //suppress unused warning by creating in two lines
-  static _transforms_module* _transforms = NULL; 
-  _transforms = new _transforms_module;
+  static _transforms_module* _transforms = new _transforms_module;
 
   import_array();  
+
+
+  Py::Dict d = _transforms->moduleDictionary();
+  d["LOG10"] = Py::Int((int)Func::LOG10);
+  d["IDENTITY"] = Py::Int((int)Func::IDENTITY);
 };
 
 
